/************************************* REXX **********************************
   Alarm.Rex

   Purpose: 

      This script demonstrates the interface to APIs for AlarmManage.  It may 
      be executed as a stand-alone program from the Start Script dialog window 
      of the AF/REMOTE Client or may be used as a callable subroutine from 
      another REXX script.

   Alarm.rex requires four parmeters as documented for the AFR_ALARM() API:  

        Host       TCP/IP hostname or address of AlarmManage server machine
        id         AlarmManage 'Parameterid' value
        desc       AlarmManage 'Description' value
        severity   Optional.  "ALARM", "WARNING", or "OK". Default if value not 
                   provided is "ALARM".
        
   Alarm.rex returns the code returned by AFR_ALARM:  

        Returns 'N' if provided parameter is invalid.

   Modification history:
      December 12, 2002 - Jim Collins - Initial implementation 
      January 14, 2003  - Jim Collins - made parameter acquisition more robust
      February 14, 2003 - JCOLL - Removed 'under construction' comment.
                                  Change label for RC 0 from OK to Success.

 *******************************************************************************/


/* set some error detection signals */
signal on syntax
signal on halt
signal on novalue

/* Suggest initalized variables be done here */
Debug = 1
Host = ''
Id   = ''
Desc = ''
Sev = 'ALARM'
Debug = 1
SevRet.   = 'Unknown return code'
SevRet.0  = 'Success'
SevRet.1  = 'Failed to load AFR support DLL: RpAlarm.DLL'
SevRet.2  = 'Failed to locate RpAlarm procedure'
SevRet.3  = 'Newer RpAlarm version needed.'
SevRet.5  = 'Shutdown in progress'
Sevret.10 = 'Windows socket error'
Sevret.12 = 'Cannot open socket'
SevRet.13 = 'No response from host'
SevRet.14 = 'Connect error'
SevRet.15 = 'Socket send error'
SevRet.16 = 'Socket read error'


/* retrieve the name of the source program */
parse source name
program_name = substr(name,lastpos('\',name)+1)':'
say 'Starting' program_name
call afr_log 'Starting' program_name

/* retrieve the current version and level */
parse version ver
say 'Version =' ver
program_version = 'January 15, 2003'
say 'Program version:' program_version
call afr_log 'Program version:' program_version

/* Main line program goes here */
Sevin = ''
Args = arg()
if Args = 1 then
    do
    parse arg Temp
    if pos(',', Temp) > 0 then
        do
        parse arg Host ',' Id ',' Desc ',' Sevin
        end
    else
        do
        parse arg Host Id Desc
        Sevin = 'Alarm'
        end
    end
else if Args < 3 then
    do
    say program_name 'Insuffiencient parameters provided'
    return 'N'
    end
else 
    do
    Host = arg(1)
    ID =   arg(2)
    Desc = arg(3)
    end
    
if Args = 4 then
   Sevin = arg(4)
/* strip off leading and trailing spaces */
Host =  strip(HOST)
ID =    strip(ID)
Desc =  strip(Desc)
Sevin = strip(Sevin)

if Debug then
    say program_name 'Host:'Host 'ID:'ID 'Desc:'Desc 'Sev:'Sevin 
if Host = '' then
    do
    call LogSay 'First Parameter not supplied.'
    return 'N'
    end
if ID = '' then
    do
    call LogSay 'Second Parameter not supplied.'
    return 'N'
    end
if Desc = '' then
    do
    call Logsay 'Third Parameter not supplied.'
    return 'N'
    end
if Sevin <> '' then
    do
    parse upper var Sevin Sev
    if Sev <> 'ALARM' & Sev <> 'WARNING' & Sev <> 'OK' then
        do
        call LogSay 'Invalid fourth parameter:' Sev
        return 'N'
        end
    end
    
ret = AFR_ALARM(Host, Id, Desc, Sev)

reta = strip(abs(ret) )
say 'ret:'ret
call LogSay 'AFR_ALARM('Host','Id','Desc','Sev') returned:' ret SevRet.reta
return ret

/* pretty good idea to return a value to the caller */
return ' '

/* branch to this point if program halted from AF/REMOTE Client Scripts window */
halt:
   result = program_name 'Terminated by Operator'
   ret = 'AFR_ERROR'(result,0)
   say result
/* whatever else needs to be done to clean up after a shutdown */
   return 'N'

/* branch to this point if syntax error encountered or function call in error */
syntax:
   result = program_name 'Terminated for Syntax, Line' SIGL
   ret = 'AFR_ERROR'(result,1)
   say result
/* whatever else needs to be done to clean up after a syntax shutdown */
   return 'N'

/* branch to this point if novalue error encountered */
novalue:
   result = program_name 'Terminated for NoValue, Line' SIGL
   ret = 'AFR_ERROR'(result,1)
   say result
/* whatever else needs to be done to clean up after a No Value shutdown */
   return 'N'


/* subroutines, functions, procedures, ??? */
LogSay:
   text = arg(1)
   say time() program_name text
   x = afr_log(text)
   return 0
