/************************************* REXX **********************************
   Script:
      AlarmWtor.Rex

   Purpose: 

      This sample script demonstrates the interface to APIs for AlarmManage. 
      It may be executed as a stand-alone program from the Start Script dialog
      window of the AF/REMOTE Client or may be used as a callable subroutine
      from another REXX script.

      This sample script uses the same parameter string normally passed to the
      sample script, BEEPCALL.REX, when it is used with WTORSCAN.REX. However,
      it supports an AlarmManage interface only.  
   
      AlarmWtor "<TYPE><PHONE> <PID> <MSG>"

            where TYPE  =  a | A for AlarmManage beep

                  PHONE =  Phone number including PABX access symbols
                           Note: Currently ignored in AFR_ALARM call. Phone
                           number must be defined in AlarmManage

                  PID   =  Pager ID assigned by the paging service (one only)
                           Concatenated to AlarmPrefix and passed as
                           AlarmManage ParameterID
                   
                  MSG   =  Optional message text to be sent to paging service
                           Passed as AlarmManage Description

      SUCCESS = Character string indicating result of attempted AlarmManage call
   
      Example: 'a9,8491050 2015382 This is a test of REXX-driven paging via AlarmManage'
               
      Notes:   The user of this script must make the following modification:
                1. Change the variable AlarmPrefix to the id contained in AlarmManager 
                   Policy expression.  The PID field of the WTOR is then appended to this.
                2. Change the variable AlarmHost to the name of the AlarmManager IP address or
                   the alias thereof. The alias may be case sensitive.  It was for the 
                   developer.
                
                
   Modification history:
      December 12, 2002 - Jim Collins - Initial implementation 
      January 15, 2003  - Jim Collins - Define transformation pid to Parameter ID.
      January 24, 2003  - Jim Collins - Final changes
      February 14, 2003 - JCOLL - Removed 'under construction' comment.
                                  Change label for RC 0 from OK to Success.
 *********************************************************************************/


/* set some error detection signals */
signal on syntax
signal on halt
signal on novalue

/* Suggest initalized variables be done here */
version     = 'January 15, 2003'
Debug       = 1
/* the following two variables are to determined by the user */
AlarmHost   = 'IPAddrss'   /* this must be change to the IP address of AlarmManager (Centauri) */
AlarmPrefix = 'ParmeterID' /* The must be changed to the Parameter ID defined for customer installation   */
SevRet.   = 'Unknown return code'
SevRet.0  = 'Success'
SevRet.1  = 'Failed to load AFR support DLL:RpAlarm.DLL'
SevRet.2  = 'Failed to locate RpAlarm procedure'
SevRet.3  = 'Newer RpAlarm version needed.'
SevRet.5  = 'Shutdown in progress'
Sevret.10 = 'Windows socket error'
Sevret.12 = 'Cannot open socket'
SevRet.13 = 'No response from host'
SevRet.14 = 'Connect error'
SevRet.15 = 'Socket send error'
SevRet.16 = 'Socket read error'


/* retrieve the name of the source program */
parse source name
program_name = substr(name,lastpos('\',name)+1)':'
say 'Starting' program_name
call afr_log "Starting" program_name

/* retrieve the current version and level */
parse version ver
say 'Version =' ver
call afr_log 'Version =' Ver

/* Main line program goes here */
parse arg Phone Pid Msg
Type  = substr(Phone,1,1)
Phone = substr(Phone,2)
if debug then
    do
    say 'Type:' Type 'Phone:' Phone 'PID:' Pid 'Msg:' Msg
    end
/* Transform data extracted from incoming parameter into that required by AFR_ALARM().*/
Host = AlarmHost
Desc = Msg
ID   = AlarmPrefix||strip(pid)
Sev  = 'ALARM'

if Debug then
    do
    s = program_name "Host:"Host "ID:"Id "Desc:"Desc "Sev:" Sev 
    say Date() Time() s
    call afr_log s
    end
Ret = AFR_ALARM(Host, Id, Desc, Sev)
Reta = abs(Ret)

if Debug then
   do
   s = program_name "AFR_ALARM("Host","Id","Desc","Sev") returned:" Ret SevRet.Reta
   say Date() Time() s
   call afr_log s
   end
call LogSay 'AFR_ALARM('Host','Id','Desc','Sev') returned:' ret SevRet.reta
return ret

/* pretty good idea to return a value to the caller */
return ' '

/* branch to this point if program halted from AF/REMOTE Client Scripts window */
halt:
   result = program_name 'Terminated by Operator'
   ret = 'AFR_ERROR'(result,0)
   say result
/* whatever else needs to be done to clean up after a shutdown */
   return 'N'

/* branch to this point if syntax error encountered or function call in error */
syntax:
   result = program_name 'Terminated for Syntax, Line' SIGL
   ret = 'AFR_ERROR'(result,1)
   say result
/* whatever else needs to be done to clean up after a syntax shutdown */
   return 'N'

/* branch to this point if novalue error encountered */
novalue:
   result = program_name 'Terminated for NoValue, Line' SIGL
   ret = 'AFR_ERROR'(result,1)
   say result
/* whatever else needs to be done to clean up after a No Value shutdown */
   return 'N'


/* subroutines, functions, procedures, ??? */
LogSay:
   text = arg(1)
   say time() program_name text
   x = afr_log(text)
   return 0
