/*************************** REXX *********************************************
   SCRIPT: BEEPCALL.REX
   12-December-2003

   PURPOSE: Initiates a paging beep.  Conditionally retries pages not responding
            with a 'Y' return code.

   SYNOPSIS
      SUCCESS = call BEEPCALL.REX "<TYPE><PHONE> <PID> <MSG>"
         where TYPE = m | M | t | T  for Modem or Tone beep
               PHONE = phone number including PABX access symbols
               PID = Pager ID assigned by the paging service
                   = two or more PIDs in the form <PID>,<PID>
                   = file name of list of PIDs in the form 0<filename>.
                     If <filename> does not explicitly name an extent, '.PID' is assumed.
                     If <filename> contains an explicit extent, it will not be changed.
                     The PID file shall contain one PID per line followed by a space and
                      optional comment.  Do not use a tab character to create the space.
                     The PID file is assumed to be in the AF/Remote Server BIN directory.
               MSG = optional message text to be sent to paging service
               SUCCESS = character string indicating result of attempted page.

      Example: 'm9,8491050 2015382 This is a test of REX driven beeper paging.'
               't9,2015382 0 21320714002724'
               'm9,8491050 2015382,2021382 Multiple PIDS.'
               'm9,8491050 012345 Multiple PIDS contained in a file named 12345.PID.'
               'm9,8491050 012345.txt Multiple PIDS contained in a file named 12345.TXT.'
               '1200,i,e,1/^\m9,8491050 2015382 This adds serial port settings.'
               '1200,i,e,1/^\m9,8491050 2015382,2021382 Multiple PIDS.'
   CALLS:      beeptry

   RETURNS: 'Y' if beeper call is perceived to be successful
            'N' if beeper call is perceived to be unsuccessful
                  or retry count exceeds retries specified

   Modification History:
     Oct 27, 1994 - JMC
       1. Provide capablity to run modem outdial/disconnect using high-levl
          AFR_MODEMCALL()/AFR_MODEMDISCONNECT() functions OR using the
          lower level AFR_COMPUTSTR(), AFR_COMGETSTR(), AFR_COMCTRL(), and
          associated function calls.  Selected by setting the variable
          HighLevel to 1 or 0 respectively, default 0.

       2. Implemented capability to transmit the same message to multiple
          PID on a single connect to the paging computer.

       3. Updated the modem paging portion of the program (IXO) to more
          closely comform to the Telocator Alphanumeric Protocol (TAP), the
          formalisation of the IXO beeper paging protocol.

     Dec 16, 1994 - JMC
       1. Added logic to terminate protocol on loss of carrier between modems,
          returning a 'N' as the success indicator.

       2. Added logic and state tables initiate beeper pages via SKYTEL Remote
          Terminal interface.  This is a non-IXO/TAP protocol.  This protocol
          can be run from a communcication program such as Procomm or Qmodem.

       3. Added a conditional statement to force a null message string, which
          is legal to a single space character.  Some ixo paging services will
          not accept a zero length message string.

     May 7, 1998 - JMC
       The following modifications were added to allow paging of multiple
       persons on multiple paging services from a single event.  See the program
       NOTIFY.REX for details.  This program may also be driven by WTORSCAN.REX
       as in the past, and via the manual paging pulldown in AF/REMOTE.  The
       basis of this program (it's parent) is BEEPCALL.REX.

       The following details the changes made.

       1. Revised logic flow to sense serial port parameters in the main body of
          the program.  Provided for an optional prefix to allow specification
          of serial port parameters by the callers instead of interegating the
          'BP_' settings in afr_getenv().  The format of the prefix string is:
          <rate>,<bits>,<parity>,<stopbits>/^\
            where
                rate is one of following 300, 1200, 2400, 9600, 19200, or 38400.
                bits is 7 or 8
                parity is E (even)
                          O (odd)
                          N (none)
                stopbits  1 or 2
                /^\ is the delimiter.  If this string is not found, then the
                prefix is considered to not exist. In that case the value
                retreived using AFR_GETENV() with environment values of
                BP_RATE, BP_PARITY, BP_BITS, BP_STOP are used to set the serial port
                characteristics.

       2. The main body will attempt to use the pid number as a filename for
          expansion of the list of PIDs to be transmitted.  If such a file is
          found the PID field is modified to the form <PID>,<PID>,....,<PID>
          (comma separated PIDs).  If the file is not found the PID field is
          used as received.

       3. The main body will break out the PID(s) to a stem variable which is
          used later for creating the portions of the protocol where PIDs are
          sent.  Both the TAP (also known as IXO) and the SKYPAGE protocol
          support messaging to multiple receipients during a paging call
          connection.
    December 1, 2000
        Made the outdial process more robust.  Outcall not does a monitored
        initialization of the mode in preparation for use.  In the constants
        section of the main body of the program is a stem variable which
        contains the initialization strings.  Currently this version contains
        initialization for the Hayes series of modem.  Also provided are the
        commented out stem variable for the US Robotics Sportster modem fixed
        at 1200 baud.

        Also added a function LogSay to say a string and then log it to the
        RPServer log.  All say statement are revised to call AfrSay.  In the
        RpServer log, you can find all of the logged data from the AFR_LOG()
        API call by searching for the string AfrLog.

    April 30, 2003
        Correct misleading commentary regarding PID file requirements.  Made
        processing of PID files more robust.

    December 12, 2003
        Minor changes to logging

    April 28, 2006
        Minor change to fix pid=='0' logic

 ******************************************************************************/

/* retrieve the name of the source program */
parse source name
program_name = substr(name,lastpos('\',name)+1)':'
program_path = left(name,lastPos('\',name))':'

/* set some error detection signals */
signal on halt
signal on syntax

/* set up constants and variables */
version     = 'December 12, 2003'
handle      = 0             /* handle returned by AFR_COMOPEN() */
retries     = 100           /* number of times to retry busy phone */
RetryDelay  = 30000         /* time to delay between connect attempts */
HighLevel   = 0             /* selects high or level outdial logic */
ShowSends   = 1             /* enables display of strings sent, diagnostic
                               aid */
ShowRcvs    = 1             /* enables display of string received, diagnostic
                               aid */
ShowDateTime= 1             /* enables display of date/time stamp at start and
                               end of IXO sequence */
Broadcast   = 0             /* enables broadcast of paging activity to console
                               sessions */
LogActivity = 1             /* enables logging of computer to computer dialog
                               diagnostic aid */
LogConnect  = 1             /* enables logging of modem connection activities */
Color       = 0             /* color of broadcast message, set by program */
PortWaitTime= 300           /* seconds to wait for port to become available */
PortDelay   =  5            /* seconds to wait before asking for port again */

/* the following are generic initializations */
Initstr.  = ''
Initstr.0 = 4
Initstr.1 = 'AT&F'
Initstr.2 = 'ATE1V1'
Initstr.3 = 'AT&D2&C1M0'
Initstr.4 = 'AT&W'
/* the following are initializations for Hay Optima
Initstr.  = ''
Initstr.0 = 5
Initstr.1 = 'AT&F'
Initstr.2 = 'ATE1V1'
Initstr.3 = 'AT&D2&C1M0'
Initstr.4 = 'AT&K0&Q0S48=128Q0'
Initstr.5 = 'AT&W'
Initstr.6 = 'ATI4'
*/
/* the following are initializations for USR Sporster
Initstr.  = ''
Initstr.0 = 6
Initstr.1 = 'AT&F'
Initstr.2 = 'ATE1V1'
Initstr.3 = 'AT&D2&C1M0'
Initstr.4 = 'AT&B2&N3'          /* fixed buad rate, 1200 buad */
Initstr.5 = 'AT&K0&Q0S48=128Q0' /* &K0 - Disable compression*/
                                /* Q0   - Enable message display */
                                /* &Q0 and S48 - no description available */
Initstr.6 = 'AT&W'
*/
RespStr.0 = 7
RespStr.1 = 'OK'
RespStr.2 = 'CONNECT'
RespStr.3 = 'ERROR'
RespStr.4 = 'NO DIALTONE'
RespStr.5 = 'NO ANSWER'
RespStr.6 = 'NO CARRIER'
RespStr.7 = 'BUSY'

/* retreive beeper configuration data from the configuation editor */
rate = 'AFR_GETENV'("BP_RATE")
bits = 'AFR_GETENV'("BP_BITS")
stop = 'AFR_GETENV'("BP_STOP")
parity = 'AFR_GETENV'("BP_PARITY")

/* establish connection with paging service */
/* constants/variables available for procedure expose */
Constants   = "program_name handle retries RetryDelay HighLevel ShowSends",
              "ShowRcvs ShowDateTime Broadcast LogActivity Color program_path",
              "InitStr. RespStr. save_ansi LogConnect"

/* get beeper arguments */
parse arg phone pid msg

call LogSay Version 'Input:' phone pid msg

/* Test for a filename.  This is a numeric filename starting with zero "0" */
/*  but a pid of exactly '0' means no pid */
if length(pid) > 1 & substr(pid,1,1) = '0' then do
    /* strip off leading zero */
    PidFile = substr(pid,2)
    /* test for presence of trailing file extent, usually ".PID" */
    if pos('.', PidFile) <= 0 then
        PidFile = PidFile'.PID'

    call LogSay 'PID File used:' PidFile

    /* attempt to open the PID file */
    OpenResult = stream(PidFile,'C','open read')
    if OpenResult = 'READY:' then
        do
        pid = ''
        /* then read in each line. Extract the PID from each line, create
           string containing the PIDs separated by commas*/
        do while lines(PidFile) > 0
            /* read the next line*/
            temp = linein(PidFile)
            /* the first field is the PID */
            parse var temp t .
            /* in case somebody uses the TAB key */
            pos = pos(x2c(09), t)
            if pos > 0 then
                /* then lop off the remaining string */
                t = substr(t,1,pos-1)
            /* suppress leading or trailing spaces */
            t = strip(t)
            /* if this is the first PID */
            if pid = '' then
                pid = t
            /* else concatenate as a comma separated string */
            else
                pid = pid','t
            end
        /* then close the file */
        n = stream(PidFile,'C', 'close')
        end
        else
            do
            call LogSay 'PID File open error:' OpenResult
            return 'N'
        end
    end

/* log PID we are using */
call LogSay 'pid:' pid

/* save ansi status; turn off; echo input; restore */
save_ansi = afr_ansi()

if pos('/^\', phone) > 0 then
    do
    parse var phone PortParams '/^\' phone
    /* PortParams should be in the form rrrr,b,p,s where
        rrrr = baud rate
           b = bits/word
           p = parity
           s = stop
    */
    parse var PortParams rate ',' bits ',' parity ',' stop
    end

rc = afr_ansi(0)
if ShowDateTime then
    Say Program_name date() time()

CurDelay = 0
do while Handle = 0
    handle = 'AFR_COMOPEN'(0)
    if Handle = 0 then
       do
       CurDelay = CurDelay + PortDelay
       if CurDelay > PortWaitTime then
          do
          call LogSay 'Unable to acquired port after' PortWaitTime 'seconds'
          signal halt
          end
       n = afr_suspend(PortDelay * 1000)
       end
    end

/* set up comm port parameters per beeper configuration data */
rc = 'AFR_COMSETBAUD'(handle,rate)
rc = 'AFR_COMSETLINEC'(handle,bits,stop,parity);
call LogSay 'Modem Parameters, Rate:'rate 'Bits:'bits 'Parity:'parity 'Stop:'stop

/* try to send the page retries times */
do try = 1 to retries

   /* go try to send a beeper page */
   call LogSay 'calling beeptry:' phone '/' pid '/' msg
   ret = beeptry(phone pid msg)

   /* evaluate this attempt */
   success = 'UNSUCCESSFUL' ret
   if ret = 'Y' then
      success = 'SUCCESSFUL'
   if ret = 'N' then
      success = 'UNSUCCESSFUL'
   if ret = 'R' then
      rc = 'AFR_SUSPEND'(RetryDelay)
   else
      /* pass back result of beeper paging attempt */
      do
         Color = 1
         leave
      end
end

result = phone msg ret
call LogSay result
rc = afr_ansi(save_ansi)
n = afr_comclose(handle)
if ShowDateTime then
    Say program_name date() time()

result = program_name phone msg ret
if Broadcast then
    rc = 'AFR_ERROR'(result,Color)
else
    if LogActivity then
        rc = AFR_LOG(result)

return ret

/* branch to this point if program terminated by REXX management pulldown */
halt:
   result = program_name 'Terminated by Operator'
   Say result
   if Broadcast then
       rc = 'AFR_ERROR'(result,1)
   else
       if LogActivity then
           rc = AFR_LOG(result)

   if HighLevel then
      rc = 'AFR_MODEMDISCONNECT'(handle)
   else
      rc = 'AFR_COMCTRL'(handle,0,1)

   ret = 'AFR_COMCLOSE'(handle)
   rc = afr_ansi(save_ansi)
/* whatever else needs to be done to clean up after a shutdown */
   exit 'Y'

/* branch to this point if novalue error encountered */
novalue:
   result = program_name 'Terminated for Novalue, Line' SIGL
   Say result
   if Broadcast then
       rc = 'AFR_ERROR'(result,0)
   else
       if LogActivity then
           rc = AFR_LOG(result)

   if HighLevel then
      rc = 'AFR_MODEMDISCONNECT'(handle)
   else
      rc = 'AFR_COMCTRL'(handle,0,1)

   ret = 'AFR_COMCLOSE'(handle)
   rc = afr_ansi(save_ansi)
/* whatever else needs to be down to clean up after a No Value shutdown */
   exit 'N'

/* branch to this point if syntax error encountered or function call in error */
syntax:
   result = program_name 'Terminated for Syntax, Line' SIGL 'Code' RC
   Say result
   if Broadcast then
       rc = 'AFR_ERROR'(result,0)
   else
       if LogActivity then
           rc = AFR_LOG(result)

   if HighLevel then
      rc = 'AFR_MODEMDISCONNECT'(handle)
   else
      rc = 'AFR_COMCTRL'(handle,0,1)

   ret = 'AFR_COMCLOSE'(handle)
   rc = afr_ansi(save_ansi)
/* whatever else needs to be down to clean up after a syntax shutdown */
   exit 'Y'

/*************************** REXX *********************************************
   PROCEDURE: BEEPTRY  expose (Constants)

   PURPOSE: Initiates a paging beep.

   SYNOPSIS
      SUCCESS = beeptry "<TYPE><PHONE> <PID> <MSG>"
         where TYPE = m | M | t | T  for Modem or Tone beep
               PHONE = phone number including PABX access symbols
               PID = Pager ID assigned by the paging service
                   = two or more PIDs in the form <PID>,<PID>
                   = file name of list of PIDs in the form #filename, with the
                     file ext .PID.  File shall contain one PID per line
                     followed by a space or new line character
               MSG = optional message text to be sent to paging service
               SUCCESS = character string indicating result of attempted page.

      CALLS:   MSTATNMS to evaluate return code returned by call to
                  AFR_MODEMCALL.
               IXO.REX to execute the IOX protocol used by several of the
               U.S. based Alphanumeric beeper paging services.

      RETURNS: 'Y' if the beeper call appeared to be successful,
               'N' if the beeper call appears to be unsuccessful
               'R' if the beeper call attempt encountered a busy signal
                     from the called telephone
               otherwise returns reason modem could not complete call

      Example: 'm9,8491050 2015382 This is a test of REX driver beeper paging.'
               't9,2015382 0 21320714002724'


 ******************************************************************************/

beeptry:procedure expose (Constants)

parse arg phone msg

error_flag = 0
fail_lit = 'FAILED'
tinit = 'ATE1V1X4'
tinit = 'AT'
/* tinit1 = 'AT&D2&C1' */
tinit1 = 'AT'
if length(phone) = 0 then
   do
      return 'Invalid Telephone Number'
   end

/* separate type from phone number field of WTOR string */
type = left(phone,1)
phone = substr(phone,2,length(phone) - 1)

/* set of time to wait for modem to connect */
DIAL_TO = '60000'

/* determine type of page */
if type = 'm' | type = 'M' then  /* its a modem (IXO) type of page */
   do
      if HighLevel then  /* use high-level AFR_MODEMCALL() ? */
        do
        rc = 'AFR_MODEMCALL'(handle,phone,DIAL_TO);
        mstat = MODSTAT.REX(rc)
        end
      else                /* do outcall with low-level comm transmission
                             functions */
        MSTAT = OUTCALL(HANDLE,PHONE,DIAL_TO)

      select
         when mstat = 'CONNECT' then
            /* now execute the beeper protocol based value of 'type' */
            success = beepixo(handle,msg)

         when mstat = 'BUSY' then  /* if busy then need to try again */
            success = 'R'

         when mstat = 'PHONENOANS' then /* if no answer then try again */
            success = 'R'

         when mstat = 'PHONEBUSY' then /* if busy then need to try again */
            success = 'R'

         when mstat = 'NO CARRIER' then /* no carrier indicates the phone answered, */
            success = 'N'               /* but no communications were established */

         otherwise
            do                 /* give up */
               mstat= modstat(afr_modemstatus(handle))
               success = fail_lit mstat
            end
      end
      /* break the connection and release the resource */
      if HighLevel then
          rc = 'AFR_MODEMDISCONNECT'(handle)
      else
          rc = 'AFR_COMCTRL'(handle,0,1)

   end
else
   do

      /* disassemble data */
      parse var msg pid msg1
      if pid = 0 then pid = ''

      /* reassemble the way we need it,
         one comma(,) for each two seconds delay */
      tphone = 'ATDT'phone',,,'pid','msg1

      /* enable DTR and DCD so we can hang up the phone when done */
      ret = afr_comctrl(handle,1,1);

      /* send first initialization string */
      ret = afr_computstr(handle,tinit,1)

      /* send second initialization string */
      ret = afr_computstr(handle,tinit1,1)

      /* wait so modem has time to respond */
      ret = afr_suspend(2000)

      /* check for response */
      size = afr_cominputbytes(handle)

      /* check to assure a path to the modem */
      if size > 0 then
         ret = afr_comgetstr(handle,size);
      else
         ret = ''
      call LogSay 'Init Ret:' ret
      if pos('OK',ret) > 0 then           /* modems talking to us */
         do

            /* call beeper paging service */
            ret = afr_computstr(handle,tphone,1)
            Response = ''
            success = ''

            /* wait for and evaluate response */
            do 60                        /* number of seconds to try */

               ret = afr_suspend(1000)   /* waiting one second each time */
               call LogSay 'Status:' d2x(afr_comstatus(handle))

               size = afr_cominputbytes(handle)
               if size > 0 then
                  do /* maybe we can recognize a response */
 /* status returned by the modem after successfully connecting
    to a telephone not answered by a modem is dependent
    how the tone paging service terminates the call.
    some services will terminate a successfull paging call
    with a busy signal, causing the modem to return a dialing
    response of 'BUSY'!  This section of of code may have to be
    modified to account for these types of operational
    peculuarities.
    */

                     Response = Response||afr_comgetstr(handle,size)
                     call LogSay 'Response:' Response
                     if pos('ERROR',Response) > 0 then
                        success = fail_lit 'ERROR'

                     if pos('CONNECT',Response) > 0 then
                        success = fail_lit 'ERROR'

                     if pos('NO DIAL TONE',Response) > 0 then
                        success = fail_lit 'NO DIALTONE'

                     if pos('NO DIALTONE',Response) > 0 then
                        success = fail_lit 'NO DIALTONE'

                     if pos('NO ANSWER',Response) > 0 then
                        success = 'R'

                     if pos('NO CARRIER',Response) > 0 then
                        success = 'Y'

                     if pos('BUSY',Response) > 0 then
                        success = 'Y'

                     if pos('OK',Response) > 0 then
                        success = 'N'

                     if pos('VOICE',Response) > 0 then
                        success = 'Y'

                     if success \= '' then
                        do
                        success = 'N'
                        leave
                        end
                  end
            end
         end
      else
         do   /* well, we don't seem to be talking to the modem! */
            success = fail_lit 'PATHFAIL'
         end

   end

/* release resources and return result */
rc = 'AFR_COMCTRL'(handle,0,0)
if HighLevel then
    rc = 'AFR_MODEMDISCONNECT'(handle)
else
    rc = 'AFR_COMCTRL'(handle,0,1)

rc = 'AFR_SUSPEND'(1000)
trace('O')
return success

/*********************************** REXX ************************************
   PROCEDURE: BEEPIXO

   PURPOSE: Implements the IXO paging service modem dialog (protocol)

   SYNOPSIS:
      SUCCESS = call beepixo '<HANDLE><PID> <MSG>'
         where
               HANDLE = handle of a successfully completed modem call
               PID = Pager ID assigned by the paging service
                   = two or more PIDs in the form <PID>,<PID>
                   = file name of list of PIDs in the form #filename, with the
                     file ext .PID.  File shall contain one PID per line
                     followed by a space or new line character
               MSG = optional message text to be sent to paging service
               SUCCESS = character string indicating result of attemtped page.

      Example: '2015382 This is a test of REX driver beeper paging.'


   RETURNS: string indicating the success or failure of page, and reason if
            failure.

 ******************************************************************************/

beepixo: procedure expose (Constants)

parse arg handle,pid msg

msg1 = program_name 'SENDING' pid msg
if Broadcast then
    rc = 'AFR_ERROR'(msg1,1)
else
    if LogActivity then
        rc = AFR_LOG(msg1)

msg1 = 'SENDING' pid msg
call LogSay msg1
if msg = '' then
    msg = ' '

/* define the non-printable characters */
CR =  'd'x
ESC = '1b'x
ACK = '6'x
NAK = '15'x
STX = '2'x
ETX = '3'x
EOT = '4'x
RS  = '1e'x
SIGNON = 'PG1000000'

IXOProto = 1
SKYProto = 2
IXOStart          = 1   ; Literal.IXOStart  = 'IXOStart '
IXOPswrd          = 2   ; Literal.IXOPswrd  = 'IXOPswrd '
IXOMsg            = 3   ; Literal.IXOMsg    = 'IXOMsg   '
IXOTerm           = 4   ; Literal.IXOTerm   = 'IXOTerm  '
IXOFail           = 5   ; Literal.IXOFail   = 'IXOFail  '
IXOStrt1          = 6   ; Literal.IXOStrt1  = 'IXOStrt1 '
SkyWaitID         = 10  ; Literal.SkyWaitID = 'SkyWaitID'
SkyPID            = 11  ; Literal.SkyPID    = 'SkyPID   '
SkyPIDSnd         = 12  ; Literal.SkyPIDSnd = 'SkyPIDSnd'
SkySndMsg         = 13  ; Literal.SkySndMsg = 'SkySndMsg'
SkySndTxt         = 14  ; Literal.SkySndTxt = 'SkySndTxt'
SkyLoop           = 15  ; Literal.SkyLoop   = 'SkyLoop  '

/* build password */
Password = ESC||SIGNON||CR

/* set step/state transition control data */
   States   = 10
 Protocol.0 = 0
  Timeout.0 = 0

/* state starts the actual dialog */
    Sends.IXOStart     = 1             /* number strings to send */
     Send.IXOStart.1   = CR            /* token to send */
  Retries.IXOStart     = 3             /* number of times to try step */
     wait.IXOStart     = 3000          /* time to wait for response (msec)*/
  timeout.IXOStart     = IXOStrt1      /* time out state to go to */
Responses.IXOStart     = 2             /* number of responses to test */
 Response.IXOStart.1   = 'ID='         /* expected response */
     Next.IXOStart.1   = IXOPswrd      /* next state if response found */
 Response.IXOStart.2   = 'SKYTEL'      /* expected response */
     Next.IXOStart.2   = SkyPID        /* next state if response found */
Delimiter.IXOStart     = '='           /* delimiter used by this state */
  Success.IXOStart     = 'N'           /* the state represent success ? */
 Protocol.IXOStart     = IXOProto

/* state sends the password for signon */
    Sends.IXOPswrd     = 1
     Send.IXOPswrd.1   = Password
  Retries.IXOPswrd     = 3
Responses.IXOPswrd     = 4
     wait.IXOPswrd     = 5000
  timeout.IXOPswrd     = 0
 Response.IXOPswrd.1   = ESC||'p'
     Next.IXOPswrd.1   = IXOMsg
 Response.IXOPswrd.2   = ESC||'[p'
     Next.IXOPswrd.2   = IXOMsg
 Response.IXOPswrd.3   = 'BAD PASSWORD'||CR
     Next.IXOPswrd.3   = IXOTerm
 Response.IXOPswrd.4   = EOT
     Next.IXOPswrd.4   = 0
Delimiter.IXOPswrd     = CR
  Success.IXOPswrd     = 'N'
 Protocol.IXOPswrd     = IXOProto

/* state sends the message */
  Retries.IXOMsg     = 3
     wait.IXOMsg     = 10000
Responses.IXOMsg     = 11
  timeout.IXOMsg     = 4
 Response.IXOMsg.1   = '^C'                /* MetroMedia uses this as success */
     Next.IXOMsg.1   = IXOTerm
 Response.IXOMsg.2   = 'TOO MANY NAKS'
     Next.IXOMsg.2   = IXOFail
 Response.IXOMsg.3   = 'BAD SEQUENCE'
     Next.IXOMsg.3   = IXOFail
 Response.IXOMsg.4   = 'TOO MUCH DATA'
     Next.IXOMsg.4   = IXOFail
 Response.IXOMsg.5   = 'TIME EXCEEDED'
     Next.IXOMsg.5   = IXOFail
 Response.IXOMsg.6   = EOT
     Next.IXOMsg.6   = IXOTerm
 Response.IXOMsg.7   = 'PAGE SENT'        /* another paging service uses this */
     Next.IXOMsg.7   = IXOTerm
 Response.IXOMsg.8   = ACK                /* still another relies on the ACK  */
     Next.IXOMsg.8   = IXOTerm
 Response.IXOMsg.9   = 'CHARACTERS ACCEPTED'
     Next.IXOMsg.9   = IXOTerm
 Response.IXOMsg.10  = 'Illegal Pager ID'
     Next.IXOMsg.10  = IXOFail
 Response.IXOMsg.11  = RS
     Next.IXOMsg.11  = IXOFail
Delimiter.IXOMsg     = CR
  Success.IXOMsg     = 'Y'
 Protocol.IXOMsg     = IXOProto

/* state terminates the session */
     Send.IXOTerm.1     = EOT||CR
    Sends.IXOTerm     = 1
  Retries.IXOTerm     = 3
     wait.IXOTerm     = 3000
Responses.IXOTerm     = 1
  timeout.IXOTerm     = 0
 Response.IXOTerm.1   = ESC||EOT
     Next.IXOTerm.1   = 0
Delimiter.IXOTerm     = CR
  Success.IXOTerm     = 'Y'
 Protocol.IXOTerm     = IXOProto

/* state terminates the session */
     Send.IXOFail.1     = EOT||CR
    Sends.IXOFail     = 1
  Retries.IXOFail     = 3
     wait.IXOFail     = 3000
Responses.IXOFail     = 1
  timeout.IXOFail     = 0
 Response.IXOFail.1   = ESC||EOT
     Next.IXOFail.1   = 0
Delimiter.IXOFail     = CR
  Success.IXOFail     = 'N'
 Protocol.IXOFail     = IXOProto

/* state starts the actual dialog */
    Sends.IXOStrt1     = 1             /* number strings to send */
     Send.IXOStrt1.1   = CR||CR||CR||CR||CR||CR||CR||CR/* token to send */
  Retries.IXOStrt1     = 6             /* number of times to try step */
     wait.IXOStrt1     = 3000          /* time to wait for response (msec)*/
  timeout.IXOStrt1     = 0             /* time out state to go to */
Responses.IXOStrt1     = 2             /* number of responses to test */
 Response.IXOStrt1.1   = 'ID='         /* expected response */
     Next.IXOStrt1.1   = IXOPswrd      /* next state if response found */
 Response.IXOStrt1.2   = 'SKYTEL'      /* expected response */
     Next.IXOStrt1.2   = SkyWaitID     /* next state if response found */
Delimiter.IXOStrt1     = '='           /* delimiter used by this state */
  Success.IXOStrt1     = 'N'           /* the state represent success ? */
 Protocol.IXOStrt1     = IXOProto

/* state SKYTEL - waits for pager ID prompt */
    Sends.SkyWaitID     = 0
  Retries.SkyWaitID     = 3
     wait.SkyWaitID     = 5000
Responses.SkyWaitID     = 2
  timeout.SkyWaitID     = 0
 Response.SkyWaitID.1   = 'PAGER ID:'
     Next.SkyWaitID.1   = SkyPID
 Response.SkyWaitID.2   = 'INVALID'
     Next.SkyWaitID.2   = SkyLoop
Delimiter.SkyWaitID     = ':'
  Success.SkyWaitID     = 'N'
 Protocol.SkyWaitID     = SkyProto

/* state SKYTEL - sends pager ID */
    Sends.SkyPID     = 1
     Send.SkyPID.1   = '99'||CR
  Retries.SkyPID     = 3
     wait.SkyPID     = 5000
Responses.SkyPID     = 2
  timeout.SkyPID     = 0
 Response.SkyPID.1   = 'FUNCTION:'
     Next.SkyPID.1   = SkyPIDSnd
 Response.SkyPID.2   = 'DISCONNECTING'
     Next.SkyPID.2   = 0
Delimiter.SkyPID     = ':'
  Success.SkyPID     = 'N'
 Protocol.SkyPID     = SkyProto

/* state SKYTEL - request to send message */
     Send.SkyPIDSnd.1   = '99'||CR
    Sends.SkyPIDSnd     = 1
  Retries.SkyPIDSnd     = 3
     wait.SkyPIDSnd     = 5000
Responses.SkyPIDSnd     = 3
  timeout.SkyPIDSnd     = 0
 Response.SkyPIDSnd.1   = 'ENTER ALPHA'
     Next.SkyPIDSnd.1   = SkySndMsg
 Response.SkyPIDSnd.2   = 'ENTER NUMERIC'
     Next.SkyPIDSnd.2   = SkySndMsg
 Response.SkyPIDSnd.3   = 'INVALID'
     Next.SkyPIDSnd.3   = 0
Delimiter.SkyPIDSnd     = ':'
  Success.SkyPIDSnd     = 'N'
 Protocol.SkyPIDSnd     = SkyProto

/* state SKYTEL - sends message text */
    Sends.SkySndMsg     = 1
     Send.SkySndMsg.1   = '99'||CR
  Retries.SkySndMsg     = 3
     wait.SkySndMsg     = 5000
Responses.SkySndMsg     = 2
  timeout.SkySndMsg     = 0
 Response.SkySndMsg.1   = 'SEND ?'
     Next.SkySndMsg.1   = SkySndTxt
 Response.SkySndMsg.2   = 'INVALID:'
     Next.SkySndMsg.2   = 0
Delimiter.SkySndMsg     = ':'
  Success.SkySndMsg     = 'N'
 Protocol.SkySndMsg     = SkyProto

/* state SKYTEL - sends text */
     Send.SkySndTxt.1   = CR
    Sends.SkySndTxt     = 1
  Retries.SkySndTxt     = 3
     wait.SkySndTxt     = 10000
Responses.SkySndTxt     = 2
  timeout.SkySndTxt     = 0
 Response.SkySndTxt.1   = 'FUNCTION:'
     Next.SkySndTxt.1   = SkyLoop
 Response.SkySndTxt.2   = 'INVALID'
     Next.SkySndTxt.2   = 0
Delimiter.SkySndTxt     = ':'
  Success.SkySndTxt     = 'N'
 Protocol.SkySndTxt     = SkyProto

/* state SKYTEL - requests to send pager ID  or disconnect */
     Send.SkyLoop.1  = '99'||CR
    Sends.SkyLoop    = 1
  Retries.SkyLoop    = 3
     wait.SkyLoop    = 5000
Responses.SkyLoop    = 2
  timeout.SkyLoop    = 0
 Response.SkyLoop.1  = 'PAGER ID:'
     Next.SkyLoop.1  = SkyPID
 Response.SkyLoop.2  = 'DISCONNECTING'
     Next.SkyLoop.2  = 0
Delimiter.SkyLoop    = ':'
  Success.SkyLoop    = 'Y'
 Protocol.SkyLoop    = SkyProto

Pids            = 0
Received        = ''

Success = 'N'
error_flag = 1

/* parse comma separated PID into individulal entries in the PID array */
do while pid <> ''
    Pids = Pids + 1
    parse var pid t ',' pid
    Pid.Pids = strip(t)
    end

/* build IXO message strings, one for each PID identified */
do PidX = 1 to Pids
   /* build the string to transmit */
   temp = STX||Pid.PidX||CR||msg||CR||ETX
   /* calculate the check sum */
   len = length(temp)
   sum = 0
   do i = 1 to len
      sum = sum + c2d(substr(temp,i,1))
   end

   /* normalize if needed */
   if sum > 4095 then
      sum = sum - 4096

   /* compute the checksum characters */
   do i = 3 to 1 by -1
      rem = sum // 16
      sum = sum % 16
      sum.i = substr('0123456789:;<=>?',rem+1,1);
   end

   /* and attach them to complete the message string for step 3, and stuff
      into the storage array */
    Send.IXOMsg.PidX      = temp||sum.1||sum.2||sum.3||CR
   PidX1                  = PidX
    Send.SkyPID.PidX1     = Pid.PidX||CR       /* Pid */
    Send.SkyPIDSnd.PidX1  = '1'CR              /* send message */
    Send.SkySndMsg.PidX1  = msg||CR            /* message */
    Send.SkySndTxt.PidX1  = CR                 /* next Pid */
    Send.SkyLoop.PidX1    = '3'||CR            /* next Pid */
   end
Send.SkyLoop.PidX1        =  '99'CR            /* last must be a disconnect */
/* note the number of messages to send this connect */
Sends.IXOMsg              = Pids
Sends.SkyPID              = Pids + 1
Sends.SkyPIDSnd           = Pids + 1
Sends.SkySndMsg           = Pids + 1
Sends.SkySndTxt           = Pids + 1
Sends.SkyLoop             = Pids + 1

/* now execute the beeper protocol */
State                     = 1
ResponseFound             = 0
Connect                   = 1

/* IXO/TAPS protocol loop */
do while Protocol.State = IXOProto

   /* set the expected response delimiter */
   Delimiter = Delimiter.State

   /* if there are no responses then go to the next state */
/* if Responses.State = 0 then
      do
      State = Timeout.State
      iterate
      end *//* if do */

   /* this loop sends multiple transmissions as determined by the content of
      of Sends.State.  At this time only step three will have more than one
      transmission to make */
   do Sent = 1 to Sends.State while ((Protocol.State = IXOProto) & Connect)
      ResponseFound = 0

      /* make sure it gets sent */
      do Tried = Retries.State to 0 by -1 until (ResponseFound | Protocol.State <> IXOProto)
         /* send out the data */
         if ShowSends then
             Say program_name time() 'State:'Literal.State 'Sent:'Sent 'Sending:'Send.State.Sent
         if LogActivity then
            n = afr_log('State:'Literal.State 'Sent:'Sent 'Sending:'Send.State.Sent)
         rc = 'AFR_COMPUTSTR'(handle,Send.State.Sent,0)

         /* wait for a response back from modem */
         call IXOWaitResponse
         end /* do tried */

      end /* do Sent = 1 */

      if ResponseFound then
          do
          Success = Success.State
          State = Next.State.RespIndex
          iterate
          end

      if Tried <= 0 then
          State = Timeout.State

   end /* do while State > */

/* SKYTEL Interactive Remote Terminal protocol loop */
do while Protocol.State = SkyProto
   do Sent = PidX1 to 1
      /* set the expected response delimiter */
      ResponseFound = 0
      Delimiter = Delimiter.State

      /* send out the data */
      if Sends.State > 0 then
         do
         if ShowSends then
             Say program_name time() 'State:'Literal.State 'Sent:'Sent 'Sending:'Send.State.Sent
         if LogActivity then
            n = afr_log('State:'Literal.State 'Sent:'Sent 'Sending:'Send.State.Sent)
         rc = 'AFR_COMPUTSTR'(handle,Send.State.Sent,0)
         end

      /* if there are no responses then go to the next state */
/*    if Responses.State = 0 then
         do
         State = Timeout.State
         iterate
         end *//* if do */

      /* wait for a response back from modem */
      call IXOWaitResponse
      if ResponseFound then
          do
          Success = Success.State
          State = Next.State.RespIndex
          end
      end
   end

return Success

/* this procedure accepts the response from the paging responses and attempts
   to match them to one of the reponses defined for that state.  The variable
   Found is set to true (1) if that match is made.  */

IXOWaitResponse:

   /* initialize delay */
   Delay = Wait.State
   Wait  = 500

   /* do until response matched or delay expires */
   do while Delay > 0
      /* wait a while for the response */
      rc = 'AFR_SUSPEND'(Wait)
      if afr_comstatus(handle) < 128 then
         do
         Connect = 0
         State = 0
         return
         end
      Delay = Delay - Wait

      /* get the response */
      received = received||'AFR_COMGETSTR'(handle,10)
      Pos = pos(Delimiter, Received)
      if Pos = 0 then
         do
         if Delimiter <> CR then
            Pos = pos(CR, Received)
         end

      if Pos > 0 then
         do
         Resp = left(Received, Pos)
         Received = substr(Received, Pos + 1)
         if ShowRcvs then
             Say program_name time() 'State' Literal.State 'Received' Resp
         if LogActivity then
             n = afr_log('State' Literal.State 'Received' Resp)
         /* scan through the possible responses for this state */
         do respndx = 1 to Responses.State
            /* a response is identified the set the next state */
            expected = Response.State.respndx
            if pos(expected,resp) > 0 then
               do
               ResponseFound = 1
               RespIndex = respndx
               return
               end
            end /* do respndx */

         end /* if Pos > */

      end /* do while */

    call LogSay 'delay expired State:'Literal.State 'Tries:'Tried
return

/* OUTCALL implements the outdial to a modem, and evaluates the response
   returned by the modem as a result of the outdial attempt */

OutCall: procedure expose Handle Phone InitStr. RespStr. LogConnect program_name LogActivity Broadcast HighLevel save_ansi

   call ModemInit

   RespSize = afr_cominputbytes(Handle)
   if RespSize > 0 then
      Rc1 = afr_comgetstr(Handle,RespSize)

   Resp = ''

   Rc1 = afr_comctrl(Handle,1,1)
   Rc1 = afr_computstr(Handle,'ATDT'Phone,1)
   Say 'Calling:' Phone
   if LogConnect then
       call afr_log 'Calling:' Phone

   do 20
      Resp = ModemResp(60.0)
      select
         when Resp = 1 then
            iterate
         when Resp = 2 then
            return RespStr.Resp
         otherwise
            return RespStr.Resp
         end
      end

return 'TIMED OUT'

ModemInit: procedure expose Handle Initstr. RespStr. LogConnect program_name LogActivity Broadcast HighLevel save_ansi

   n = afr_comctrl(Handle, 1, 1)
   ansiSave = afr_ansi()
   n = afr_ansi(0)

   Say 'Starting modem Initialization'
   if LogConnect then
      call afr_log 'Starting modem Initialization'
   RespSize = afr_Cominputbytes(Handle)
   RespString = ''

   if RespSize > 0 then
      Rc1 = afr_Comgetstr(Handle,RespSize)


   RespStr = ''
   Start = 1
   do i = 1 to Initstr.0
      Say 'Sent:'Initstr.i
      if LogConnect then
         call afr_log 'Sent:'Initstr.i
      ret = afr_computstr(Handle, Initstr.i, 1)
      ETime = time('E') + 10.0
      Resp = ModemResp(10.0)
      /*call LogSay Resp RespStr.Resp*/
      end
   n = afr_ansi(ansiSave)
return

ModemResp: procedure expose Handle Initstr. RespStr. LogConnect program_name LogActivity Broadcast HighLevel save_ansi

    parse arg Timeout
    CRLF = d2c(13)||d2c(10)
    ETime = time('E') + Timeout
    Response = ''
    do while ETime > time('E')
        call afr_suspend(1000)
        RespLen = afr_cominputbytes(Handle)
        if RespLen = 0 then
            iterate
        Response = Response||afr_comgetstr(Handle, RespLen)
        RPos = pos(CRLF, Response)
        do while RPos > 0
            First = substr(Response, 1, RPos-1)
            Response = substr(Response, RPos+2)
            Say 'Resp:' First
            if LogConnect then
               call afr_log 'Resp:' First
            if length(First) >= 2 then
                do
                if compare('AT', First) <> 2 then
                    do
                    do i = 1 to RespStr.0
                        CLen = compare(First, RespStr.i)
                        RLen = Length(RespStr.i)
                            if Clen = 0 | CLen >= RLen then
                            do
                            return i
                            end
                        end
                    end
                end
            RPos = pos(CRLF, Response)
            end
        end

return 0

LogSay: procedure expose program_name LogActivity

   text = arg(1)
   Say time() program_name text
   if LogActivity then
      x = afr_log(program_name text)
   return
