/*************************** REXX *********************************************

   SCRIPT: HMCACT.REX
   October 14, 2005

   PURPOSE: This is a function routine callable from customer written automation
            scripts to perform the following actions on a 9672 HMC:
               Activate or deactivate a CPC Object.
               Activate (IPL) or perform a system reset on a CPC Image.
               Send a operating system command to a CPC Image.

   SYNOPSIS:
      rc = HMCACT(Host Community CPC LPAR Profile Action Command)
         or
      call HMCACT Host Community CPC LPAR Profile Action Command

   Inputs:  Arg(1): Host Name.
            Arg(2): Community Name. (in quotes to maintain character case)
            Arg(3): CPC Name.
            Arg(4): LPAR Name.
            Arg(5): Profile Name.
            Arg(6): Action Type.  (POR, POFF, IPL, SYSRESET, SO, or SEND_OPSYS)
            Arg(7): Command to issue.

   Outputs: Status Display.

   Returns: 0 Success
            Non 0 Failure

   Note: Due to the complexity of the program it is recommended that customers
         do not modify this program.

   Modification History:

      August 22, 1996, Jim Collins, preparation for initial delivery to
      customer.

      March 6, 1998   - Jim Collins, cleaned up Initialize and Terminate
       sequences to work properly with Warp 4 (HMC) DLLs.

      March 17, 1998  - Jim Collins, added 'LOAD' capability with parameters.
                        Paramters are:
                            Load Address:
                            Load Parameter:
                            Clear Indicator:
                               Clear indicator may be:
                                  YES for Load Clear
                                  NO  for No Load Clear
                                  SAD for Stand Alone Dump

      June 28, 2002   - This program is a clone of the program delivered with
                        AF/Remote for OS/2.  Added DLL selection logic from the
                        Win NT (Rp100) version which does not include the 'LOAD'
                        command.
      September 13, 2002 - Corrected DEACTIVATE command to DEACTIVATE vs ACTIVATE
      October 16, 2002   - Added missing return code evaluation and associated messages
                           to the sending of the ACTIVATE and LOAD commands.
      November 27, 2002  - Wrong final result on successful SEND_OPSYS command,
                           (MSG2007), corrected.
                           Added statements to control which HMC DLL is to be loaded
                           if not already loaded, (RpRxHMC, Candle) or (hwmcaorx, IBM).
                           Set to load the Candle provided DLL.

      October 14, 2005  - Per HMC team - change to loop (avoid RC=3 too much data).
         Major cleanup resulting from live testing (e.g. zSeries trailing NULL -
         thanks to Steve Barry at Farmers).  Retry rc=19 a couple of times.

 ******************************************************************************/

/* trace 'o' trace is off
   trace 'r' trace (results) is on */
Trace 'o'

/* Debug = 0 debug logging disabled
   Debug = 1 debug logging enabled */
Debug = 0

/* set the timeout value, in milliseconds, to be used in all API calls */
API_TIMEOUT = 20000

/* number of times to retry a get if it times out */
RETRY_TIMEOUT = 3

/* Set some error detection signals        */
Signal On Halt
Signal On Syntax

/* initalized some control constants and variables */
HMCMsg     = 0           /* Changed to 1 if error message issued */
LogDetails = 1           /* Set to 1 to log parameters provided
                            Set to 0 to disable */

/* retrieve the operating system platform and the name of the source program */
Parse Source pgm_name
os_platform = SubWord(pgm_name, 1, 1)
program_name = SubStr(pgm_name,Lastpos('\',pgm_name)+1)':'
program_version = 'October 14, 2005'

/* see if the AF/Remote function package is loaded and set flag */
AFR_ENV = 1             /* default environement */
If RxFuncQuery('AFR_CLS') Then
   AFR_ENV = 0          /* if not running in AFR then set flag off */

/* assume using the IBM HMC Windows DLLs */
HMCDLL = 'I'
If HMCDLL = 'C' Then HMCDLL = 'RpRxHMC'
                Else HMCDLL = 'hwmcaorx'

Say "Program    : "program_name
Say "Version    : "program_version
Say "Date       : "Date()
Say "Time       : "Time()

Parse Arg Args
Parse Var Args hmc_host Hmc_Community Hmc_CPCName Hmc_LPARName Hmc_Profile Action Command
Full_LPAR   = Hmc_CPCName || ':' || Hmc_LPARName

/* must be at least 6 parameters to continue */
If Arg() < 6 & Words(Args) < 6 Then Do
   Call MSG9005
   Return '8'
End

Select
   When Action = 'ACTIVATE'   then NOP
   When Action = 'DEACTIVATE' then NOP
   When Action = 'POR'        then NOP
   When Action = 'POFF'       then NOP
   When Action = 'IPL'        then NOP
   When Action = 'SYSRESET'   then NOP
   When Action = 'SO'         then NOP
   When Action = 'SEND_OPSYS' then NOP
   When Action = 'LOAD'       then NOP
   When Action = 'SAD'        then NOP
   When Action = 'NOOP'       then NOP
   Otherwise Do
      Call MSG9001
      Return '8'
   End
End

If (Action = 'LOAD') Then Do
        Parse Var Command Load_Addr Load_Parm Clear_Msg
        Command = "NONE"
End
Else Do
        Load_Addr = "NONE"
        Load_Parm = "NONE"
        Clear_Msg = "NONE"
End

/* Display parameters provided */
Say 'HOST:     ' hmc_host
Say 'Community:' hmc_community
Say 'CPC:      ' hmc_CPCName
Say 'LPAR:     ' hmc_LPARNAME
Say 'Full LPAR:' Full_LPAR
Say 'Profile:  ' Hmc_Profile
Say 'Action:   ' Action

If (Action = 'LOAD') Then Do
   Say 'Load Address:        ' Load_Addr
   Say 'Load Parameter:      ' Load_Parm
   If (Clear_Msg = 'SAD') Then Say 'Stand Alone Dump:    ' 'Yes'
                          Else Say 'Load Clear:          ' Clear_Msg
End
Else Say 'Command:  ' Command

Select
   When Action = 'POR'  Then Wait_Time = 1800000 /* 30 Minutes */
   When Action = 'POFF' Then Wait_Time = 1800000 /* 30 Minutes */
   Otherwise                 Wait_Time = 180000  /* 3 minutes */
End

/* 5 seconds to see these messages */
Call SysSleep 5

Call Init_HMC

 /* obtain status */
Call Obtain_Status

 /* display CPC status */
Call Get_CPCObjNum
Call Get_LPARObjNum Full_LPAR

/* */
Call Do_Action Action Hmc_Profile CPCObjNum LPARObjNum Load_Addr Load_Parm Clear_Msg Command

/* Done */
Call Exit_HMC

If HMCMsg Then Return '8'
          Else Return '0'

/***********************************************************************
    Subroutine Section starts here
 ************************************************************************/

/***********************************************************************
*
*  Obtain CPC names and attributes
*
***********************************************************************/
Get_CPCs:

   If Debug Then Say 'entering Get_CPCs'

   /****************************************************************************/
   /* Ask GetNext to return each SNMP ID for all defined CPC's.                */
   /****************************************************************************/

   CPC.0 = 0
   id = HWMCA_CFG_CPC_ID||'.'||HWMCA_NAME_SUFFIX

   rc = HWMCA_DE_NO_ERROR
   prefix = id
   done = 0
   i = 1

   Do While rc == HWMCA_DE_NO_ERROR & done == 0
      current_attribute = 'Defined CPC Group'
      Call Get_aGetNext id current_attribute
      If rc \= HWMCA_DE_NO_ERROR Then Do
         Call Term_HMC
         Exit 'N'
      End

      If prefix == SubStr(OUTPUT.1.DATA,1,Length(prefix)) Then Do
         /*  strip any trailing null */
         id = Strip(OUTPUT.1.DATA,'Trailing','00'x)
         /*  change any '0A'x(line feed) byte to a blank and strip any trailing null */
         nameo = Strip(Translate(OUTPUT.2.DATA,':','0A'x),'Trailing','00'x)
         objid = HWMCA_CFG_CPC_ID||SubStr(id,LastPos('.',id))
         If Debug Then Say "CPC #:"i "has name of:" nameo "and object id of:" objid
         CPC.0 = i

         /* initialize variable attributes */
         CPC.i.ADDR     = objid
         CPC.i.NAME     = ' '
         CPC.i.STATERR  = ' '
         CPC.i.STATUS   = ' '
         CPC.i.ASTATUS  = ' '
         CPC.i.IMLMODE  = 0
         CPC.i.ACTPROF  = ' '
         CPC.i.NOOFIMAGES = 0

         /* obtain attributes for the CPC  */
         Call Get_CPC
         If rc \= HWMCA_DE_NO_ERROR Then Do
            Call Term_HMC
            Exit 'N'
         End

         i=i+1
      End /* do */
      Else Do
         done = 1
      End /* do */
   End /* do */

   If Debug Then Say "Number of CPCs:" CPC.0
   If CPC.0 = 0 Then Do
      Call MSG2013 current_attribute
      Call Term_HMC
      Exit 'N'
   End

   Return


/***********************************************************************
*
*  Obtain CPC object attributes
*
***********************************************************************/
Get_CPC:

   /**************************************************************************/
   /* Object attributes for CPCs                                             */
   /**************************************************************************/

   /**************************************************************************/
   /* attribute: Name                                                        */
   /* operation: Get                                                         */
   /* type     : OCTETSTRING                                                 */
   /**************************************************************************/

   current_attribute = 'Name'

   rc = RxHwmcaBuildAttributeId('CPCID',CPC.i.ADDR,HWMCA_NAME_SUFFIX)
   If rc \= HWMCA_DE_NO_ERROR Then Call MSG2004 current_attribute
   Call Get_aGet CPCID current_attribute
   If rc \= HWMCA_DE_NO_ERROR Then Return
   /*  change any '0A'x(line feed) byte to a blank and strip any trailing null */
   CPC.i.NAME = Strip(Translate(OUTPUT.1.DATA,':','0A'x),'Trailing','00'x)
   If Debug Then Say "name built:" CPC.i.NAME

   /**************************************************************************/
   /* attribute: Status error                                                */
   /* operation: Get                                                         */
   /* type     : INTEGER                                                     */
   /**************************************************************************/

   current_attribute = 'Status Error'

   rc = RxHwmcaBuildAttributeId('CPCID',CPC.i.ADDR,HWMCA_STATUS_ERROR_SUFFIX)
   If rc \= HWMCA_DE_NO_ERROR Then Call MSG2004 current_attribute
   Call Get_aGet CPCID current_attribute
   If rc \= HWMCA_DE_NO_ERROR Then Return
   CPC.i.STATERR = OUTPUT.1.DATA
   If Debug Then Say "Staterr built:" CPC.i.STATERR

   /**************************************************************************/
   /* attribute: Status                                                      */
   /* operation: Get                                                         */
   /* type     : INTEGER                                                     */
   /**************************************************************************/

   current_attribute = 'Status'

   rc = RxHwmcaBuildAttributeId('CPCID',CPC.i.ADDR,HWMCA_STATUS_SUFFIX)
   If rc \= HWMCA_DE_NO_ERROR Then Call MSG2004 current_attribute
   Call Get_aGet CPCID current_attribute
   If rc \= HWMCA_DE_NO_ERROR Then Return
   CPC.i.STATUS = OUTPUT.1.DATA
   If Debug Then Say "Status built:" CPC.i.STATUS

   /**************************************************************************/
   /* attribute: Acceptable status                                           */
   /* operation: Get                                                         */
   /* type     : INTEGER                                                     */
   /**************************************************************************/

   current_attribute = 'Acceptable Status'

   rc = RxHwmcaBuildAttributeId('CPCID',CPC.i.ADDR,HWMCA_EXPECTED_STATUS_SUFFIX)
   If rc \= HWMCA_DE_NO_ERROR Then Call MSG2004 current_attribute
   Call Get_aGet CPCID current_attribute
   If rc \= HWMCA_DE_NO_ERROR Then Return
   CPC.i.ASTATUS = OUTPUT.1.DATA
   If Debug Then Say "Astatus built:" CPC.i.ASTATUS

   /**************************************************************************/
   /* attribute: IML mode                                                    */
   /* operation: Get                                                         */
   /* type     : INTEGER                                                     */
   /**************************************************************************/

   current_attribute = 'IML Mode'

   rc = RxHwmcaBuildAttributeId('CPCID',CPC.i.ADDR,HWMCA_IMLMODE_SUFFIX)
   If rc \= HWMCA_DE_NO_ERROR Then Call MSG2004 current_attribute
   Call Get_aGet CPCID current_attribute
   If rc \= HWMCA_DE_NO_ERROR Then Return
   CPC.i.IMLMODE = OUTPUT.1.DATA
   If Debug Then Say "IMLMODE built:" CPC.i.IMLMODE

   /**************************************************************************/
   /* attribute: Activation Profile name                                     */
   /* operation: get                                                         */
   /* type     : OCTETSTRING                                                 */
   /**************************************************************************/

   current_attribute = 'Active Profile'

   rc = RxHwmcaBuildAttributeId('CPCID',CPC.i.ADDR,HWMCA_ACTIVATION_PROFILE_SUFFIX)
   If rc \= HWMCA_DE_NO_ERROR Then Call MSG2004 current_attribute
   Call Get_aGet CPCID current_attribute
   If rc \= HWMCA_DE_NO_ERROR Then Return
   /*  strip any trailing null */
   CPC.i.ACTPROF = Strip(OUTPUT.1.DATA,'Trailing','00'x)
   If Debug Then Say "Actprof built:" CPC.i.ACTPROF

   Return


/***********************************************************************
*
*  Obtain CPC object images
*
***********************************************************************/
Get_CPC_Images:

   If Debug Then Say 'entering Get_CPC_Images'

   /****************************************************************************/
   /* Ask GetNext to return each SNMP ID for all defined CPC images.           */
   /****************************************************************************/

   CPCIMAGE.0 = 0
   id = HWMCA_CPC_IMAGE_ID||'.'||HWMCA_NAME_SUFFIX

   rc = HWMCA_DE_NO_ERROR
   prefix = id
   done = 0
   i = 1

   Do While rc == HWMCA_DE_NO_ERROR & done == 0
      current_attribute = 'Defined CPC Images'
      Call Get_aGetNext id current_attribute
      If rc \= HWMCA_DE_NO_ERROR Then Do
         Call Term_HMC
         Exit 'N'
      End

      If prefix == SubStr(OUTPUT.1.DATA,1,Length(prefix)) Then Do
         /*  strip any trailing null */
         id = Strip(OUTPUT.1.DATA,'Trailing','00'x)
         /*  change any '0A'x(line feed) byte to a blank and strip any trailing null */
         nameo = Strip(Translate(OUTPUT.2.DATA,':','0A'x),'Trailing','00'x)
         objid = HWMCA_CPC_IMAGE_ID||SubStr(id,LastPos('.',id))
         If Debug Then Say "Image #:"i "has name of:" nameo "and object id of:" objid
         CPCIMAGE.0 = i

         /* initialize variable attributes */
         CPCIMAGE.i.ADDR    = objid
         CPCIMAGE.i.PNAME   = ' '
         CPCIMAGE.i.OSNAME  = ' '
         CPCIMAGE.i.OSTYPE  = ' '
         CPCIMAGE.i.OSLEVEL = ' '
         CPCIMAGE.i.IMLMODE = 0

         /* obtain attributes for the Image  */
         Call Get_CPC_Image
         If rc \= HWMCA_DE_NO_ERROR Then Do
            Call Term_HMC
            Exit 'N'
         End

         i=i+1
      End /* do */
      Else Do
         done = 1
      End /* do */
   End /* do */

   If Debug Then Say "Number of CPCs:" CPCIMAGE.0
   If CPCIMAGE.0 = 0 Then Do
      Call MSG2012 current_attribute
      Call Term_HMC
      Exit 'N'
   End

   Return


/***********************************************************************
*
*  Obtain CPC object image attributes
*
***********************************************************************/
Get_CPC_Image:

   /**************************************************************************/
   /* Object attributes for CPC Image                                        */
   /**************************************************************************/

   /**************************************************************************/
   /* attribute: Name                                                        */
   /* operation: Get                                                         */
   /* type     : OCTETSTRING                                                 */
   /**************************************************************************/

   current_attribute = 'Name'

   rc = RxHwmcaBuildAttributeId('IMAGEID',CPCIMAGE.i.ADDR,HWMCA_NAME_SUFFIX)
   If rc \= HWMCA_DE_NO_ERROR Then Call MSG2004 current_attribute
   Call Get_aGet IMAGEID current_attribute
   If rc \= HWMCA_DE_NO_ERROR Then Return
   /*  change any '0A'x(line feed) byte to a blank and strip any trailing null */
   CPCIMAGE.i.NAME = Strip(Translate(OUTPUT.1.DATA,':','0A'x),'Trailing','00'x)
   If Debug Then Say "name built:" CPCIMAGE.i.NAME

   /**************************************************************************/
   /* attribute: Parent Name                                                 */
   /* operation: Get                                                         */
   /* type     : OCTETSTRING                                                 */
   /**************************************************************************/

   current_attribute = 'Parent Name'

   rc = RxHwmcaBuildAttributeId('IMAGEID',CPCIMAGE.i.ADDR,HWMCA_PARENT_NAME_SUFFIX)
   If rc \= HWMCA_DE_NO_ERROR Then Call MSG2004 current_attribute
   Call Get_aGet IMAGEID current_attribute
   If rc \= HWMCA_DE_NO_ERROR Then Return
   /*  change any '0A'x(line feed) byte to a blank and strip any trailing null */
   CPCIMAGE.i.PNAME = Strip(Translate(OUTPUT.1.DATA,':','0A'x),'Trailing','00'x)
   If Debug Then Say "pname built:" CPCIMAGE.i.PNAME

   /**************************************************************************/
   /* attribute: Status                                                      */
   /* operation: Get                                                         */
   /* type     : INTEGER                                                     */
   /**************************************************************************/

   current_attribute = 'Status'

   rc = RxHwmcaBuildAttributeId('IMAGEID',CPCIMAGE.i.ADDR,HWMCA_STATUS_SUFFIX)
   If rc \= HWMCA_DE_NO_ERROR Then Call MSG2004 current_attribute
   Call Get_aGet IMAGEID current_attribute
   If rc \= HWMCA_DE_NO_ERROR Then Return
   CPCIMAGE.i.STATUS = OUTPUT.1.DATA
   If Debug Then Say "Status built:" CPCIMAGE.i.STATUS

   /**************************************************************************/
   /* attribute: Acceptable status                                           */
   /* operation: Get                                                         */
   /* type     : INTEGER                                                     */
   /**************************************************************************/

   current_attribute = 'Acceptable Status'

   rc = RxHwmcaBuildAttributeId('IMAGEID',CPCIMAGE.i.ADDR,HWMCA_EXPECTED_STATUS_SUFFIX)
   If rc \= HWMCA_DE_NO_ERROR Then Call MSG2004 current_attribute
   Call Get_aGet IMAGEID current_attribute
   If rc \= HWMCA_DE_NO_ERROR Then Return
   CPCIMAGE.i.ASTATUS = OUTPUT.1.DATA
   If Debug Then Say "Astatus built:" CPCIMAGE.i.ASTATUS

   /**************************************************************************/
   /* attribute: Activation Profile                                          */
   /* operation: Get                                                         */
   /* type     : OCTETSTRING                                                 */
   /**************************************************************************/

   current_attribute = 'Activation Profile'

   rc = RxHwmcaBuildAttributeId('IMAGEID',CPCIMAGE.i.ADDR,HWMCA_ACTIVATION_PROFILE_SUFFIX)
   If rc \= HWMCA_DE_NO_ERROR Then Call MSG2004 current_attribute
   Call Get_aGet IMAGEID current_attribute
   If rc \= HWMCA_DE_NO_ERROR Then Return
   /*  change any '0A'x(line feed) byte to a blank and strip any trailing null */
   CPCIMAGE.i.ACTPROF = Strip(Translate(OUTPUT.1.DATA,':','0A'x),'Trailing','00'x)
   If Debug Then Say "ACTPROF built:" CPCIMAGE.i.ACTPROF

   Return


/***********************************************************************
*
*  Thread objects routine
*
***********************************************************************/
Thread_Objects:

   If Debug Then Say 'entering Thread_Objects'

   /* If CPC object in LPAR or Native mode, make sure to thread image */
   Do i = 1 To CPC.0

      /*************************************************************************************************/
      /* If CPC is set to LPAR mode get images for CPC, Cross reference image and CPC name and status  */
      /*************************************************************************************************/

      If (CPC.i.IMLMODE = HWMCA_IML_LPAR_MODE) | (CPC.i.IMLMODE = HWMCA_IML_ESA390_MODE) Then Do
         Do j = 1 To CPCIMAGE.0
            If CPC.i.NAME = CPCIMAGE.j.PNAME Then Do
               CPC.i.NOOFIMAGES = CPC.i.NOOFIMAGES + 1
               imnum = CPC.i.NOOFIMAGES
               CPC.i.imnum.IMAGENAME = CPCIMAGE.j.NAME
               CPC.i.imnum.IMAGENUM  = j
               CPC.i.imnum.ISTATUS   = CPCIMAGE.j.STATUS
               CPC.i.imnum.IMODE     = CPC.i.IMLMODE
               CPCIMAGE.j.PNUMBER    = i
               If Debug Then Say "Threaded:" CPCIMAGE.j.NAME "to" CPC.i.NAME
            End
         End
      End
   End

   Return


/***********************************************************************
*
*  Obtain status sub-routine
*
***********************************************************************/
Obtain_Status:

   Call Clear_Screen
   Call Say_Header
   text0 = '**********************************************************'
   text1 = '* Obtaining Defined CPC and CPC Image status information *'
   text2 = '*                   Please Standby                       *'
   text3 = '**********************************************************'
   Say
   Say Center(text0,80)
   Say Center(text1,80)
   Say Center(text2,80)
   Say Center(text3,80)

   /* get defined cpcs */
   Call Get_CPCs

   /* in case we don't call Get_CPC_Images */
   CPCIMAGE.0 = 0

   /*****************************************************************
        Check for CPC images only if action is not:
          1) a Power On Reset (POR)
              Note: a POR after a POFF will blow up in the Get_CPC_Images mode
          2) a Power Off Command (POFF)
   ******************************************************************/

   If (Action \= "POR") & (Action \= "POFF") Then Do
      /* get defined cpc images */
      Call Get_CPC_Images
      /* thread CPC and CPC Images into arrays  */
      Call Thread_Objects
   End

   Return


/***********************************************************************
*
*    Get CPC Object Number
*
***********************************************************************/
Get_CPCObjNum:

   If Debug Then Say 'entering Get_CPCObjNum'

   CPCObjNum = ''

   Do j = 1 To CPC.0
      If CPC.j.NAME = HMC_CPCName Then Do
         CPCObjNum = j
         x = CPC.j.STATUS
         y = CPC.j.IMLMODE
         Say 'CPC:'
         Say Right(j,3)')' Left(CPC.j.NAME,20) 'Status =' Left(STATUS.x,20) 'Mode =' MODE.y
         Leave j
      End
   End /* do */
   If CPCObjNum = '' Then Call MSG9002

   Return

/***********************************************************************
*
*    Get LPAR Object Number
*
***********************************************************************/
Get_LPARObjNum:

   If Debug Then Say 'entering Get_LPARObjNum'

   Parse Arg my_Full_LPAR

   LPARObjNum = ''

   Do i = 1 To CPC.cpcobjnum.NOOFIMAGES
      Do di = 1 To CPCIMAGE.0
         If CPCIMAGE.di.NAME = CPC.cpcobjnum.i.IMAGENAME Then Do
            If CPCIMAGE.di.NAME = my_Full_LPAR Then Do
               LPARObjNum = di
               x = CPC.cpcobjnum.i.ISTATUS
               y = CPC.cpcobjnum.i.IMODE
               Say 'CPC Image:'
               Say Right(di,3)')' Left(CPCIMAGE.di.NAME,20) 'Status =' Left(STATUS.x,20) 'Mode =' MODE.y
               Leave i
            End
         End
      End
   End
   If LPARObjNum = '' Then Call MSG9004

   Return

/***********************************************************************
*
*    Do Action requested by caller
*
***********************************************************************/
Do_Action:

   If Debug Then Say 'entering Do_Action'

   Parse Arg Action Hmc_Profile CPCObjNum LPARObjNum Load_Address Load_Parameter Clear_Indicator Command

   Select
      When Action = 'SEND_OPSYS' | Action = 'SO' Then
         Call Process_Command LPARObjnum HWMCA_CPC_IMAGE_OBJECT 'SEND_OPSYS NONE NONE NONE' Command
      When Action = 'POR' Then
         Call Process_Command CPCobjnum HWMCA_CPC_OBJECT 'ACTIVATE'
      When Action = 'POFF' Then
         Call Process_Command CPCobjnum HWMCA_CPC_OBJECT 'DEACTIVATE'
      When Action = 'ACTIVATE' Then
         Call Process_Command CPCobjnum HWMCA_CPC_IMAGE_OBJECT 'ACTIVATE'
      When Action = 'DEACTIVATE' Then
         Call Process_Command CPCobjnum HWMCA_CPC_IMAGE_OBJECT 'DEACTIVATE'
      When Action = 'IPL' Then
         Call Process_Command LPARObjnum HWMCA_CPC_IMAGE_OBJECT 'ACTIVATE'
      When Action = 'SYSRESET' Then
         Call Process_Command LPARObjnum HWMCA_CPC_IMAGE_OBJECT 'RESETNORMAL'
      When Action = 'LOAD' Then
         Call Process_Command LPARObjnum HWMCA_CPC_IMAGE_OBJECT 'LOAD' Load_Address Load_Parameter Clear_Indicator
      When Action = 'SAD' Then
         Call Process_Command LPARObjnum HWMCA_CPC_IMAGE_OBJECT 'LOAD' Load_Address Load_Parameter 'SAD'
      When Action = 'NOOP' Then
         Return
      Otherwise
         Call MSG9003
   End

   Return


/*** process command routine ***/
Process_Command:

   If Debug Then Say 'entering Process_Command'

   Parse Upper Arg obj_num obj_type command_verb Load_Address Load_Parameter Clear_Indicator opsys_Command

   If \DataType(obj_num,'N') Then Do
      Call MSG2001
      Return
   End

   Select
      When command_verb = 'SEND_OPSYS'
         Then Call Process_OpSys_CMD obj_num command_verb opsys_command
      When command_verb = 'ACTIVATE'
         Then Call Process_Obj_CMD obj_num obj_type Hmc_Profile HWMCA_ACTIVATE_COMMAND HWMCA_ACTIVATE_COMMAND_SUFFIX "NONE NONE NONE"
      When command_verb = 'DEACTIVATE'
         Then Call Process_Obj_CMD obj_num obj_type Hmc_Profile HWMCA_DEACTIVATE_COMMAND HWMCA_DEACTIVATE_COMMAND_SUFFIX "NONE NONE NONE"
      When command_verb = 'RESETNORMAL'
         Then Call Process_Obj_CMD obj_num obj_type Hmc_Profile HWMCA_RESETNORMAL_COMMAND HWMCA_RESETNORMAL_COMMAND_SUFFIX "NONE NONE NONE"
      When command_verb = 'LOAD'
         Then Call Process_Obj_CMD obj_num obj_type Hmc_Profile HWMCA_LOAD_COMMAND HWMCA_LOAD_COMMAND_SUFFIX Load_Address Load_Parameter Clear_Indicator
      Otherwise /* must be a command */
         Call MSG2005
   End  /* select */

   Return


/*** process operating system command ***/
Process_OpSys_CMD:

   If Debug Then Say 'entering Process_OpSys_CMD'

   Parse Arg obj_num command_verb opsys_command

   /****************************************************************************/
   /* Setup and Issue the Command API                                          */
   /****************************************************************************/

   CMD.0 = 2
   CMD.1.TYPE = HWMCA_TYPE_INTEGER
   CMD.1.DATA = HWMCA_FALSE

   /* set command variables */

   CMD.2.TYPE = HWMCA_TYPE_OCTETSTRING
   CMD.2.DATA = opsys_command

   cpcimib = CPCIMAGE.obj_num.ADDR

   If Debug Then Do
      Say "Target MID ID:" cpcimib
      Say "Command text :" CMD.2.DATA
   End

   rc = RxHwmcaCommand(initvar,cpcimib,HWMCA_SEND_OPSYS_COMMAND,'CMD.',API_TIMEOUT)
   If rc \= HWMCA_DE_NO_ERROR  Then Do
      Call MSG2006 rc
      Call SysSleep 5
      Return
   End

   /* wait to see if the command was successful */
   rc = RxHwmcaWaitEvent(initvar,'OUTPUT.',Wait_Time)
   If rc = HWMCA_DE_NO_ERROR Then Do
      varnum = OUTPUT.0
      If Debug Then Do n = 1 To varnum
         Say "OUTPUT TYPE NUM" n
         Say "               " OUTPUT.n.TYPE
         Say "               " OUTPUT.n.DATA
      End
      command_rc = OUTPUT.6.DATA
      Call MSG2007 command_rc opsys_command
   End
   Else Do
      Call MSG2008 rc
   End

   Return


/*** process Hardware commands against the CPC's and CPC images ***/
Process_Obj_CMD:

   If Debug Then Say 'entering Process_Obj_CMD'

   Parse Arg obj_num obj_type Hmc_Profile command_verb cmd_suffix Load_Address Load_Parameter Clear_Indicator

   Select
      When obj_type =   HWMCA_CPC_OBJECT
         Then Do
            obj_mib = CPC.obj_num.ADDR
            obj_name = CPC.obj_num.NAME
            act_prof  = CPC.obj_num.ACTPROF
         End
      When obj_type =   HWMCA_CPC_IMAGE_OBJECT
         Then Do
            obj_mib = CPCIMAGE.obj_num.ADDR
            obj_name = CPCIMAGE.obj_num.NAME
            act_prof = CPCIMAGE.obj_num.ACTPROF
         End
      Otherwise /* must be invalid */
         Call MSG2009
         Return
   End        /* SELECT */

   /* confirm command being issued */
   cmd_name = CMDTYPE.cmd_suffix
   Say Action cmd_name 'for' obj_name

   /*  at this point we need to check if the command was an ACTIVATE, if so we need to check if the
       customer wants to use another activation profile. Therefore we will process this command in
       another routine.  */
   Select
      When command_verb =  HWMCA_ACTIVATE_COMMAND Then
         Call Process_Activate_CMD  obj_num obj_type command_verb cmd_suffix act_prof
      When command_verb =  HWMCA_LOAD_COMMAND Then
         Call Process_Load_CMD  obj_num obj_type command_verb cmd_suffix Load_Address Load_Parameter Clear_Indicator
      Otherwise
         rc = RxHwmcaCommand(initvar,obj_mib,command_verb,,API_TIMEOUT)
         If rc \= HWMCA_DE_NO_ERROR Then Call MSG2010 rc
   End
   If rc \= HWMCA_DE_NO_ERROR Then Do
      Call SysSleep 5
      Return
   End

   /* wait to see if the command was successful */
   rc = RxHwmcaWaitEvent(initvar,'OUTPUT.',Wait_Time)
   If rc = HWMCA_DE_NO_ERROR Then Do
      varnum = OUTPUT.0
      If Debug Then Do n = 1 To varnum
         Say "OUTPUT TYPE NUM" n
         Say "               " OUTPUT.n.TYPE
         Say "               " OUTPUT.n.DATA
      End
      If OUTPUT.2.DATA = HWMCA_EVENT_COMMAND_RESPONSE Then Do
         command_rc = OUTPUT.6.DATA
         Call MSG2011 cmd_name command_rc
      End
   End
   Else Do
      Call MSG2008 rc
   End

   Return

Process_Activate_CMD:

   If Debug Then Say 'entering Process_Activate_CMD'

   Parse Arg obj_num obj_type command_verb cmd_suffix act_prof

   Act_Prof_temp = HMC_Profile
   CMD.0 = 1
   CMD.1.TYPE = HWMCA_TYPE_OCTETSTRING
   CMD.1.DATA = act_prof_temp

   rc = RxHwmcaCommand(initvar,obj_mib,command_verb,'CMD.',API_TIMEOUT)
   If rc \= HWMCA_DE_NO_ERROR Then Call MSG2014 rc
                              Else Call MSG2015

   Return

Process_Load_CMD:

   If Debug Then Say 'entering Process_Load_CMD'

   Parse Arg obj_num obj_type command_verb cmd_suffix Load_Address Load_Parameter Clear_Indicator

   CMD.0 = 5
   CMD.1.TYPE = HWMCA_TYPE_OCTETSTRING
   CMD.1.DATA = Load_Address
   CMD.2.TYPE = HWMCA_TYPE_OCTETSTRING
   CMD.2.DATA = Load_Parameter
   CMD.3.TYPE = HWMCA_TYPE_INTEGER
   If (Clear_Indicator = "YES") Then CMD.3.DATA = HWMCA_TRUE
                                Else CMD.3.DATA = HWMCA_FALSE
   CMD.4.TYPE = HWMCA_TYPE_INTEGER
   CMD.4.DATA = '60'
   CMD.5.TYPE = HWMCA_TYPE_INTEGER
   If (Clear_Indicator = "SAD") Then CMD.5.DATA = HWMCA_TRUE
                                Else CMD.5.DATA = HWMCA_FALSE

   rc = RxHwmcaCommand(initvar,obj_mib,command_verb,'CMD.',API_TIMEOUT)
   If rc \= HWMCA_DE_NO_ERROR Then Call MSG2016 rc
                              Else Call MSG2017

   Return


/***********************************************************************
*
*  Get from HMC routine
*
***********************************************************************/
Get_aGet:

   Parse Arg object attribute

   /* issue the request to the HMC */
   rc = RxHwmcaGet(initvar,object,OUTPUT.,API_TIMEOUT)
   If rc = HWMCA_DE_NO_ERROR Then Return

   /* retry while it's TIMEOUT */
   Do j = 1 To RETRY_TIMEOUT While rc = HWMCA_DE_TIMEOUT
      rc = RxHwmcaGet(initvar,object,OUTPUT.,API_TIMEOUT)
   End

   /* log if still have an error */
   If rc \= HWMCA_DE_NO_ERROR Then Call MSG2003 rc attribute

   Return


/***********************************************************************
*
*  GetNext from HMC routine
*
***********************************************************************/
Get_aGetNext:

   Parse Arg object attribute

   /* issue the request to the HMC */
   rc = RxHwmcaGetNext(initvar,object,OUTPUT.,API_TIMEOUT)
   If rc = HWMCA_DE_NO_ERROR Then Return

   /* retry while it's TIMEOUT */
   Do j = 1 To RETRY_TIMEOUT While rc = HWMCA_DE_TIMEOUT
      rc = RxHwmcaGetNext(initvar,object,OUTPUT.,API_TIMEOUT)
   End

   /* log if still have an error */
   If rc \= HWMCA_DE_NO_ERROR Then Call MSG2003 rc attribute

   Return


/***********************************************************************
*
*  HMC initialization routine
*
***********************************************************************/
Init_HMC:

   /* Load Rexx Utilities   */
   If RxFuncQuery('SysLoadFuncs') Then
      If rxfuncadd( 'SysLoadFuncs', 'RexxUtil', 'SysLoadFuncs' ) Then Do
         Call MSG1001
         Exit '8'
      End
   Call SysLoadFuncs

   /* Load the appropriate Hwmca Rexx interface DLL */
   If RxFuncQuery('RxHwmcaLoadFuncs') Then Do
      If Debug Then Say "os_platform = "os_platform
      If os_platform = 'WindowsNT' Then Do
         Say "HMC function package to be added is "HMCDLL".DLL"
         If rxfuncadd( 'RxHwmcaLoadFuncs', HMCDLL, 'RxHwmcaLoadFuncs' ) Then Do
            Call MSG1002
            Exit '8'
         End
      End
      If os_platform = 'OS/2' Then Do
         If rxfuncadd( 'RxHwmcaLoadFuncs', 'ACTZSNMP', 'RxHwmcaLoadFuncs' ) Then Do
            Call MSG1002
            Exit '8'
         End
      End
   End

   Call RxHwmcaLoadFuncs  /* this function loads all of the other RxHwmca- functions */
   Call RxHwmcaDefineVars /* call to define Hwmca variables   */

   /* initialize IML mode strings for later display    */
   MODE.0  = 'UNKNOWN'
   MODE.1  = 'ESA 390'
   MODE.6  = 'FM'
   MODE.7  = 'FMAE'
   MODE.8  = 'HM'
   MODE.9  = 'HMEA'
   MODE.10 = 'HMEX'
   MODE.11 = 'LPAR'
   MODE.12 = 'ESA 390 TPF'
   MODE.13 = 'Coupling Facility'
   MODE.16 = 'LINUX'

   /* initialize STATUS strings for later use   */
   STATUS.1    = 'Operating'
   STATUS.2    = 'Not Operating'
   STATUS.4    = 'No Power'
   STATUS.8    = 'Not Activated'
   STATUS.16   = 'Exceptions'
   STATUS.32   = 'Status Check'
   STATUS.64   = 'Service'
   STATUS.128  = 'Link Not Active'
   STATUS.256  = 'Power Save'
   STATUS.512  = 'Serious Alert'
   STATUS.1024 = 'Alert'
   STATUS.2048 = 'Env Alert'
   STATUS.4096 = 'Service Req'

   /* initialize EVENT types */
   EVENT.0     = 'Command Response'
   EVENT.1     = 'Message'
   EVENT.2     = 'Status Change'
   EVENT.4     = 'Name Change '
   EVENT.8     = 'Activate Profile Change'
   EVENT.16    = 'Event Create'
   EVENT.32    = 'Event Destroyed'
   EVENT.64    = 'Event Exception State'
   EVENT.128   = 'Event Ended'

   /* initialize Command objects */
   CMDTYPE.1   = 'ACTIVATE'
   CMDTYPE.2   = 'DEACTIVATE'
   CMDTYPE.3   = 'OPSYS'
   CMDTYPE.4   = 'RESETNORMAL'
   CMDTYPE.5   = 'START'
   CMDTYPE.6   = 'STOP'
   CMDTYPE.7   = 'PSWRESTART'
   CMDTYPE.8   = 'INITIALIZE'
   CMDTYPE.9   = 'TERMINATE'
   CMDTYPE.10  = 'IPL'

   line_size = '20'

   /* Initialize HMC interface. Event Mask = just command response events */
   initvar = 'INITBLK.'
   INITBLK.EVENTMASK = HWMCA_DIRECT_INITIALIZE + HWMCA_EVENT_COMMAND_RESPONSE
   INITBLK.RESERVED  = 0
   INITBLK.TARGET    = hmc_host
   INITBLK.COMMUNITY = hmc_community

   If Debug Then Say 'issued RxHwmcaInitialize for INITEVNT.'
   rc = RxHwmcaInitialize(initvar,API_TIMEOUT)
   If rc \= HWMCA_DE_NO_ERROR Then Do
      Call MSG2002 rc
      Call Term_HMC
      Exit '8'
   End

   Return


/***********************************************************************
*
*  All message routines go here
*
***********************************************************************/
MSG1001:
   Say 'Unable to load system function package, Script terminating'
   HMCMsg = 1
   Return
MSG1002:
   Say 'Unable to load HWMCA function package, script terminating'
   HMCMsg = 1
   Return
MSG2001:
   Say 'Invalid data type for object number'
   HMCMsg = 1
   Return
MSG2002:
   Parse Arg rc
   Say 'Hwmca Initialization failure - script terminating rc = ' rc
   HMCMsg = 1
   Exit '8'
MSG2003:
   Parse Arg current_attribute
   Say 'Hwmca Get failure - attribute =' current_attribute
   HMCMsg = 1
   Return
MSG2004:
   Parse Arg current_attribute
   Say 'Unable to build HWMCA Attribute - attribute =' current_attribute
   HMCMsg = 1
   Return
MSG2005:
   Say 'Invalid Command - Please respecify'
   HMCMsg = 1
   Return
MSG2006:
   Parse Arg rc
   Say 'Send OPSYS command failed rc =' rc
   HMCMsg = 1
   Return
MSG2007:
   Parse Arg command_rc opsys_command
   Say 'SEND OPSYS command for command:' opsys_command 'complete. return code =' command_rc
   HMCMsg = 0
   Return
MSG2008:
   Parse Arg rc
   Say 'Waitevent failure - Command - rc =' rc
   HMCMsg = 1
   Return
MSG2009:
   Say 'Invalid object type - must be CPC or CPC Image object'
   HMCMsg = 1
   Return
MSG2010:
   Parse Arg rc
   Say 'Object command failed rc =' rc
   HMCMsg = 1
   Return
MSG2011:
   Parse Arg command_rc cmd_name
   Say cmd_name 'command complete. return code =' command_rc
   /* This is not an error message */
   HMCMsg = 0
   Return
MSG2012:
   Parse Arg current_attribute
   Say '0 variables returned from HwmcaGet - attribute = ' current_attribute
   HMCMsg = 1
   Return
MSG2013:
   Say 'No CPCs Defined - Returning to main process'
   HMCMsg = 1
   Return
MSG2014:
   Parse Arg rc
   Say 'Activate command failed rc =' rc
   HMCMsg = 1
   Return
MSG2015:
   Say 'Activate command successful'
   HMCMsg = 0
   Return
MSG2016:
   Parse Arg rc
   Say 'Load command failed rc =' rc
   HMCMsg = 1
   Return;
MSG2017:
   Say 'Load command successful'
   HMCMsg = 0
   Return
MSG9001:
   Say Action 'is not a valid ACTION type'
   HMCMsg = 1
   Call Parameter_Details
   Return
MSG9002:
   Say  HMC_CPCName 'CPC not found'
   HMCMsg = 1
   Return
MSG9003:
   Say 'Logic error in Do_Action'
   HMCMsg = 1
   Return
MSG9004:
   Say HMC_LPARName 'LPAR in' HMC_CPCName 'CPC not found'
   HMCMsg = 1
   Return
MSG9005:
   Say 'Insufficient number of parameters provided'
   HMCMsg = 1
   Call Parameter_Details
   Return

/***********************************************************************
*
*  Dump the parsed calling arguments
*
***********************************************************************/
Parameter_Details:

   If LogDetails = 1 Then Do
      Say '   HOST:     ' hmc_host
      Say '   Community:' hmc_community
      Say '   CPC:      ' hmc_CPCName
      Say '   LPAR:     ' hmc_LPARNAME
      Say '   Profile:  ' Hmc_Profile
      Say '   Action:   ' Action
      Say '   Command:  ' Command
   End

   Return

/***********************************************************************
*
*  Generic clear screen routine
*
***********************************************************************/
Clear_Screen:

   If Debug Then Return

   If AFR_ENV Then Call AFR_CLS
              Else Call SysCls

   Return

/***********************************************************************
*
*  Display header linea
*
***********************************************************************/
Say_Header:

   text0 = Left('Host:' hmc_host,line_size)
   text1 = Left('Date:' Date("l"),line_size)
   text2 = Left('Time:' Time("n"),line_size)
   Say text0
   Say text1
   Say text2
   Say

   Return


/***********************************************************************
*
*  Exit hmc routine
*
***********************************************************************/
Exit_HMC:

   Say
   Call Say_Header
   text0 = '**********************************************************'
   text1 = '*              Exiting HMC Command Routine               *'
   text2 = '**********************************************************'

   Say
   Say Center(text0,80)
   Say Center(text1,80)
   Say Center(text2,80)

   /* terminate access */
   Call Term_HMC

   Return

/***********************************************************************
*
*  HMC termination routine
*
***********************************************************************/
Term_HMC:

   rc = RxHwmcaTerminate(initvar,API_TIMEOUT)

   Return


/***********************************************************************
*
*  Branch to this point if program halted from AF/REMOTE Client Scripts window
*
***********************************************************************/
Halt:

   result = program_name 'Terminated by Operator'
   Say result
   /* whatever else needs to be done to clean up after a shutdown */
   Call Term_HMC
   Exit '8'


/***********************************************************************
*
*  Branch to this point if syntax error encountered or function call in error
*
***********************************************************************/
Syntax:

   result = program_name 'Terminated for Syntax, Line' SIGL 'Code' RC
   Say result
   /* whatever else needs to be done to clean up after a syntax shutdown */
   Call Term_HMC
   Exit '8'


/***********************************************************************
*
*   test RxHwmcaCommand (change Ry to Rx)
*
***********************************************************************/
RyHwmcaCommand: procedure expose HWMCA_DE_NO_ERROR HMC_Profile

   Parse Arg initx,mibx,verbx,cmdx,timeoutx
   Say 'INITBLK     :' initx
   Say 'OBJ_MIB     :' mibx
   Say 'command_verb:' verbx
   Say 'CMD         :' cmdx
   Say 'API_Timeout :' timeoutx
   Say 'Profile     :' HMC_Profile
   pull Waittolook
   If WaittoLook = '' Then Return HWMCA_DE_NO_ERROR
                      Else Return 'BADRC'

/***********************************************************************
*
*   test RxHwmcaWaitEvent
*
***********************************************************************/
RyHwmcaWaitEvent: procedure expose HWMCA_DE_NO_ERROR

   Parse Arg initx,outputx,timeoutx
   Say 'INITBLK     :' initx
   Say 'OUTPUT      :' mibx
   Say 'API_Timeout :' timeoutx
   pull Waittolook
   If WaittoLook = '' Then Return HWMCA_DE_NO_ERROR
                      Else Return 'BADRC'
