/*************************** REXX *********************************************

   SCRIPT: HMCONS.REX
   October 14, 2005

   PURPOSE:     To demonstrate a REXX script which displays software, hardware
                and event messages from the HMC.

   Inputs:      Arg(1): Host IP Name of the HMC PC you wish to connect to
                Arg(2): Community Name of the AF/Remote Server PC making the connection

   Outputs:     Event Messages.

   Modification History:
     November 7, 1997 - Dan Biskar, miscellaneous changes in support of running
        on a Windows NT platform and using the HWMCAWIN API.
     July 6, 1999 - Dan Biskar, added INITBLK.RESERVED initialization statement.

    February 28, 2001- Jim Collins, changed event mask to:
       HWMCA_DIRECT_INITIALIZE + HWMCA_EVENT_COMMAND_RESPONSE
       Added logic to allow Host Name and Commnity name to be passed as
       startup parameters.   At invocation if the script is started:
       HMCons 100.29.20.80  AFREMOTE, then the request from interactive input
       of the host and community names will be bypassed.

       If the script is started: HMCons
       then the script will solicite input of the host and community names.

       If the script is started: HMCons 100.20.29.89
       then the script will solicite input of the community name.

    February 24, 2004 - Lou Rivas, make clearer what the parameters are.

    February 26, 2004 - Lou Rivas, allow trailing null character in msgdate and msgtime.

    October 14, 2005  - Per HMC team - change to loop (avoid RC=3 too much data).
       Major cleanup resulting from live testing (e.g. zSeries trailing NULL).

 ******************************************************************************/

/* trace 'o' trace is off
   trace 'r' trace (results) is on */
Trace 'o'

/* Debug = 0 debug logging disabled
   Debug = 1 debug logging enabled */
Debug = 0

/* Set some error detection signals        */
Signal On Halt
Signal On Syntax

/* retrieve the operating system platform and the name of the source program */
Parse Source pgm_name
os_platform = SubWord(pgm_name, 1, 1)
program_name = SubStr(pgm_name,LastPos('\',pgm_name)+1)':'
program_version = 'October 14, 2005'

/* see if the AF/Remote function package is loaded and set flag */
AFR_ENV = 1             /* default environement */
If RxFuncQuery('AFR_CLS') Then
   AFR_ENV = 0          /* if not running in AFR then set flag off   */

/* assume using the IBM HMC Windows DLLs */
HMCDLL = 'I'
If HMCDLL = 'C' Then HMCDLL = 'RpRxHMC'
                Else HMCDLL = 'hwmcaorx'

Say "HMC event console start."
Say "Program    : "program_name
Say "Version    : "program_version
Say "Date       : "Date()
Say "Time       : "Time()

/* get host and community name */
Parse Arg hmc_host hmc_community
If hmc_host = '' Then Do
   /* get host and community name */
   Say 'Please enter the host name or IP address of the HMC PC you wish to connect to'
   Parse Pull hmc_host
   End
If hmc_community = '' Then Do
   Say 'Please enter the community name of the AF/REMOTE server connecting to the HMC'
   Parse Pull hmc_community
   End

/* define constant and variable data; also, load needed rexx functions */
Call Init_HMC

/* obtain status */
Call Obtain_Status

Call Clear_screen

text0 = '**********************************************************'
text1 = '* Waiting for an event notification from the HWMCA       *'
text2 = '**********************************************************'
Say
Say Center(text0,80)
Say Center(text1,80)
Say Center(text2,80)

Do Forever

   /* Issue HwmcaWaitEvent API to receive events from the HMC */
   rc = RxHwmcaWaitEvent('INITEVNT.','OUTPUT.',HWMCA_INFINITE_WAIT)
   If rc == HWMCA_DE_NO_ERROR Then Do
      event_mib = Strip(OUTPUT.1.DATA,'Trailing','00'x)
      event_type = Strip(OUTPUT.2.DATA,'Trailing','00'x)
      If Debug Then Do
         Say 'HMCEVENT.MAIN - event loop '
         Say 'items in stem =>' OUTPUT.0
         Do i = 1 To OUTPUT.0
            Say "OUTPUT."i".TYPE="OUTPUT.i.TYPE "OUTPUT."i".DATA="OUTPUT.i.DATA
         End
      End

      /*  call Process_Event routine                                              */
      Call Process_Event
   End
   Else do
      Say 'RXHWMCAWAITEVENT failure.  rc =' rc
   End

End  /* end of Do Forever loop */

Exit


/***********************************************************************
*  Subroutine Section starts here
************************************************************************/

/***********************************************************************
*
*  Obtain status sub-routine
*
***********************************************************************/
Obtain_Status:

   If Debug Then Say 'entering Obtain_Status'

   /* Go get the status and display */
   text0 = '**********************************************************'
   text1 = '* Obtaining Defined CPC and CPC Image status information *'
   text2 = '*                   Please Standby                       *'
   text3 = '**********************************************************'
   Say
   Say Center(text0,80)
   Say Center(text1,80)
   Say Center(text2,80)
   Say Center(text3,80)

   Drop CPC. CPCIMAGE.

   /****************************************************************************/
   /*  Initialize for HMC call. No Event mask is needed                        */
   /****************************************************************************/

   initvar = 'INITBLK.'
   INITBLK.EVENTMASK = HWMCA_DIRECT_INITIALIZE
   INITBLK.RESERVED  = 0
   INITBLK.TARGET    = hmc_host
   INITBLK.COMMUNITY = hmc_community

   rc = RxHwmcaInitialize(initvar,API_TIMEOUT)
   If rc \= HWMCA_DE_NO_ERROR Then Do
      Call MSG2002 rc
      Call Term_HMC
      Exit
   End

   /* get defined cpcs */
   Call Get_CPCs

   /* get defined cpc images */
   Call Get_CPC_Images

   /* thread CPC and CPC Images into arrays  */
   Call Thread_Objects

   /* cleanup this connection */
   Call Term_HMC

   Return


/***********************************************************************
*
*  Obtain CPC names and attributes
*
***********************************************************************/
Get_CPCs:

   If Debug Then Say 'entering Get_CPCs'

   /****************************************************************************/
   /* Ask GetNext to return each SNMP ID for all defined CPC's.                */
   /****************************************************************************/

   CPC.0 = 0
   id = HWMCA_CFG_CPC_ID||'.'||HWMCA_NAME_SUFFIX

   rc = HWMCA_DE_NO_ERROR
   prefix = id
   done = 0
   i = 1

   Do While rc == HWMCA_DE_NO_ERROR & done == 0
      current_attribute = 'Defined CPC Group'
      Call Get_aGetNext id current_attribute
      If rc \= HWMCA_DE_NO_ERROR Then Do
         Call Term_HMC
         Exit
      End

      If prefix == SubStr(OUTPUT.1.DATA,1,Length(prefix)) Then Do
         /*  strip any trailing null */
         id = Strip(OUTPUT.1.DATA,'Trailing','00'x)
         /*  change any '0A'x(line feed) byte to a blank and strip any trailing null */
         nameo = Strip(Translate(OUTPUT.2.DATA,':','0A'x),'Trailing','00'x)
         objid = HWMCA_CFG_CPC_ID||SubStr(id,LastPos('.',id))
         If Debug Then Say "CPC #:"i "has name of:" nameo "and object id of:" objid
         CPC.0 = i

         /* initialize variable attributes */
         CPC.i.ADDR     = objid
         CPC.i.NAME     = ' '
         CPC.i.STATERR  = ' '
         CPC.i.STATUS   = ' '
         CPC.i.ASTATUS  = ' '
         CPC.i.IMLMODE  = 0
         CPC.i.ACTPROF  = ' '
         CPC.i.NOOFIMAGES = 0

         /* obtain attributes for the CPC  */
         Call Get_CPC initvar
         If rc \= HWMCA_DE_NO_ERROR Then Do
            Call Term_HMC
            Exit
         End

         i=i+1
      End /* do */
      Else do
         done = 1
      End /* do */
   End /* do */

   If Debug Then Say "Number of CPCs:" CPC.0
   If CPC.0 = 0 Then Do
      Call MSG2013 current_attribute
      Call Term_HMC
      Exit
   End

   Return


/***********************************************************************
*
*  Obtain CPC object attributes
*
***********************************************************************/
Get_CPC:

   Parse Arg init_var

   /**************************************************************************/
   /* Object attributes for CPCs                                             */
   /**************************************************************************/

   /**************************************************************************/
   /* attribute: Name                                                        */
   /* operation: Get                                                         */
   /* type     : OCTETSTRING                                                 */
   /**************************************************************************/

   current_attribute = 'Name'

   rc = RxHwmcaBuildAttributeId('CPCID',CPC.i.ADDR,HWMCA_NAME_SUFFIX)
   If rc \= HWMCA_DE_NO_ERROR Then Call MSG2004 current_attribute
   Call Get_aGet init_var CPCID current_attribute
   If rc \= HWMCA_DE_NO_ERROR Then Return
   /*  change any '0A'x(line feed) byte to a blank and strip any trailing null */
   CPC.i.NAME = Strip(Translate(OUTPUT.1.DATA,':','0A'x),'Trailing','00'x)
   If Debug Then Say "name built:" CPC.i.NAME

   /**************************************************************************/
   /* attribute: Status error                                                */
   /* operation: Get                                                         */
   /* type     : INTEGER                                                     */
   /**************************************************************************/

   current_attribute = 'Status Error'

   rc = RxHwmcaBuildAttributeId('CPCID',CPC.i.ADDR,HWMCA_STATUS_ERROR_SUFFIX)
   If rc \= HWMCA_DE_NO_ERROR Then Call MSG2004 current_attribute
   Call Get_aGet init_var CPCID current_attribute
   If rc \= HWMCA_DE_NO_ERROR Then Return
   CPC.i.STATERR = OUTPUT.1.DATA
   If Debug Then Say "Staterr built:" CPC.i.STATERR

   /**************************************************************************/
   /* attribute: Status                                                      */
   /* operation: Get                                                         */
   /* type     : INTEGER                                                     */
   /**************************************************************************/

   current_attribute = 'Status'

   rc = RxHwmcaBuildAttributeId('CPCID',CPC.i.ADDR,HWMCA_STATUS_SUFFIX)
   If rc \= HWMCA_DE_NO_ERROR Then Call MSG2004 current_attribute
   Call Get_aGet init_var CPCID current_attribute
   If rc \= HWMCA_DE_NO_ERROR Then Return
   CPC.i.STATUS = OUTPUT.1.DATA
   If Debug Then Say "Status built:" CPC.i.STATUS

   /**************************************************************************/
   /* attribute: Acceptable status                                           */
   /* operation: Get                                                         */
   /* type     : INTEGER                                                     */
   /**************************************************************************/

   current_attribute = 'Acceptable Status'

   rc = RxHwmcaBuildAttributeId('CPCID',CPC.i.ADDR,HWMCA_EXPECTED_STATUS_SUFFIX)
   If rc \= HWMCA_DE_NO_ERROR Then Call MSG2004 current_attribute
   Call Get_aGet init_var CPCID current_attribute
   If rc \= HWMCA_DE_NO_ERROR Then Return
   CPC.i.ASTATUS = OUTPUT.1.DATA
   If Debug Then Say "Astatus built:" CPC.i.ASTATUS

   /**************************************************************************/
   /* attribute: IML mode                                                    */
   /* operation: Get                                                         */
   /* type     : INTEGER                                                     */
   /**************************************************************************/

   current_attribute = 'IML Mode'

   rc = RxHwmcaBuildAttributeId('CPCID',CPC.i.ADDR,HWMCA_IMLMODE_SUFFIX)
   If rc \= HWMCA_DE_NO_ERROR Then Call MSG2004 current_attribute
   Call Get_aGet init_var CPCID current_attribute
   If rc \= HWMCA_DE_NO_ERROR Then Return
   CPC.i.IMLMODE = OUTPUT.1.DATA
   If Debug Then Say "IMLMODE built:" CPC.i.IMLMODE

   /**************************************************************************/
   /* attribute: Activation Profile name                                     */
   /* operation: get                                                         */
   /* type     : OCTETSTRING                                                 */
   /**************************************************************************/

   current_attribute = 'Active Profile'

   rc = RxHwmcaBuildAttributeId('CPCID',CPC.i.ADDR,HWMCA_ACTIVATION_PROFILE_SUFFIX)
   If rc \= HWMCA_DE_NO_ERROR Then Call MSG2004 current_attribute
   Call Get_aGet init_var CPCID current_attribute
   If rc \= HWMCA_DE_NO_ERROR Then Return
   /*  strip any trailing null */
   CPC.i.ACTPROF = Strip(OUTPUT.1.DATA,'Trailing','00'x)
   If Debug Then Say "Actprof built:" CPC.i.ACTPROF

   Return


/***********************************************************************
*
*  Obtain CPC object images
*
***********************************************************************/
Get_CPC_Images:

   If Debug Then Say 'entering Get_CPC_Images'

   /****************************************************************************/
   /* Ask GetNext to return each SNMP ID for all defined CPC images.           */
   /****************************************************************************/

   CPCIMAGE.0 = 0
   id = HWMCA_CPC_IMAGE_ID||'.'||HWMCA_NAME_SUFFIX

   rc = HWMCA_DE_NO_ERROR
   prefix = id
   done = 0
   i = 1

   Do While rc == HWMCA_DE_NO_ERROR & done == 0
      current_attribute = 'Defined CPC Images'
      Call Get_aGetNext id current_attribute
      If rc \= HWMCA_DE_NO_ERROR Then Do
         Call Term_HMC
         Exit
      End

      If prefix == SubStr(OUTPUT.1.DATA,1,Length(prefix)) Then Do
         /*  strip any trailing null */
         id = Strip(OUTPUT.1.DATA,'Trailing','00'x)
         /*  change any '0A'x(line feed) byte to a blank and strip any trailing null */
         nameo = Strip(Translate(OUTPUT.2.DATA,':','0A'x),'Trailing','00'x)
         objid = HWMCA_CPC_IMAGE_ID||SubStr(id,LastPos('.',id))
         If Debug Then Say "Image #:"i "has name of:" nameo "and object id of:" objid
         CPCIMAGE.0 = i

         /* initialize variable attributes */
         CPCIMAGE.i.ADDR    = objid
         CPCIMAGE.i.PNAME   = ' '
         CPCIMAGE.i.OSNAME  = ' '
         CPCIMAGE.i.OSTYPE  = ' '
         CPCIMAGE.i.OSLEVEL = ' '
         CPCIMAGE.i.IMLMODE = 0

         /* obtain attributes for the Image  */
         Call Get_CPC_Image initvar
         If rc \= HWMCA_DE_NO_ERROR Then Do
            Call Term_HMC
            Exit
         End

         i=i+1
      End /* do */
      Else do
         done = 1
      End /* do */
   End /* do */

   If Debug Then Say "Number of CPCs:" CPCIMAGE.0
   If CPCIMAGE.0 = 0 Then Do
      Call MSG2012 current_attribute
      Call Term_HMC
      Exit
   End

   Return


/***********************************************************************
*
*  Obtain CPC object image attributes
*
***********************************************************************/
Get_CPC_Image:

   Parse Arg init_var

   /**************************************************************************/
   /* Object attributes for CPC Image                                        */
   /**************************************************************************/

   /**************************************************************************/
   /* attribute: Name                                                        */
   /* operation: Get                                                         */
   /* type     : OCTETSTRING                                                 */
   /**************************************************************************/

   current_attribute = 'Name'

   rc = RxHwmcaBuildAttributeId('IMAGEID',CPCIMAGE.i.ADDR,HWMCA_NAME_SUFFIX)
   If rc \= HWMCA_DE_NO_ERROR Then Call MSG2004 current_attribute
   Call Get_aGet init_var IMAGEID current_attribute
   If rc \= HWMCA_DE_NO_ERROR Then Return
   /*  change any '0A'x(line feed) byte to a blank and strip any trailing null */
   CPCIMAGE.i.NAME = Strip(Translate(OUTPUT.1.DATA,':','0A'x),'Trailing','00'x)
   If Debug Then Say "name built:" CPCIMAGE.i.NAME

   /**************************************************************************/
   /* attribute: Parent Name                                                 */
   /* operation: Get                                                         */
   /* type     : OCTETSTRING                                                 */
   /**************************************************************************/

   current_attribute = 'Parent Name'

   rc = RxHwmcaBuildAttributeId('IMAGEID',CPCIMAGE.i.ADDR,HWMCA_PARENT_NAME_SUFFIX)
   If rc \= HWMCA_DE_NO_ERROR Then Call MSG2004 current_attribute
   Call Get_aGet init_var IMAGEID current_attribute
   If rc \= HWMCA_DE_NO_ERROR Then Return
   /*  change any '0A'x(line feed) byte to a blank and strip any trailing null */
   CPCIMAGE.i.PNAME = Strip(Translate(OUTPUT.1.DATA,':','0A'x),'Trailing','00'x)
   If Debug Then Say "pname built:" CPCIMAGE.i.PNAME

   /**************************************************************************/
   /* attribute: Status                                                      */
   /* operation: Get                                                         */
   /* type     : INTEGER                                                     */
   /**************************************************************************/

   current_attribute = 'Status'

   rc = RxHwmcaBuildAttributeId('IMAGEID',CPCIMAGE.i.ADDR,HWMCA_STATUS_SUFFIX)
   If rc \= HWMCA_DE_NO_ERROR Then Call MSG2004 current_attribute
   Call Get_aGet init_var IMAGEID current_attribute
   If rc \= HWMCA_DE_NO_ERROR Then Return
   CPCIMAGE.i.STATUS = OUTPUT.1.DATA
   If Debug Then Say "Status built:" CPCIMAGE.i.STATUS

   /**************************************************************************/
   /* attribute: Acceptable status                                           */
   /* operation: Get                                                         */
   /* type     : INTEGER                                                     */
   /**************************************************************************/

   current_attribute = 'Acceptable Status'

   rc = RxHwmcaBuildAttributeId('IMAGEID',CPCIMAGE.i.ADDR,HWMCA_EXPECTED_STATUS_SUFFIX)
   If rc \= HWMCA_DE_NO_ERROR Then Call MSG2004 current_attribute
   Call Get_aGet init_var IMAGEID current_attribute
   If rc \= HWMCA_DE_NO_ERROR Then Return
   CPCIMAGE.i.ASTATUS = OUTPUT.1.DATA
   If Debug Then Say "Astatus built:" CPCIMAGE.i.ASTATUS

   /**************************************************************************/
   /* attribute: Activation Profile                                          */
   /* operation: Get                                                         */
   /* type     : OCTETSTRING                                                 */
   /**************************************************************************/

   current_attribute = 'Activation Profile'

   rc = RxHwmcaBuildAttributeId('IMAGEID',CPCIMAGE.i.ADDR,HWMCA_ACTIVATION_PROFILE_SUFFIX)
   If rc \= HWMCA_DE_NO_ERROR Then Call MSG2004 current_attribute
   Call Get_aGet init_var IMAGEID current_attribute
   If rc \= HWMCA_DE_NO_ERROR Then Return
   /*  change any '0A'x(line feed) byte to a blank and strip any trailing null */
   CPCIMAGE.i.ACTPROF = Strip(Translate(OUTPUT.1.DATA,':','0A'x),'Trailing','00'x)
   If Debug Then Say "ACTPROF built:" CPCIMAGE.i.ACTPROF

   Return


/***********************************************************************
*
*  Thread objects routine
*
***********************************************************************/
Thread_Objects:

   If Debug Then Say 'entering Thread_Objects'

   /* if CPC object in LPAR or Native mode, make sure to thread image */
   Do i = 1 To CPC.0

      /*************************************************************************************************/
      /* If CPC is set to LPAR mode get images for CPC, Cross reference image and CPC name and status  */
      /*************************************************************************************************/

      If (CPC.i.IMLMODE = HWMCA_IML_LPAR_MODE) | (CPC.i.IMLMODE = HWMCA_IML_ESA390_MODE) Then Do
         Do j = 1 To CPCIMAGE.0
            If CPC.i.NAME = CPCIMAGE.j.PNAME Then Do
               CPC.i.NOOFIMAGES = CPC.i.NOOFIMAGES + 1
               imnum = CPC.i.NOOFIMAGES
               CPC.i.imnum.IMAGENAME = CPCIMAGE.j.NAME
               CPC.i.imnum.IMAGENUM  = j
               CPC.i.imnum.ISTATUS   = CPCIMAGE.j.STATUS
               CPC.i.imnum.IMODE     = CPC.i.IMLMODE
               CPCIMAGE.j.PNUMBER    = i
               If Debug Then Say "Threaded:" CPCIMAGE.j.NAME "to" CPC.i.NAME
            End
         End
      End
   End

   Return


/***********************************************************************
*
*  Process event routine
*
***********************************************************************/
Process_Event:

   /* determine which event verb and call selected event routine */
   SELECT
      When event_type = HWMCA_EVENT_COMMAND_RESPONSE
         Then Return                                                    /* don't care */
      When event_type = HWMCA_EVENT_MESSAGE
         Then Call Process_Event_Message
      When event_type = HWMCA_EVENT_STATUS_CHANGE
         Then Call Process_Status_Change
      When event_type = HWMCA_EVENT_NAME_CHANGE
         Then Call Process_Name_Change
      When event_type = HWMCA_EVENT_ACTIVATE_PROF_CHANGE
         Then Call Process_Profile_Change
      When event_type = HWMCA_EVENT_CREATED
         Then Call Process_Created
      When event_type = HWMCA_EVENT_DESTROYED
         Then Call Process_Destroyed
      When event_type = HWMCA_EVENT_EXCEPTION_STATE
         Then Call Process_Exceptions
      When event_type = HWMCA_EVENT_ENDED
         Then Call Process_Ended
      When event_type = HWMCA_EVENT_HARDWARE_MESSAGE_DELETE
         Then Return                                                    /* don't care */
      When event_type = HWMCA_EVENT_ALERT
         Then Call Process_Alert_Event
      When event_type = HWMCA_EVENT_SECURITY_EVENT
         Then Call Process_Security_Event
      Otherwise
         /***********************************************************/
         /* For the purposes of this example, we ignore operating   */
         /* system events as there are too many types to be sent    */
         /***********************************************************/
         Call MSG2014
   End  /* select */

   Return


/***********************************************************************
*
*  Process event message routine
*
***********************************************************************/
Process_Event_Message:

   /*** hmc event message routine - HWMCA_EVENT_MESSAGE ***/
   /*  called when an object has a message to display  */
   msg_type = OUTPUT.4.DATA
   msg_text = Strip(OUTPUT.6.DATA,'Trailing','00'x)

   If Debug Then Do
      Say 'HMCONS.HWMCA_EVENT_MESSAGE - entered'
      Say 'msg type:' msg_type
      Say 'msg text:' msg_text
   End

   Select
      When msg_type = HWMCA_HARDWARE_MESSAGE
         Then Call Process_Hardware_Message

      When msg_type = HWMCA_OPSYS_MESSAGE
         Then Do
            /* these are sometimes empty */
            msgdate = Strip(OUTPUT.10.DATA,'Trailing','00'x)
            msgtime = Strip(OUTPUT.12.DATA,'Trailing','00'x)
            msgsysid = Strip(OUTPUT.20.DATA,'Trailing','00'x)
            /* if no prompt, try for system name */
            If msgsysid = '' Then msgsysid = Strip(OUTPUT.22.DATA,'Trailing','00'x)

            software_msg = ''
            If msgdate \= '' Then software_msg = software_msg || msgdate || ' '
            If msgtime \= '' Then software_msg = software_msg || msgtime || ' '
            If msgsysid \= '' Then software_msg = software_msg || msgsysid || ' '

            /* the message text can come in the form of a MLWTO and is separated with */
            /* carriage control newline character. We have to parse the message text  */
            /* variable.                                                              */
            msgs_remainder = msg_text

            msgs = msgs_remainder
            Do Until (Pos(newline,msgs_remainder) = 0)
               Parse Var msgs msgtext (CR_LF) msgs_remainder
               If msgtext \= '' Then Do
                  Say software_msg || msgtext

                  /* if specified log message to event log  */
                  If LOGMSG Then Do
                     count  = LineOut( EVENTLOG_FILENAME, software_msg || msgtext )
                     count  = LineOut( EVENTLOG_FILENAME )
                  End
               End
               msgs = msgs_remainder
            End        /* do until */

            If msgs \= '' Then Do
               Say software_msg || msgs

               /* if specified log message to event log  */
               If LOGMSG Then Do
                  count  = LineOut( EVENTLOG_FILENAME, software_msg || msgs )
                  count  = LineOut( EVENTLOG_FILENAME )
               End
            End
         End        /* MSG_TYPE = HWMCA_OPSYS_MESSAGE */

      Otherwise
         Say 'Invalid message type received form HwmcaWaitEvent'
   End /* Select */

   Return


/***********************************************************************
*
*  Process event hardware message routine
*
***********************************************************************/
Process_Hardware_Message:

   msgtime = Strip(OUTPUT.10.DATA,'Trailing','00'x)

   If Debug Then Do
      Say 'HMCONS.HWMCA_HARDWARE_MESSAGE - entered'
      Say 'event mib:' event_mib
   End

   /* call Get_Object_Name; it returns origin.name variable */
   Call Get_Object_Name

   /* format message and post it to the console */
   Say msgtime' 'origin.name' 'msg_text

   Return


/***********************************************************************
*
*  Process status change routine
*
***********************************************************************/
Process_Status_Change:

   /*** hmc event routine - HWMCA_EVENT_STATUS_CHANGE ***/
   /*  called when an object has changed status              */
   /*  get name of object and display change status message  */
   event_nstat = OUTPUT.4.DATA
   event_ostat = OUTPUT.6.DATA

   If Debug Then Do
      Say 'HMCONS.HWMCA_EVENT_STATUS_CHANGE - entered'
      Say 'event mib: ' event_mib
      Say 'old status:' event_ostat '-' STATUS.event_ostat
      Say 'new status:' event_nstat '-' STATUS.event_nstat
   End

   /* call Get_Object_Name; it returns origin.name variable */
   Call Get_Object_Name

   /* if we get an UNKNOWN object name try to get the attributes from the HMC and try one more time */
   If origin.name = "UNKNOWN" Then Do
      Call Process_Created
      Call Get_Object_Name
   End

   /* format message and post it to the console */
   Say origin.name" object changing status from: "   STATUS.event_ostat  " to "  STATUS.event_nstat

   Return


/***********************************************************************
*
*  Process name change routine
*
***********************************************************************/
Process_Name_Change:

   /*** hmc event routine - HWMCA_EVENT_NAME_CHANGE ***/
   /*  called when an object has its name changed  */
   event_nname = Strip(OUTPUT.4.DATA,'Trailing','00'x)
   event_oname = Strip(OUTPUT.6.DATA,'Trailing','00'x)

   If Debug Then Do
      Say 'HMCONS.HWMCA_EVENT_NAME_CHANGE - entered'
      Say 'event mib:' event_mib
      Say 'old name: ' event_oname
      Say 'new name: ' event_nname
   End

   /* call Get_Object_Name; it returns origin.name variable */
   Call Get_Object_Name

   /* if we get an UNKNOWN object name try to get the attributes from the HMC and try one more time */
   If origin.name = "UNKNOWN" Then Do
      Call Process_Created
      Call Get_Object_Name
   End

   /* format message and post it to the console */
   Say origin.name"object changing name from: "  event_oname  " to " event_nname

   Return


/***********************************************************************
*
*  Process activation profile change routine
*
***********************************************************************/
Process_Profile_Change:

   /*** hmc profile change routine - HWMCA_EVENT_ACTIVATE_PROF_CHANGE ***/
   /*  called when an object has its profile changed  */
   actp_new = Strip(OUTPUT.4.DATA,'Trailing','00'x)
   actp_old = Strip(OUTPUT.6.DATA,'Trailing','00'x)

   If Debug Then Do
      Say 'HMCONS.HWMCA_EVENT_ACTIVATE_PROF_CHANGE - entered'
      Say 'event mib:  ' event_mib
      Say 'old profile:' actp_old
      Say 'new profile:' actp_new
   End

   /* call Get_Object_Name; it returns origin.name variable */
   Call Get_Object_Name

   /* if we get an UNKNOWN object name try to get the attributes from the HMC and try one more time */
   If origin.name = "UNKNOWN" Then Do
      Call Process_Created
      Call Get_Object_Name
   End

   /* display message     */
   Say origin.name "Activate Profile has changed from: "  actp_old  " to " actp_new

   Return


/***********************************************************************
*
*  Process created routine
*
***********************************************************************/
Process_Created:

   /*** hmc profile change routine - HWMCA_EVENT_CREATED ***/
   /*  called when an object has been created  */
   If Debug Then Do
      Say 'HMCONS.HWMCA_EVENT_CREATED - entered'
      Say 'event mib:' event_mib
   End

   /* call Get_Object_Name; it returns origin.name variable */
   Call Get_Object_Name

   /* if we get an UNKNOWN object name try to get the attributes from the HMC and try one more time */
   If origin.name = "UNKNOWN" Then Do
      Call Create_Object
      Call Get_Object_Name
   End

   /* display message */
   Say origin.name " object has been created "

   Return


/***********************************************************************
*
*  Process destroyed routine
*
***********************************************************************/
Process_Destroyed:

   /*** hmc profile change routine - HWMCA_EVENT_DESTROYED ***/
   /*  called when an object has been created  */
   If Debug Then Do
      Say 'HMCONS.HWMCA_EVENT_DESTROYED - entered'
      Say 'event mib:' event_mib
   End

   /* call Get_Object_Name; it returns origin.name variable */
   Call Get_Object_Name

   /* issue message */
   Say origin.name " object has been destroyed "

   /* notice that we really don't delete anything from CPC. or CPCIMAGE. */
   Return


/***********************************************************************
*
*  Process exception routine
*
***********************************************************************/
Process_Exceptions:

   /*** hmc profile change routine - HWMCA_EVENT_EXCEPTION_STATE ***/
   /*  called when an object has entered or left an exception state  */
   state = OUTPUT.4.DATA
   cstatus = OUTPUT.6.DATA

   If Debug Then Do
      Say 'HMCONS.HWMCA_EVENT_EXCEPTION_STATE - entered'
      Say 'event mib:' event_mib
      Say 'enter?:' state
      Say 'status:' cstatus
   End

   /* call Get_Object_Name; it returns origin.name variable */
   Call Get_Object_Name

   /* if we get an UNKNOWN object name try to get the attributes from the HMC and try one more time */
   If origin.name = "UNKNOWN" Then Do
      Call Process_Created
      Call Get_Object_Name
   End

   origin_number = origin.number

   If state = HWMCA_TRUE Then Do
      If origin.type = HWMCA_CPC_OBJECT Then CPC.origin_number.STATERR = HWMCA_TRUE
      If origin.type = HWMCA_CPC_IMAGE_OBJECT Then CPCIMAGE.origin_number.STATERR = HWMCA_TRUE
      state_text = 'Entering into Exception State. Current Status: '
   End

   If state = HWMCA_FALSE Then Do
      If origin.type = HWMCA_CPC_OBJECT Then CPC.origin_number.STATERR = HWMCA_FALSE
      If origin.type = HWMCA_CPC_IMAGE_OBJECT Then CPCIMAGE.origin_number.STATERR = HWMCA_FALSE
      state_text = 'Leaving Exception State. Current Status: '
   End

   Say origin.name' 'state_text||STATUS.cstatus

Return


/***********************************************************************
*
*  Process ended routine
*
***********************************************************************/
Process_Ended:

   /*** hmc event routine - HWMCA_EVENT_ENDED ***/
   If Debug Then Do
      Say 'HMCONS.HWMCA_EVENT_ENDED - entered'
   End

   Call Clear_screen

   msg_text = 'WARNING: The Hardware Management Console Application is ending. HOST = ' hmc_host
   e = d2c(27)||"[7;31;44m"
   Say e msg_text
   Say d2c(27)||"[7;30;46m"

   Return


/***********************************************************************
*
*  Process alert event routine
*
***********************************************************************/
Process_Alert_Event:

   /*** hmc alert event routine - HWMCA_EVENT_ALERT ***/
   /*  called when an object has an alert to display  */
   If Debug Then Do
      Say 'HMCONS.HWMCA_EVENT_ALERT - entered'
      Say 'event mib ' event_mib
   End

   msg_text = Strip(OUTPUT.4.DATA,'Trailing','00'x)
   msgsysid = Strip(OUTPUT.6.DATA,'Trailing','00'x)

   /* format message and post it to the console */
   Say msgsysid msg_text

   Return


/***********************************************************************
*
*  Process security event routine
*
***********************************************************************/
Process_Security_Event:

   /*** hmc security event routine - HWMCA_EVENT_SECURITY_EVENT ***/
   /*  called when an object has a security message to display  */
   If Debug Then Do
      Say 'HMCONS.HWMCA_EVENT_SECURITY_EVENT - entered'
      Say 'event mib ' event_mib
   End

   msg_text = Strip(OUTPUT.6.DATA,'Trailing','00'x)
   msgtime = Strip(OUTPUT.4.DATA,'Trailing','00'x)
   msgsysid = Strip(OUTPUT.8.DATA,'Trailing','00'x)

   /* format message and post it to the console */
   Say msgtime msgsysid msg_text

   Return


/***********************************************************************
*
*  Get object name routine
*
***********************************************************************/
Get_Object_Name:

   /****************************************************************************/
   /* Get name of object                                                       */
   /* if get fails the object name will be 'UNKNOWN'                           */
   /*      input : Host name, Community name, MIB name                         */
   /*      output: origin_name variable containing object name                 */
   /****************************************************************************/

   origin.name = 'UNKNOWN'

   /* first check CPCs */
   Do j = 1 To CPC.0

      If CPC.j.ADDR = event_mib Then Do
         origin.name = CPC.j.NAME
         origin.number = j
         origin.record = CPC.j.RECID
         origin.type  = HWMCA_CPC_OBJECT
         If Debug Then Do
            Say 'origin_name = ' origin.name
            Say 'origin_type = ' origin.type
            Say 'origin_number=' origin.number
         End

         If Debug Then Say 'origin_name = ' origin.name
         Return origin.
      End
   End

   /* next check cpcimage names */
   Do j = 1 To CPCIMAGE.0

      If CPCIMAGE.j.ADDR = event_mib Then Do
         origin.name = CPCIMAGE.j.NAME
         origin.number = j
         origin.record = CPCIMAGE.j.RECID
         origin.type  = HWMCA_CPC_IMAGE_OBJECT
         If Debug Then Do
            Say 'origin_name = ' origin.name
            Say 'origin_type = ' origin.type
            Say 'origin_number=' origin.number
         End

         If Debug Then Say 'origin_name = ' origin.name
         Return origin.
      End
   End

   /* if we can't find the name then */
   If Debug Then Say 'origin_name = ' origin.name
   Return origin.


/***********************************************************************
*
*  Create object name routine
*
***********************************************************************/
Create_Object:

   /* Need a unique variable name so we are using a cmd counter among cmd threads */
   hmc_handle_number = hmc_handle_number + 1
   intnum = hmc_handle_number
   varid = create||intnum
   initvarc = CREOBJ'.'varid'.'
   CREOBJ.varid.TARGET = hmc_host
   CREOBJ.varid.COMMUNITY = hmc_community

   If Debug Then Do
      Say 'obj host: ' hmc_host
      Say 'obj mib : ' event_mib
      Say 'host var: ' CREOBJ.varid.TARGET
      Say 'initvar : ' initvarc
   End

   /****************************************************************************/
   /*  Initialize for HMC call. No Event mask is needed                        */
   /****************************************************************************/

   If Debug Then Say 'HMCONS_event_created_object issued RxHwmcaInitialize'
   rc = RxHwmcaInitialize(initvarc,API_TIMEOUT)
   If rc \= HWMCA_DE_NO_ERROR  Then Do
      Say 'HMCONS_event_created_object issued RxHwmcaInitialize failure rc=' rc
   End

   /**************************************************************************/
   /* Get Object attributes Name and Object type of created MIB              */
   /**************************************************************************/

   /**************************************************************************/
   /* attribute: Object Type                                                 */
   /* operation: Get                                                         */
   /* type     : OCTETSTRING                                                 */
   /**************************************************************************/

   current_attribute = 'Object Type'

   rc = RxHwmcaBuildAttributeId('IMAGEID',event_mib,HWMCA_OBJECT_TYPE_SUFFIX)
   If rc \= HWMCA_DE_NO_ERROR Then Call MSG2004 current_attribute
   Call Get_aGet initvarc IMAGEID current_attribute
   If rc == HWMCA_DE_NO_ERROR Then Do
      create_object_type = Strip(OUTPUT.1.DATA,'Trailing','00'x)
      Say "Object type built:" create_object_objtype
   End


   /**************************************************************************/
   /* attribute: Name                                                        */
   /* operation: Get                                                         */
   /* type     : OCTETSTRING                                                 */
   /**************************************************************************/

   current_attribute = 'Name'

   rc = RxHwmcaBuildAttributeId('IMAGEID',event_mib,HWMCA_NAME_SUFFIX)
   If rc \= HWMCA_DE_NO_ERROR Then Call MSG2004 current_attribute
   Call Get_aGet initvarc IMAGEID current_attribute
   If rc == HWMCA_DE_NO_ERROR Then Do
      /*  change any '0A'x(line feed) byte to a blank and strip any trailing null */
      create_object_name = Strip(Translate(OUTPUT.1.DATA,':','0A'x),'Trailing','00'x)
      Say "Name built:" create_object_name
   End

   /**************************************************************************/
   /* if it is a cpc image build the cpc image attribute variables           */
   /**************************************************************************/
   If create_object_type = HWMCA_CPC_IMAGE_OBJECT Then Do
      /* add one to the cpc images */
      CPCIMAGE.0 = CPCIMAGE.0 + 1
      i = CPCIMAGE.0

      /* initialize variable attributes */
      CPCIMAGE.i.ADDR     = event_mib
      CPCIMAGE.i.PNAME    = ' '
      CPCIMAGE.i.OSNAME   = ' '
      CPCIMAGE.i.OSTYPE   = ' '
      CPCIMAGE.i.OSLEVEL  = ' '
      CPCIMAGE.i.IMLMODE  = 0

      If Debug Then Do
         Say "CPC Image number:" i
         Say "CPC MIB ID      :" CPCIMAGE.i.ADDR
         Say "init var        :" initvarc
         Say "object name     :" create_object_name
      End

      /* obtain attributes for the Image  */
      Call Get_CPC_Image initvarc
   End

   /**************************************************************************/
   /* if it is a cpc build the cpc attribute variables                       */
   /**************************************************************************/
   If create_object_type = HWMCA_CPC_OBJECT Then Do
      /* add one to the cpcs */
      CPC.0 = CPC.0 + 1
      i = CPC.0

      /* initialize variable attributes */
      CPC.i.ADDR     = event_mib
      CPC.i.NAME     = ' '
      CPC.i.STATERR  = ' '
      CPC.i.STATUS   = ' '
      CPC.i.ASTATUS  = ' '
      CPC.i.IMLMODE  = 0
      CPC.i.ACTPROF  = ' '
      CPC.i.NOOFIMAGES = 0

      If Debug Then Do
         Say "CPC number:" i
         Say "CPC MIB ID:" CPC.i.ADDR
         Say "init var  :" initvarc
      End

      /* obtain attributes for the CPC  */
      Call Get_CPC initvarc
   End

   rc = RxHwmcaTerminate(initvarc,API_TIMEOUT)

   Return


/***********************************************************************
*
*  Get from HMC routine
*
***********************************************************************/
Get_aGet:

   Parse Arg init_var object attribute

   /* issue the request to the HMC */
   rc = RxHwmcaGet(init_var,object,OUTPUT.,API_TIMEOUT)
   If rc = HWMCA_DE_NO_ERROR Then Return

   /* retry while it's TIMEOUT */
   Do j = 1 To RETRY_TIMEOUT While rc = HWMCA_DE_TIMEOUT
      rc = RxHwmcaGet(init_var,object,OUTPUT.,API_TIMEOUT)
   End

   /* log if still have an error */
   If rc \= HWMCA_DE_NO_ERROR Then Call MSG2003 rc attribute

   Return


/***********************************************************************
*
*  GetNext from HMC routine
*
***********************************************************************/
Get_aGetNext:

   Parse Arg object attribute

   /* issue the request to the HMC */
   rc = RxHwmcaGetNext(initvar,object,OUTPUT.,API_TIMEOUT)
   If rc = HWMCA_DE_NO_ERROR Then Return

   /* retry while it's TIMEOUT */
   Do j = 1 To RETRY_TIMEOUT While rc = HWMCA_DE_TIMEOUT
      rc = RxHwmcaGetNext(initvar,object,OUTPUT.,API_TIMEOUT)
   End

   /* log if still have an error */
   If rc \= HWMCA_DE_NO_ERROR Then Call MSG2003 rc attribute

   Return


/***********************************************************************
*
*  HMC initialization routine
*
***********************************************************************/
Init_HMC:
   /* Load Rexx Utilities   */
   If RxFuncQuery('SysLoadFuncs') Then
      If rxfuncadd( 'SysLoadFuncs', 'RexxUtil', 'SysLoadFuncs' ) Then Do
         Call MSG1001
         Exit
      End
   Call SysLoadFuncs

   /* Load Hwmca Rexx interface function dynalink   */
   If RxFuncQuery('RxHwmcaLoadFuncs') Then Do
      If Debug Then Say "os_platform = "os_platform
      If os_platform = 'WindowsNT' Then Do
         Say "Using     : " HMCDLL'.DLL'
         If rxfuncadd( 'RxHwmcaLoadFuncs', HMCDLL, 'RxHwmcaLoadFuncs' ) Then Do
            Call MSG1002
            Exit
         End
      End
      If os_platform = 'OS/2' Then Do
         If rxfuncadd( 'RxHwmcaLoadFuncs', 'ACTZSNMP', 'RxHwmcaLoadFuncs' ) Then Do
            Call MSG1002
            Exit
         End
      End
   End

   Call RxHwmcaLoadFuncs  /* load all of the other RxHwmca- functions */
   Call RxHwmcaDefineVars /* Define the HWMCA_ variables so they're available to this Rexx script */

   /* initialize IML mode strings for later display    */
   MODE.0  = 'UNKNOWN'
   MODE.1  = 'ESA 390'
   MODE.6  = 'FM'
   MODE.7  = 'FMAE'
   MODE.8  = 'HM'
   MODE.9  = 'HMEA'
   MODE.10 = 'HMEX'
   MODE.11 = 'LPAR'
   MODE.12 = 'ESA 390 TPF'
   MODE.13 = 'Coupling Facility'
   MODE.16 = 'LINUX'

   /* initialize STATUS strings for later use   */
   STATUS.1    = 'Operating'
   STATUS.2    = 'Not Operating'
   STATUS.4    = 'No Power'
   STATUS.8    = 'Not Activated'
   STATUS.16   = 'Exceptions'
   STATUS.32   = 'Status Check'
   STATUS.64   = 'Service'
   STATUS.128  = 'Link Not Active'
   STATUS.256  = 'Power Save'
   STATUS.512  = 'Serious Alert'
   STATUS.1024 = 'Alert'
   STATUS.2048 = 'Env Alert'
   STATUS.4096 = 'Service Req'

   /* initialize EVENT types */
   EVENT.0     = 'Command Response'
   EVENT.1     = 'Message'
   EVENT.2     = 'Status Change'
   EVENT.4     = 'Name Change '
   EVENT.8     = 'Activate Profile Change'
   EVENT.16    = 'Event Create'
   EVENT.32    = 'Event Destroyed'
   EVENT.64    = 'Event Exception State'
   EVENT.128   = 'Event Ended'

   /* initialize Command objects */
   CMDTYPE.1   = 'ACTIVATE'
   CMDTYPE.2   = 'DEACTIVATE'
   CMDTYPE.3   = 'OPSYS'
   CMDTYPE.4   = 'RESETNORMAL'
   CMDTYPE.5   = 'START'
   CMDTYPE.6   = 'STOP'
   CMDTYPE.7   = 'PSWRESTART'

   hmc_handle_number  =  0          /* command instance number */
   hmc                =  hmc_host
   varhmchno          = 'hmc_handle_number'

   CR_LF   = '0d0a'x
   newline = '0A'x
   cr      = '0D'x
   null    = '00'x

   API_TIMEOUT                = 20000                              /* API Timeout value in ms.              */
   RETRY_TIMEOUT              = 3                                  /* Number of times to retry a timeout    */
   EVENTLOG_FILENAME          = "..\logs\AFRHMC.LOG"               /* event log filename                    */
   LOGMSG                     = 1                                  /* 1 = log messages to file; 0 = don't   */

   /* Initialize HMC interface. Event Mask = all events */
   initvar = 'INITEVNT.'
   INITEVNT.EVENTMASK = HWMCA_DIRECT_INITIALIZE + HWMCA_EVENT_ALL_EVENTS
   INITEVNT.RESERVED  = 0
   INITEVNT.TARGET    = hmc_host
   INITEVNT.COMMUNITY = hmc_community

   If Debug Then Say 'issued RxHwmcaInitialize for INITEVNT.'
   rc = RxHwmcaInitialize(initvar,API_TIMEOUT)
   If rc \= HWMCA_DE_NO_ERROR Then Do
      Call MSG2002 rc
      Call Term_HMC
      Exit
   End

   Return


/***********************************************************************
*
*  All message routines go here
*
***********************************************************************/
MSG1001:
   Say 'Unable to load system function package, script terminating'
   Return
MSG1002:
   Say 'Unable to load HWMCA function package, script terminating'
   Return
MSG2001:
   Say 'Invalid data type for object number'
   Return
MSG2002:
   Parse Arg rc
   Say 'Hwmca Initialization failure - script terminating rc =' rc
   Exit
MSG2003:
   Parse Arg rc current_attribute
   Say 'Hwmca Get failure - rc =' rc '- attribute =' current_attribute
   Return
MSG2004:
   Parse Arg current_attribute
   Say 'Unable to build HWMCA Attribute - attribute =' current_attribute
   Return
MSG2005:
   Say 'Invalid Command - Please respecify'
   Return
MSG2006:
   Parse Arg rc
   Say 'SEND OPSYS command failed rc =' rc
   Return
MSG2007:
   Parse Arg command_rc opsys_command
   Say 'SEND OPSYS command for command:' opsys_command 'complete. Return code =' command_rc
   Return
MSG2008:
   Parse Arg rc
   Say 'Waitevent failure - Command - rc =' rc
   Return
MSG2009:
   Say 'Invalid object type - must be CPC or CPC Image object'
   Return
MSG2010:
   Parse Arg rc
   Say 'Object command failed rc =' rc
   Return
MSG2011:
   Parse Arg command_rc cmd_name
   Say cmd_name 'command complete. Return code =' command_rc
   Return
MSG2012:
   Parse Arg current_attribute
   Say '0 variables returned from HwmcaGet - attribute =' current_attribute
   Return
MSG2013:
   Say 'No CPCs Defined - Returning to main process'
   Return
MSG2014:
   Say 'Invalid event type : 'event_type ||' received from HwmcaWaitEvent API call. '
   Say 'OUTPUT.1.DATA = ' event_mib '(event_mib)'
   Say 'OUTPUT.2.DATA = ' event_type '(event_type)'
   Say 'OUTPUT.3.DATA = ' OUTPUT.3.DATA
   Say 'OUTPUT.4.DATA = ' OUTPUT.4.DATA
   Say 'OUTPUT.5.DATA = ' OUTPUT.5.DATA
   Say 'OUTPUT.6.DATA = ' OUTPUT.6.DATA
   Return


/***********************************************************************
*
*  Generic clear screen routine
*
***********************************************************************/
Clear_screen:

   If Debug Then Return

   If AFR_ENV Then Call AFR_CLS
              Else Call SysCls
   Return


/***********************************************************************
*
*  HMC termination routine
*
***********************************************************************/
Term_HMC:

   rc = RxHwmcaTerminate(initvar,API_TIMEOUT)

   Return


/***********************************************************************
*
*  Branch to this point if program halted from AF/REMOTE Client Scripts window
*
***********************************************************************/
Halt:

   result = program_name 'Terminated by Operator'
   Say result
   /* whatever else needs to be done to clean up after a shutdown */
   Call Term_HMC
   Exit 'N'


/***********************************************************************
*
*  Branch to this point if syntax error encountered or function call in error
*
***********************************************************************/
Syntax:

   result = program_name 'Terminated for Syntax, Line' SIGL 'Code' RC
   Say result
   /* whatever else needs to be done to clean up after a syntax shutdown */
   Call Term_HMC
   Exit 'N'
