/*************************** REXX *********************************************

   SCRIPT: HMCSTAT.REX
   October 14, 2005

   PURPOSE:     To display status of defined CPC and CPC Image objects.
                To issue commands against objects and obtain command responses.

   Inputs:      Arg(1): Host IP Name of the HMC PC you wish to connect to
                Arg(2): Community Name of the AF/Remote Server PC making the connection

   Outputs:     Status Display.

   Modification History:
    August 28, 1996  - Jim Collins, removed reference to uninitialized variable
                       varname.  Changed to INITBLK.
    November 7, 1997 - Dan Biskar, miscellaneous changes in support of running
                       on a Windows NT platform and using the HWMCAWIN API.
    March 17, 1998  - Jim Collins, added 'LOAD' capability with parameters.
      if 'Load' is requested, the program prompts for the paramter values.
                      Paramters are:
                          Load Address:
                          Load Parameter:
                          Clear Indicator:
                             Clear indicator may be:
                                YES for Load Clear
                                NO  for No Load Clear
                                SAD for Stand Alone Dump

    June 30, 1999 - Steve Berry, added return codes to MSG2008 messages (IM619213)
    July 6, 1999 - Dan Biskar, added INITBLK.RESERVED initialization statement.

    March 21, 2000 - Jim Collins, centralized RxMcaInitialize() and
                      RxMcaTerminate() funcions.

    February 28, 2001- Jim Collins, changed event mask to:
       HWMCA_DIRECT_INITIALIZE + HWMCA_EVENT_COMMAND_RESPONSE
       Added logic to allow Host Name and Commnity name to be passed as
       startup parameters.   At invocation if the script is started:
       HMCStat 100.29.20.80  AFREMOTE, then the request from interactive input
       of the host and community names will be bypassed.

       If the script is started: HMCStat
       then the script will solicite input of the host and community names.

       If the script is started: HMCStat 100.20.29.89
       then the script will solicite input of the community name.

    February 24, 2004 - Lou Rivas, make clearer what the parameters are.

    March 10, 2004 - Lou Rivas, fix NULL in CPC.n.NAME (zSeries incompatibility)

    August 5, 2005 - Lou Rivas, retry rc=19 a couple of times and reduce timeout

    October 14, 2005  - Per HMC team - change to loop (avoid RC=3 too much data).
       Major cleanup resulting from live testing (e.g. zSeries trailing NULL).

 ******************************************************************************/

/* trace 'o' trace is off
   trace 'r' trace (results) is on */
Trace 'o'

/* Debug = 0 debug logging disabled
   Debug = 1 debug logging enabled */
Debug = 0

/* set the timeout value, in milliseconds, to be used in all API calls */
API_TIMEOUT = 20000

/* number of times to retry a get if it times out */
RETRY_TIMEOUT = 3

/* Set some error detection signals        */
Signal On Halt
Signal On Syntax

/* retrieve the operating system platform and the name of the source program */
Parse Source pgm_name
os_platform = Subword(pgm_name, 1, 1)
program_name = Substr(pgm_name,LastPos('\',pgm_name)+1)':'
program_version = 'October 14, 2005'

/* see if the AF/Remote function package is loaded and set flag */
AFR_ENV = 1             /* default environement */
If RxFuncQuery('AFR_CLS') Then
   AFR_ENV = 0          /* if not running in AFR then set flag off   */

/* assume using the IBM HMC Windows DLLs */
HMCDLL = 'I'
If HMCDLL = 'C' Then HMCDLL = 'RpRxHMC'
                Else HMCDLL = 'hwmcaorx'

Say "Program    : "program_name
Say "Version    : "program_version
Say "Date       : "Date()
Say "Time       : "Time()

/* get host and community name */
Parse Arg hmc_host hmc_community
If hmc_host = '' Then Do
   /* get host and community name */
   Say 'Please enter the host name or IP address of the HMC PC you wish to connect to'
   Parse Pull hmc_host
End

If hmc_community = '' Then Do
   Say 'Please enter the community name of the AF/REMOTE server connecting to the HMC'
   Parse Pull hmc_community
End

/* define constant and variable data; also, load needed rexx functions */
Call Init_HMC

/* obtain current hmc status */
Call Obtain_Status

/* display CPC status - note that Show_CPCs EXITs instead of RETURNs */
Call Show_CPCs

/***********************************************************************
*  Subroutine Section starts here
************************************************************************/

/***********************************************************************
*
*  Obtain status sub-routine
*
***********************************************************************/
Obtain_Status:

   If Debug Then Say 'entering Obtain_Status'

   /* Go get the status and display */
   Call Clear_Screen
   Call Say_header
   text0 = '**********************************************************'
   text1 = '* Obtaining Defined CPC and CPC Image status information *'
   text2 = '*                   Please Standby                       *'
   text3 = '**********************************************************'
   Say
   Say Center(text0,80)
   Say Center(text1,80)
   Say Center(text2,80)
   Say Center(text3,80)

   Drop CPC. CPCIMAGE.

   /* get defined cpcs */
   Call Get_CPCs

   /* get defined cpc images */
   Call Get_CPC_Images

   /* thread CPC and CPC Images into arrays  */
   Call Thread_Objects

   Return


/***********************************************************************
*
*  Obtain CPC names and attributes
*
***********************************************************************/
Get_CPCs:

   If Debug Then Say 'entering Get_CPCs'

   /****************************************************************************/
   /* Ask GetNext to return each SNMP ID for all defined CPC's.                */
   /****************************************************************************/

   CPC.0 = 0
   id = HWMCA_CFG_CPC_ID||'.'||HWMCA_NAME_SUFFIX

   rc = HWMCA_DE_NO_ERROR
   prefix = id
   done = 0
   i = 1

   Do While rc == HWMCA_DE_NO_ERROR & done == 0
      current_attribute = 'Defined CPC Group'
      Call Get_aGetNext id current_attribute
      If rc \= HWMCA_DE_NO_ERROR Then Do
         Call Term_HMC
         Exit 'N'
      End

      If prefix == Substr(OUTPUT.1.DATA,1,Length(prefix)) Then Do
         /*  strip any trailing null */
         id = Strip(OUTPUT.1.DATA,'Trailing','00'x)
         /*  change any '0A'x(line feed) byte to a blank and strip any trailing null */
         nameo = Strip(Translate(OUTPUT.2.DATA,':','0A'x),'Trailing','00'x)
         objid = HWMCA_CFG_CPC_ID||Substr(id,LastPos('.',id))
         If Debug Then Say "CPC #:"i "has name of:" nameo "and object id of:" objid
         CPC.0 = i

         /* initialize variable attributes */
         CPC.i.ADDR     = objid
         CPC.i.NAME     = ' '
         CPC.i.STATERR  = ' '
         CPC.i.STATUS   = ' '
         CPC.i.ASTATUS  = ' '
         CPC.i.IMLMODE  = 0
         CPC.i.ACTPROF  = ' '
         CPC.i.NOOFIMAGES = 0

         /* obtain attributes for the CPC  */
         Call Get_CPC
         If rc \= HWMCA_DE_NO_ERROR Then Do
            Call Term_HMC
            Exit 'N'
         End

         i=i+1
      End /* do */
      Else Do
         done = 1
      End /* do */
   End /* do */

   If Debug Then Say "Number of CPCs:" CPC.0
   If CPC.0 = 0 Then Do
      Call MSG2013 current_attribute
      Call Term_HMC
      Exit 'N'
   End

   Return


/***********************************************************************
*
*  Obtain CPC object attributes
*
***********************************************************************/
Get_CPC:

   /**************************************************************************/
   /* Object attributes for CPCs                                             */
   /**************************************************************************/

   /**************************************************************************/
   /* attribute: Name                                                        */
   /* operation: Get                                                         */
   /* type     : OCTETSTRING                                                 */
   /**************************************************************************/

   current_attribute = 'Name'

   rc = RxHwmcaBuildAttributeId('CPCID',CPC.i.ADDR,HWMCA_NAME_SUFFIX)
   If rc \= HWMCA_DE_NO_ERROR Then Call MSG2004 current_attribute
   Call Get_aGet CPCID current_attribute
   If rc \= HWMCA_DE_NO_ERROR Then Return
   /*  change any '0A'x(line feed) byte to a blank and strip any trailing null */
   CPC.i.NAME = Strip(Translate(OUTPUT.1.DATA,':','0A'x),'Trailing','00'x)
   If Debug Then Say "name built:" CPC.i.NAME

   /**************************************************************************/
   /* attribute: Status error                                                */
   /* operation: Get                                                         */
   /* type     : INTEGER                                                     */
   /**************************************************************************/

   current_attribute = 'Status Error'

   rc = RxHwmcaBuildAttributeId('CPCID',CPC.i.ADDR,HWMCA_STATUS_ERROR_SUFFIX)
   If rc \= HWMCA_DE_NO_ERROR Then Call MSG2004 current_attribute
   Call Get_aGet CPCID current_attribute
   If rc \= HWMCA_DE_NO_ERROR Then Return
   CPC.i.STATERR = OUTPUT.1.DATA
   If Debug Then Say "Staterr built:" CPC.i.STATERR

   /**************************************************************************/
   /* attribute: Status                                                      */
   /* operation: Get                                                         */
   /* type     : INTEGER                                                     */
   /**************************************************************************/

   current_attribute = 'Status'

   rc = RxHwmcaBuildAttributeId('CPCID',CPC.i.ADDR,HWMCA_STATUS_SUFFIX)
   If rc \= HWMCA_DE_NO_ERROR Then Call MSG2004 current_attribute
   Call Get_aGet CPCID current_attribute
   If rc \= HWMCA_DE_NO_ERROR Then Return
   CPC.i.STATUS = OUTPUT.1.DATA
   If Debug Then Say "Status built:" CPC.i.STATUS

   /**************************************************************************/
   /* attribute: Acceptable status                                           */
   /* operation: Get                                                         */
   /* type     : INTEGER                                                     */
   /**************************************************************************/

   current_attribute = 'Acceptable Status'

   rc = RxHwmcaBuildAttributeId('CPCID',CPC.i.ADDR,HWMCA_EXPECTED_STATUS_SUFFIX)
   If rc \= HWMCA_DE_NO_ERROR Then Call MSG2004 current_attribute
   Call Get_aGet CPCID current_attribute
   If rc \= HWMCA_DE_NO_ERROR Then Return
   CPC.i.ASTATUS = OUTPUT.1.DATA
   If Debug Then Say "Astatus built:" CPC.i.ASTATUS

   /**************************************************************************/
   /* attribute: IML mode                                                    */
   /* operation: Get                                                         */
   /* type     : INTEGER                                                     */
   /**************************************************************************/

   current_attribute = 'IML Mode'

   rc = RxHwmcaBuildAttributeId('CPCID',CPC.i.ADDR,HWMCA_IMLMODE_SUFFIX)
   If rc \= HWMCA_DE_NO_ERROR Then Call MSG2004 current_attribute
   Call Get_aGet CPCID current_attribute
   If rc \= HWMCA_DE_NO_ERROR Then Return
   CPC.i.IMLMODE = OUTPUT.1.DATA
   If Debug Then Say "IMLMODE built:" CPC.i.IMLMODE

   /**************************************************************************/
   /* attribute: Activation Profile name                                     */
   /* operation: get                                                         */
   /* type     : OCTETSTRING                                                 */
   /**************************************************************************/

   current_attribute = 'Active Profile'

   rc = RxHwmcaBuildAttributeId('CPCID',CPC.i.ADDR,HWMCA_ACTIVATION_PROFILE_SUFFIX)
   If rc \= HWMCA_DE_NO_ERROR Then Call MSG2004 current_attribute
   Call Get_aGet CPCID current_attribute
   If rc \= HWMCA_DE_NO_ERROR Then Return
   /*  strip any trailing null */
   CPC.i.ACTPROF = Strip(OUTPUT.1.DATA,'Trailing','00'x)
   If Debug Then Say "Actprof built:" CPC.i.ACTPROF

   Return


/***********************************************************************
*
*  Obtain CPC object images
*
***********************************************************************/
Get_CPC_Images:

   If Debug Then Say 'entering Get_CPC_Images'

   /****************************************************************************/
   /* Ask GetNext to return each SNMP ID for all defined CPC images.           */
   /****************************************************************************/

   CPCIMAGE.0 = 0
   id = HWMCA_CPC_IMAGE_ID||'.'||HWMCA_NAME_SUFFIX

   rc = HWMCA_DE_NO_ERROR
   prefix = id
   done = 0
   i = 1

   Do While rc == HWMCA_DE_NO_ERROR & done == 0
      current_attribute = 'Defined CPC Images'
      Call Get_aGetNext id current_attribute
      If rc \= HWMCA_DE_NO_ERROR Then Do
         Call Term_HMC
         Exit 'N'
      End

      If prefix == Substr(OUTPUT.1.DATA,1,Length(prefix)) Then Do
         /*  strip any trailing null */
         id = Strip(OUTPUT.1.DATA,'Trailing','00'x)
         /*  change any '0A'x(line feed) byte to a blank and strip any trailing null */
         nameo = Strip(Translate(OUTPUT.2.DATA,':','0A'x),'Trailing','00'x)
         objid = HWMCA_CPC_IMAGE_ID||Substr(id,LastPos('.',id))
         If Debug Then Say "Image #:"i "has name of:" nameo "and object id of:" objid
         CPCIMAGE.0 = i

         /* initialize variable attributes */
         CPCIMAGE.i.ADDR    = objid
         CPCIMAGE.i.PNAME   = ' '
         CPCIMAGE.i.OSNAME  = ' '
         CPCIMAGE.i.OSTYPE  = ' '
         CPCIMAGE.i.OSLEVEL = ' '
         CPCIMAGE.i.IMLMODE = 0

         /* obtain attributes for the Image  */
         Call Get_CPC_Image
         If rc \= HWMCA_DE_NO_ERROR Then Do
            Call Term_HMC
            Exit 'N'
         End

         i=i+1
      End /* do */
      Else Do
         done = 1
      End /* do */
   End /* do */

   If Debug Then Say "Number of CPCs:" CPCIMAGE.0
   If CPCIMAGE.0 = 0 Then Do
      Call MSG2012 current_attribute
      Call Term_HMC
      Exit 'N'
   End

   Return


/***********************************************************************
*
*  Obtain CPC object image attributes
*
***********************************************************************/
Get_CPC_Image:

   /**************************************************************************/
   /* Object attributes for CPC Image                                        */
   /**************************************************************************/

   /**************************************************************************/
   /* attribute: Name                                                        */
   /* operation: Get                                                         */
   /* type     : OCTETSTRING                                                 */
   /**************************************************************************/

   current_attribute = 'Name'

   rc = RxHwmcaBuildAttributeId('IMAGEID',CPCIMAGE.i.ADDR,HWMCA_NAME_SUFFIX)
   If rc \= HWMCA_DE_NO_ERROR Then Call MSG2004 current_attribute
   Call Get_aGet IMAGEID current_attribute
   If rc \= HWMCA_DE_NO_ERROR Then Return
   /*  change any '0A'x(line feed) byte to a blank and strip any trailing null */
   CPCIMAGE.i.NAME = Strip(Translate(OUTPUT.1.DATA,':','0A'x),'Trailing','00'x)
   If Debug Then Say "name built:" CPCIMAGE.i.NAME

   /**************************************************************************/
   /* attribute: Parent Name                                                 */
   /* operation: Get                                                         */
   /* type     : OCTETSTRING                                                 */
   /**************************************************************************/

   current_attribute = 'Parent Name'

   rc = RxHwmcaBuildAttributeId('IMAGEID',CPCIMAGE.i.ADDR,HWMCA_PARENT_NAME_SUFFIX)
   If rc \= HWMCA_DE_NO_ERROR Then Call MSG2004 current_attribute
   Call Get_aGet IMAGEID current_attribute
   If rc \= HWMCA_DE_NO_ERROR Then Return
   /*  change any '0A'x(line feed) byte to a blank and strip any trailing null */
   CPCIMAGE.i.PNAME = Strip(Translate(OUTPUT.1.DATA,':','0A'x),'Trailing','00'x)
   If Debug Then Say "pname built:" CPCIMAGE.i.PNAME

   /**************************************************************************/
   /* attribute: Status                                                      */
   /* operation: Get                                                         */
   /* type     : INTEGER                                                     */
   /**************************************************************************/

   current_attribute = 'Status'

   rc = RxHwmcaBuildAttributeId('IMAGEID',CPCIMAGE.i.ADDR,HWMCA_STATUS_SUFFIX)
   If rc \= HWMCA_DE_NO_ERROR Then Call MSG2004 current_attribute
   Call Get_aGet IMAGEID current_attribute
   If rc \= HWMCA_DE_NO_ERROR Then Return
   CPCIMAGE.i.STATUS = OUTPUT.1.DATA
   If Debug Then Say "Status built:" CPCIMAGE.i.STATUS

   /**************************************************************************/
   /* attribute: Acceptable status                                           */
   /* operation: Get                                                         */
   /* type     : INTEGER                                                     */
   /**************************************************************************/

   current_attribute = 'Acceptable Status'

   rc = RxHwmcaBuildAttributeId('IMAGEID',CPCIMAGE.i.ADDR,HWMCA_EXPECTED_STATUS_SUFFIX)
   If rc \= HWMCA_DE_NO_ERROR Then Call MSG2004 current_attribute
   Call Get_aGet IMAGEID current_attribute
   If rc \= HWMCA_DE_NO_ERROR Then Return
   CPCIMAGE.i.ASTATUS = OUTPUT.1.DATA
   If Debug Then Say "Astatus built:" CPCIMAGE.i.ASTATUS

   /**************************************************************************/
   /* attribute: Activation Profile                                          */
   /* operation: Get                                                         */
   /* type     : OCTETSTRING                                                 */
   /**************************************************************************/

   current_attribute = 'Activation Profile'

   rc = RxHwmcaBuildAttributeId('IMAGEID',CPCIMAGE.i.ADDR,HWMCA_ACTIVATION_PROFILE_SUFFIX)
   If rc \= HWMCA_DE_NO_ERROR Then Call MSG2004 current_attribute
   Call Get_aGet IMAGEID current_attribute
   If rc \= HWMCA_DE_NO_ERROR Then Return
   /*  change any '0A'x(line feed) byte to a blank and strip any trailing null */
   CPCIMAGE.i.ACTPROF = Strip(Translate(OUTPUT.1.DATA,':','0A'x),'Trailing','00'x)
   If Debug Then Say "ACTPROF built:" CPCIMAGE.i.ACTPROF

   Return


/***********************************************************************
*
*  Thread objects routine
*
***********************************************************************/
Thread_Objects:

   If Debug Then Say 'entering Thread_Objects'

   /* if CPC object in LPAR or Native mode, make sure to thread image */
   Do i = 1 To CPC.0

      /*************************************************************************************************/
      /* If CPC is set to LPAR mode get images for CPC, Cross reference image and CPC name and status  */
      /*************************************************************************************************/

      If (CPC.i.IMLMODE = HWMCA_IML_LPAR_MODE) | (CPC.i.IMLMODE = HWMCA_IML_ESA390_MODE) Then Do
         Do j = 1 To CPCIMAGE.0
            If CPC.i.NAME = CPCIMAGE.j.PNAME Then Do
               CPC.i.NOOFIMAGES = CPC.i.NOOFIMAGES + 1
               imnum = CPC.i.NOOFIMAGES
               CPC.i.imnum.IMAGENAME = CPCIMAGE.j.NAME
               CPC.i.imnum.IMAGENUM  = j
               CPC.i.imnum.ISTATUS   = CPCIMAGE.j.STATUS
               CPC.i.imnum.IMODE     = CPC.i.IMLMODE
               CPCIMAGE.j.PNUMBER    = i
               If Debug Then Say "Threaded:" CPCIMAGE.j.NAME "to" CPC.i.NAME
            End
         End
      End
   End

   Return


/***********************************************************************
*
*  Perform cpc status dialog
*
***********************************************************************/
Show_CPCs:

   Do Forever
      Call Clear_Screen
      Call Say_header
      text0 = '**********************************************************'
      text1 = '*              Defined CPC status information            *'
      text2 = '**********************************************************'
      Say
      Say Center(text0,80)
      Say Center(text1,80)
      Say Center(text2,80)
      Say

      Do j = 1 To CPC.0
         x = CPC.j.STATUS
         y = CPC.j.IMLMODE
         Say Right(j,3)')' Left(CPC.j.NAME,20) 'Status =' Left(STATUS.x,20) 'Mode =' MODE.y
      End /* do */
      Say
      Say
      Say 'Enter (U)p to refresh CPC status, object number followed by (D)own,'
      Say ' Command Name, or E(X)it.  Enter ? for Help.'

      Parse Pull cmd_reply
      Parse Upper Var cmd_reply obj_num command_verb

      SELECT
         When command_verb = 'D'
            Then Call Show_CPC_Image obj_num
         When command_verb = 'U'
            Then Call Obtain_Status
         When command_verb = 'X'
            Then Call Exit_HMC
         When obj_num = 'U'
            Then Call Obtain_Status
         When obj_num = '?'
            Then Call Command_Help
         When obj_num = 'X'
            Then Call Exit_HMC
         Otherwise /* must be a command */
            Say 'Issuing:' obj_num command_verb
            Call Process_Command obj_num HWMCA_CPC_OBJECT command_verb
      End  /* select */
   End /* do forever */

   Return


/***********************************************************************
*
*  Perform show cpc image dialog
*
***********************************************************************/
Show_CPC_Image:

   /* on entry the cpc parent object index is passed */
   Parse Arg i

   If \datatype(obj_num,'N') Then Do
      Call MSG2001
      Return
   End
   If i < 1 | i > CPC.0 Then Do
      Say i 'not in the range 1 to' CPC.0||'.'
      Call SysSleep 3
      Return
   End

   If CPC.i.NOOFIMAGES = 0 Then Do
      Say 'No images found for this CPC.'
      Call SysSleep 3
      Return
   End

   Do Forever
      Call Clear_Screen
      Call Say_header
      text0 = '**********************************************************'
      text1 = '*         Defined CPC Image status information           *'
      text2 = '**********************************************************'
      Say
      Say Center(text0,80)
      Say Center(text1,80)
      Say Center(text2,80)
      Say

      Do j = 1 To CPC.i.NOOFIMAGES
         Do dj = 1 To CPCIMAGE.0
            If CPCIMAGE.dj.NAME = CPC.i.j.IMAGENAME Then Do
               x = CPC.i.j.ISTATUS
               y = CPC.i.j.IMODE
               Say Right(dj,3)')' Left(CPCIMAGE.dj.NAME,20) 'Status =' Left(STATUS.x,20) 'Mode =' MODE.y
            End
         End
      End
      Say
      Say
      Say 'Enter (U)p, E(X)it, or object number followed by Command Name.'
      Say 'Enter ? for Help'

      Parse Pull cmd_reply
      Parse Upper Var cmd_reply obj_num command_verb opsys_command

      SELECT
         When cmd_reply = 'U'
            Then Return
         When cmd_reply = 'X'
            Then Call Exit_HMC
         When obj_num = 'X'
            Then Call Exit_HMC
         When obj_num = 'U'
            Then Return
         When obj_num = '?'
            Then Call Command_Help
         Otherwise /* must be a command */
            Say 'Issuing:' obj_num command_verb opsys_command
            Call Process_Command obj_num HWMCA_CPC_IMAGE_OBJECT command_verb opsys_command
      End  /* select */
   End /* do forever */

   Return


/***********************************************************************
*
*  Process command routine
*
***********************************************************************/
Process_Command:

   Parse upper arg obj_num obj_type command_verb opsys_command

   If \datatype(obj_num,'N') Then Do
      Call MSG2001
      Return
   End

   SELECT
      When command_verb = 'SEND_OPSYS' | command_verb = 'SO'
         Then Call Process_OpSys_CMD obj_num 'SEND_OPSYS' opsys_command
      When command_verb = 'ACTIVATE'
         Then Call Process_Obj_CMD obj_num obj_type HWMCA_ACTIVATE_COMMAND HWMCA_ACTIVATE_COMMAND_SUFFIX
      When command_verb = 'DEACTIVATE'
         Then Call Process_Obj_CMD obj_num obj_type HWMCA_DEACTIVATE_COMMAND HWMCA_DEACTIVATE_COMMAND_SUFFIX
      When command_verb = 'RESETNORMAL'
         Then Call Process_Obj_CMD obj_num obj_type HWMCA_RESETNORMAL_COMMAND HWMCA_RESETNORMAL_COMMAND_SUFFIX
      When command_verb = 'START'
         Then Call Process_Obj_CMD obj_num obj_type HWMCA_START_COMMAND HWMCA_START_COMMAND_SUFFIX
      When command_verb = 'STOP'
         Then Call Process_Obj_CMD obj_num obj_type HWMCA_STOP_COMMAND HWMCA_STOP_COMMAND_SUFFIX
      When command_verb = 'PSWRESTART'
         Then Call Process_Obj_CMD obj_num obj_type HWMCA_PSWRESTART_COMMAND HWMCA_PSWRESTART_COMMAND_SUFFIX
      When command_verb = 'LOAD'
         Then Call Process_Obj_CMD obj_num obj_type HWMCA_LOAD_COMMAND HWMCA_LOAD_COMMAND_SUFFIX
      Otherwise /* must be a command */
         Call MSG2005
   End  /* select */

   Return


/***********************************************************************
*
*  Process operating system command
*
***********************************************************************/
Process_OpSys_CMD:

   Parse Arg obj_num command_verb opsys_command

   /****************************************************************************/
   /* Setup and Issue the Command API                                          */
   /****************************************************************************/
   CMD.0 = 2
   CMD.1.TYPE = HWMCA_TYPE_INTEGER
   CMD.1.DATA = HWMCA_FALSE

   /* set command variables */
   CMD.2.TYPE = HWMCA_TYPE_OCTETSTRING
   CMD.2.DATA = opsys_command

   cpcimib = CPCIMAGE.obj_num.ADDR

   If Debug Then Do
      Say "Target MIB ID:" cpcimib
      Say "Command text :" CMD.2.DATA
   End

   rc = RxHwmcaCommand(initvar,cpcimib,HWMCA_SEND_OPSYS_COMMAND,'CMD.',API_TIMEOUT)
   If rc \= HWMCA_DE_NO_ERROR Then Do
      Call MSG2006 rc
      Call SysSleep 5
      Return
   End

   /* wait to see if the command was successful */
   rc = RXHwmcaWaitEvent(initvar,'OUTPUT.',API_TIMEOUT)
   If rc = HWMCA_DE_NO_ERROR Then Do
      varnum = OUTPUT.0
      If Debug Then Do n = 1 To varnum
         Say "OUTPUT TYPE NUM" n
         Say "               " OUTPUT.n.TYPE
         Say "               " OUTPUT.n.DATA
      End
      command_rc = OUTPUT.6.DATA
      Call MSG2007 command_rc opsys_command
   End
   Else Do
      Call MSG2008 rc
   End

   /* 5 seconds to see the messages */
   Call SysSleep 5

   Return


/***********************************************************************
*
*  Process Hardware commands against the CPC's and CPC images
*
***********************************************************************/
Process_Obj_CMD:

   Parse Arg obj_num obj_type command_verb cmd_suffix

   SELECT
      When obj_type =   HWMCA_CPC_OBJECT
         Then Do
            obj_mib = CPC.obj_num.ADDR
            obj_name = CPC.obj_num.NAME
            act_prof  = CPC.obj_num.ACTPROF
         End
      When obj_type =   HWMCA_CPC_IMAGE_OBJECT
         Then Do
            obj_mib = CPCIMAGE.obj_num.ADDR
            obj_name = CPCIMAGE.obj_num.NAME
            act_prof = CPCIMAGE.obj_num.ACTPROF
         End
      Otherwise /* must be invalid */
         Call MSG2009
         Return
   End        /* SELECT */

   /* confirm command being issued */
   cmd_name = CMDTYPE.cmd_suffix
   Call Clear_Screen
   Say 'Confirm' cmd_name 'for' obj_name '. Reply Y or N'
   Pull confirm_reply
   If confirm_reply \= 'Y' Then Return

   /*  at this point we need to check if the command was an ACTIVATE, if so we need to check if the
  customer wants to use another activation profile. Therefore we will process this command in
  another routine. */
   Select
      When command_verb =  HWMCA_ACTIVATE_COMMAND Then
         Call Process_Activate_CMD  obj_num obj_type command_verb cmd_suffix act_prof
      When command_verb =  HWMCA_LOAD_COMMAND Then
         Call Process_Load_CMD  obj_num obj_type command_verb cmd_suffix
      Otherwise
         rc = RxHwmcaCommand(initvar,obj_mib,command_verb,,API_TIMEOUT)
   End

   /* Process_Activate_CMD and Process_Load_CMD set rc before returning */
   If rc \= HWMCA_DE_NO_ERROR Then Do
      Call MSG2010 rc
      Call SysSleep 5
      Return
   End

   /* wait to see if the command was successful */
   rc = RXHwmcaWaitEvent(initvar,'OUTPUT.',API_TIMEOUT)
   If rc = HWMCA_DE_NO_ERROR Then Do
      varnum = OUTPUT.0
      If Debug Then Do n = 1 To varnum
         Say "OUTPUT TYPE NUM" n
         Say "               " OUTPUT.n.TYPE
         Say "               " OUTPUT.n.DATA
      End
      If OUTPUT.2.DATA = HWMCA_EVENT_COMMAND_RESPONSE Then Do
         command_rc = OUTPUT.6.DATA
         Call MSG2011 cmd_name command_rc
      End
   End
   Else Do
      Call MSG2008 rc
   End

   Return


/***********************************************************************
*
***********************************************************************/
Process_Activate_CMD:

   Parse Arg obj_num obj_type command_verb cmd_suffix act_prof

   Say 'Please specify activation profile or hit enter to use the current profile:     ' act_prof
   Parse Pull act_prof_temp
   If act_prof_temp = "" Then act_prof_temp = act_prof
   CMD.0 = 1
   CMD.1.TYPE = HWMCA_TYPE_OCTETSTRING
   CMD.1.DATA = act_prof_temp

   rc = RxHwmcaCommand(initvar,obj_mib,command_verb,'CMD.',API_TIMEOUT)

   Return

/***********************************************************************
*
***********************************************************************/
Process_Load_CMD:

   Parse Arg obj_num obj_type command_verb cmd_suffix

   /* setup literals */
   response. = ''
   Label.1  = 'Load address:            '
   prompt.1 = 'Please enter the Load address to be used.'
   remind.1 = ''
   Label.2  = 'Load parameter:          '
   prompt.2 = 'Please enter the Load parameter to be used.'
   remind.2 = ''
   Label.3  = 'Clear memory before load?'
   prompt.3 = "Should memory be cleared before performing the Load?"
   remind.3 = "Press 'Y' for Yes and 'N' for No"
   Label.4  = 'Timeout (seconds):       '
   prompt.4 = 'Please enter the timeout value to use when performing the Load.'
   remind.4 = "(Must be between 60 and 600 seconds.)"
   Label.5  = 'Store status befor load? '
   prompt.5 = "Should status be stored before performing the Load?"
   remind.5 = "(Press 'Y' for Yes or 'N' for No)"

   /* obtain load settings from operator until operator says they are correct*/
   Do Forever
      /* solicite and receive each parameter */
      Do RespNdx = 1 To 5
         Say prompt.RespNdx
         If remind.RespNdx <> '' Then
            Say remind.RespNdx
         Call Charout ,'====>'
         Pull response
         /* response is not a zero length (null) string then store */
         If response <> '' Then
            response.RespNdx = response
      End

      /* now list the responses for operator view and verification */
      Say ' '
      Do RespNdx = 1 To 5
         Say Label.RespNdx response.RespNdx
      End
      Say ' '

      /* get his response for correctness */
      Say 'Are all responses correct?'
      Say "(Press 'Y' for Yes and 'N' or No)"
      Call Charout ,'====>'
      Pull response
      If response = 'y' | response = 'Y' Then
         Leave

      /* at this point we do it all over again, previous response will be
         retained if the operator responds with carriage return only */
      Say ' '
      Say "Press 'Enter' without text to retain previous entry"
      Say ' '
   End

   CMD.0 = 5
   CMD.1.TYPE = HWMCA_TYPE_OCTETSTRING
   CMD.1.DATA = Response.1
   CMD.2.TYPE = HWMCA_TYPE_OCTETSTRING
   CMD.2.DATA = Response.2
   CMD.3.TYPE = HWMCA_TYPE_INTEGER
   Response = Substr(Response.3,1,1)
   If (Response = "Y" | Respose = 'y') Then
      CMD.3.DATA = HWMCA_TRUE
   Else
      CMD.3.DATA = HWMCA_FALSE
   CMD.4.TYPE = HWMCA_TYPE_INTEGER
   CMD.4.DATA = Response.4
   CMD.5.TYPE = HWMCA_TYPE_INTEGER
   Response = Substr(Response.5,1,1)
   If (Response = "Y" | Respose = 'y') Then
      CMD.5.DATA = HWMCA_TRUE
   Else
      CMD.5.DATA = HWMCA_FALSE

   rc = RxHwmcaCommand(initvar,obj_mib,command_verb,'CMD.',API_TIMEOUT)

   Return


/***********************************************************************
*
*  Get from HMC routine
*
***********************************************************************/
Get_aGet:

   Parse Arg object attribute

   /* issue the request to the HMC */
   rc = RxHwmcaGet(initvar,object,OUTPUT.,API_TIMEOUT)
   If rc = HWMCA_DE_NO_ERROR Then Return

   /* retry while it's TIMEOUT */
   Do j = 1 To RETRY_TIMEOUT While rc = HWMCA_DE_TIMEOUT
      rc = RxHwmcaGet(initvar,object,OUTPUT.,API_TIMEOUT)
   End

   /* log if still have an error */
   If rc \= HWMCA_DE_NO_ERROR Then Call MSG2003 rc attribute

   Return


/***********************************************************************
*
*  GetNext from HMC routine
*
***********************************************************************/
Get_aGetNext:

   Parse Arg object attribute

   /* issue the request to the HMC */
   rc = RxHwmcaGetNext(initvar,object,OUTPUT.,API_TIMEOUT)
   If rc = HWMCA_DE_NO_ERROR Then Return

   /* retry while it's TIMEOUT */
   Do j = 1 To RETRY_TIMEOUT While rc = HWMCA_DE_TIMEOUT
      rc = RxHwmcaGetNext(initvar,object,OUTPUT.,API_TIMEOUT)
   End

   /* log if still have an error */
   If rc \= HWMCA_DE_NO_ERROR Then Call MSG2003 rc attribute

   Return


/***********************************************************************
*
*  HMC initialization routine
*
***********************************************************************/
Init_HMC:

   /* Load Rexx Utilities   */
   If RxFuncQuery('SysLoadFuncs') then
      If rxfuncadd( 'SysLoadFuncs', 'RexxUtil', 'SysLoadFuncs' ) Then Do
         Call MSG1001
         Exit 'N'
      End
   Call SysLoadFuncs

   /* Load Hwmca Rexx interface function dynalink   */
   If RxFuncQuery('RxHwmcaLoadFuncs') Then Do
      If Debug Then Say "os_platform = "os_platform
      If os_platform = 'WindowsNT' Then Do
         Say "HMC function package to be added is "HMCDLL".DLL"
         If rxfuncadd( 'RxHwmcaLoadFuncs', HMCDll, 'RxHwmcaLoadFuncs' ) Then Do
            Call MSG1002
            Exit 'N'
         End
      End
      If os_platform = 'OS/2' Then Do
         If rxfuncadd( 'RxHwmcaLoadFuncs', 'ACTZSNMP', 'RxHwmcaLoadFuncs' ) Then Do
            Call MSG1002
            Exit 'N'
         End
      End
   End

   Call RxHwmcaLoadFuncs  /* load all of the other RxHwmca- functions */
   Call RxHwmcaDefineVars /* Define the HWMCA_ variables so they're available to this Rexx script */

   /* initialize IML mode strings for later display    */
   MODE.0  = 'UNKNOWN'
   MODE.1  = 'ESA 390'
   MODE.6  = 'FM'
   MODE.7  = 'FMAE'
   MODE.8  = 'HM'
   MODE.9  = 'HMEA'
   MODE.10 = 'HMEX'
   MODE.11 = 'LPAR'
   MODE.12 = 'ESA 390 TPF'
   MODE.13 = 'Coupling Facility'
   MODE.16 = 'LINUX'

   /* initialize STATUS strings for later use   */
   STATUS.1    = 'Operating'
   STATUS.2    = 'Not Operating'
   STATUS.4    = 'No Power'
   STATUS.8    = 'Not Activated'
   STATUS.16   = 'Exceptions'
   STATUS.32   = 'Status Check'
   STATUS.64   = 'Service'
   STATUS.128  = 'Link Not Active'
   STATUS.256  = 'Power Save'
   STATUS.512  = 'Serious Alert'
   STATUS.1024 = 'Alert'
   STATUS.2048 = 'Env Alert'
   STATUS.4096 = 'Service Req'

   /* initialize EVENT types */
   EVENT.0     = 'Command Response'
   EVENT.1     = 'Message'
   EVENT.2     = 'Status Change'
   EVENT.4     = 'Name Change '
   EVENT.8     = 'Activate Profile Change'
   EVENT.16    = 'Event Create'
   EVENT.32    = 'Event Destroyed'
   EVENT.64    = 'Event Exception State'
   EVENT.128   = 'Event Ended'

   /* initialize Command objects */
   CMDTYPE.1   = 'ACTIVATE'
   CMDTYPE.2   = 'DEACTIVATE'
   CMDTYPE.3   = 'OPSYS'
   CMDTYPE.4   = 'RESETNORMAL'
   CMDTYPE.5   = 'START'
   CMDTYPE.6   = 'STOP'
   CMDTYPE.7   = 'PSWRESTART'

   line_size = '30'

   /* Initialize HMC interface. Event Mask = just command response events */
   initvar = 'INITBLK.'
   INITBLK.EVENTMASK = HWMCA_DIRECT_INITIALIZE + HWMCA_EVENT_COMMAND_RESPONSE
   INITBLK.RESERVED  = 0
   INITBLK.TARGET    = hmc_host
   INITBLK.COMMUNITY = hmc_community

   If Debug Then Say 'issued RxHwmcaInitialize for INITEVNT.'
   rc = RxHwmcaInitialize(initvar,API_TIMEOUT)
   If rc \= HWMCA_DE_NO_ERROR Then Do
      Call MSG2002 rc
      Call Term_HMC
      Exit 'N'
   End

   Return


/***********************************************************************
*
*  All message routines go here
*
***********************************************************************/
MSG1001:
   Say 'Unable to load system function package, Script terminating'
   Return
MSG1002:
   Say 'Unable to load HWMCA function package, script terminating'
   Return
MSG2001:
   Say 'Invalid data type for object number'
   Call SysSleep 3
   Return
MSG2002:
   Parse Arg rc
   Say 'Hwmca Initialization failure - script terminating rc =' rc
   Return
MSG2003:
   Parse Arg rc current_attribute
   Say 'Hwmca Get failure - rc =' rc '- attribute =' current_attribute
   Return
MSG2004:
   Parse Arg current_attribute
   Say 'Unable to build HWMCA Attribute - attribute =' current_attribute
   Return
MSG2005:
   Say 'Invalid Command - Please respecify'
   Call SysSleep 3
   Return
MSG2006:
   Parse Arg rc
   Say 'SEND OPSYS command failed rc =' rc
   Return
MSG2007:
   Parse Arg command_rc opsys_command
   Say 'SEND OPSYS command for command:' opsys_command 'complete. Return code =' command_rc
   Return
MSG2008:
   Parse Arg rc
   Say 'Waitevent failure - Command - rc =' rc
   Return
MSG2009:
   Say 'Invalid object type - must be CPC or CPC Image object'
   Return
MSG2010:
   Parse Arg rc
   Say 'Object command failed rc =' rc
   Return
MSG2011:
   Parse Arg command_rc cmd_name
   Say cmd_name 'command complete. Return code =' command_rc
   Return
MSG2012:
   Parse Arg current_attribute
   Say '0 variables returned from HwmcaGet - attribute =' current_attribute
   Return
MSG2013:
   Say 'No CPCs Defined - returning to main process'
   Return


/***********************************************************************
*
*  Command Help routine
*
***********************************************************************/
Command_Help:

   Call Clear_Screen
   Say Center('Command Help',line_size)
   Say
   Say Left('Commands:',line_size)
   Say
   Say Left('Activate',line_size)    '- Activate a CPC or CPC image'
   Say Left('Deactivate',line_size)  '- Deactivate a CPC or CPC image'
   Say Left('Stop',line_size)        '- Stop a CPC image'
   Say Left('Start',line_size)       '- Start a CPC image'
   Say Left('ResetNormal',line_size) '- System Reset a CPC image'
   Say Left('PSWRestart',line_size)  '- PSW Restart a CPC image'
   Say Left('Load',line_size)        '- Load a CPC image'
   Say Left('Send_OpSys',line_size)  '- Send an operating system command to a CPC image'
   Say Left('so',line_size)          '- Alias for send_opsys command'
   Say
   Say Left('Examples:',line_size)
   Say
   Say Left('1 Activate',line_size)      '- Activate first object (CPC or CPC image)'
   Say Left('3 Stop',line_size)          '- Stop CPC image number 3'
   Say Left('1 send_opsys $da',line_size) '- Send CPC image number 1 a JES2 command'
   Say
   Say 'Press Enter to continue'
   Parse Pull cmd_reply
   Return


/***********************************************************************
*
*  Generic clear screen routine
*
***********************************************************************/
Clear_Screen:

   If Debug Then Return

   If AFR_ENV Then Call AFR_CLS
              Else Call SysCls
   Return


/***********************************************************************
*
*  Display header lines
*
***********************************************************************/
Say_header:

   text0 = Left('Host:' hmc_host,line_size)
   text1 = Left('Date:' Date("l"),line_size)
   text2 = Left('Time:' Time("n"),line_size)
   Say text0
   Say text1
   Say text2
   Say
   Return


/***********************************************************************
*
*  Exit hmc routine
*
***********************************************************************/
Exit_HMC:

   Call Term_HMC
   Call Clear_Screen
   Call Say_header
   text0 = '**********************************************************'
   text1 = '*              Exiting HMC Command Routine               *'
   text2 = '**********************************************************'
   Say
   Say Center(text0,80)
   Say Center(text1,80)
   Say Center(text2,80)
   Exit 'Y'


/***********************************************************************
*
*  HMC termination routine
*
***********************************************************************/
Term_HMC:

   rc = RxHwmcaTerminate(initvar,API_TIMEOUT)

   Return


/***********************************************************************
*
*  Branch to this point if program halted from AF/REMOTE Client Scripts window
*
***********************************************************************/
Halt:

   result = program_name 'Terminated by Operator'
   Say result
   /* whatever else needs to be done to clean up after a shutdown */
   Call Term_HMC
   Exit 'N'


/***********************************************************************
*
*  Branch to this point if syntax error encountered or function call in error
*
***********************************************************************/
Syntax:

   result = program_name 'Terminated for Syntax, Line' SIGL 'Code' RC
   Say result
   /* whatever else needs to be done to clean up after a syntax shutdown */
   Call Term_HMC
   Exit 'N'
