/***************************************************************************/
/*                                                                         */
/*     Program:  INTERACT.REX                                              */
/*     Date:     February, 1990                                            */
/*     Author:   Mark D. Dean                                              */
/*                                                                         */
/*     Purpose:  To demonstrate a REXX script which uses the voice card    */
/*               to allow a human to control a simulated mainframe IPL.    */
/*                                                                         */
/*               The mainframe, via AF/Operator or a human sending WTORs,  */
/*               generates the following:                                  */
/*                  - sends a WTOR with "EM102I" to the MVS console        */
/*                  - if the reply is a 1, then the WTOR is sent again     */
/*                    otherwise, the reply is ignored                      */
/*                                                                         */
/*               The WTOR message simulates VTAM failing.  When found,     */
/*               INTERACT.REX proceeds to dial the PhoneNumber, speaks the */
/*               voice messages to the person answering the phone, and     */
/*               solicits TouchTone choices.  The selected action (either  */
/*               simulating a restart of VTAM or performing the IPL) is    */
/*               then performed.                                           */
/*                                                                         */
/*     Copyright (C) Candle Corporation, 1990-1999.  All rights reserved.  */
/*                                                                         */
/***************************************************************************/

/* retrieve the name of the source program */
parse source name
program_name = substr(name,lastpos('\',name)+1)':'

say program_name "Started at" time() "on" date()

/* set some error detection signals */
signal on halt
signal on novalue
signal on syntax

Console_MVS = ""
TermMod_MVS = 2
Console_3090 = ""
TermMod_3090 = 2

/* get use/title/screen for first possible emulator */
emulator_use = 'AFR_GETENV'("E1_USE")
emulator_title = 'AFR_GETENV'("E1_TITLE")
emulator_screen = 2
if subword('AFR_GETENV'("E1_SCREEN"), 4, 1) = '43' then
   emulator_screen = 4

/* loop for all possible emulators after first */
do i = 2 to 999 until emulator_use == "" | (Console_MVS \= "" & Console_3090 \= "")

   /* check if emulator in use */
   if emulator_use \= "None" then do

      /* inform anybody watching */
      say program_name 'Emulator' emulator_title 'is a(n)' emulator_use 'Console'

      /* check if MVS console */
      if pos("MVS Console",emulator_use) > 0 then do
         Console_MVS = emulator_title
         TermMod_MVS = emulator_screen
      end

      /* check if 3090 console */
      if pos("3090 Console",emulator_use) > 0 then do
         Console_3090 = emulator_title
         TermMod_3090 = emulator_screen
      end
   end

   /* get use/title/screen for next possible emulator */
   emulator_use = 'AFR_GETENV'("E"||i||"_USE")
   emulator_title = 'AFR_GETENV'("E"||i||"_TITLE")
   emulator_screen = 2
   if subword('AFR_GETENV'("E"||i||"_SCREEN"), 4, 1) = '43' then
      emulator_screen = 4
end

/* ensure have appropriate consoles */
if Console_MVS == "" | Console_3090 == "" then do

   /* can't do it */
   say program_name 'Terminated for Insufficient Consoles'
   exit 'N'
end

/* main loop; get a phone number to dial */

do forever
   say "INTERACT.REX - enter phone number to call, or CR to terminate"
   pull PhoneNumber

   if (PhoneNumber = "") then do
      say "Terminating now"
      exit
   end

/* after running the script, hang up the phone for next time */

   ret = IPL(PhoneNumber)
   nul = afr_VoiceHangup()

   if (ret <> 0) then               /* if error returned, print message */
      say "FATAL ERROR: " ret
end

/* advise completion of program */
say program_name "Terminated Normally at" time() "on" date()

/* just for drill */
return 'OK'

/* branch to this point if program halted from AF/REMOTE Client Scripts window */
halt:
   result = program_name 'Terminated by Operator'
   say result
   ret = 'AFR_ERROR'(result,1)
/* whatever else needs to be done to clean up after a shutdown */
   exit 'N'

/* branch to this point if novalue error encountered */
novalue:
   result = program_name 'Terminated for NoValue, Line' SIGL
   say result
   ret = 'AFR_ERROR'(result,0)
/* whatever else needs to be done to clean up after a No Value shutdown */
   exit 'N'

/* branch to this point if syntax error encountered or function call in error */
syntax:
   result = program_name 'Terminated for Syntax, Line' SIGL 'Code' RC
   say result
   ret = 'AFR_ERROR'(result,0)
/* whatever else needs to be done to clean up after a syntax shutdown */
   exit 'N'

IPL: procedure expose program_name Console_MVS TermMod_MVS Console_3090 TermMod_3090
parse arg PhoneNumber

/* Initialize the voice card; try up to five times if problems */

   say
   say " INTERACT.REX: script beginning now, calling" PhoneNumber

   do i = 1 to 5
      ret = afr_VoiceInit()
      if (ret = 0) then leave                        /* success; exit loop */

      if (i = 5) then
         return "unable to initialize voice card"

      say "Error" ret "initializing voice card; will retry" 5-i "more times"
      nul = afr_suspend(1000)
   end
   say "Voice Card initialized"

/* select the MVS Master Console (Emulator B), and wait for the WTOR */

   if (afr_select(Console_MVS) <> 0) then return "selecting MVS console"

   if (TermMod_MVS = 2) then
      handle = afr_set_wtor("EM102I", 0, 1920, 80)
   if (TermMod_MVS = 4) then
      handle = afr_set_wtor("EM102I", 0, 3440, 80)
   if (handle < 0) then
      return "getting handle for WTOR"

   say "Waiting on WTOR from AF/OPERATOR . . . "
   do forever
      trap = afr_check_wtor(handle, 2000)            /* two second timeout */
      if (trap <> "") then leave
      nul = afr_suspend(2000)
   end
   say "WTOR from AF/OPERATOR detected"

/* call someone for a decision on what to do with this trap */

   say "Dialing" PhoneNumber
   ret = afr_VoiceDial(PhoneNumber, 3)                 /* (number, nRings) */

   if (ret == -1) then say " DIAL: unknown error"
   if (ret == -2) then say " DIAL: busy signal"
   if (ret == -3) then say " DIAL: fast-busy signal"
   if (ret == -4) then say " DIAL: special information tone"
   if (ret == -5) then say " DIAL: timeout with no answer"

   if (ret <> 0) then return "dialing " || PhoneNumber

   say " DIAL: voice detected"
   nul = afr_VoiceGetTones(5, "", 0)               /* clear any TouchTones */

/* play the greeting file IPL_1 to whomever answered the phone, asking */
/* for TouchTone 1 if they can continue, or a TouchTone-2 to terminate */

   say "Saying greeting"
   ret = afr_VoiceSpeak("ipl_1.vox", 0)    /* "Hello, this is Candle ... " */
   if (ret <> 0) then return ret || " from VoiceSpeak"
   key = afr_VoiceGetTones(1, "#", 20)

/* if user didn't hit TT-1, he doesn't want the call; say good-bye */

   if (key <> 1) then do
      say "User hit a" key "; saying good-bye, and terminating"
      nul = afr_VoiceSpeak("ipl_7.vox", 0)                   /* "Good-bye" */
      return 0
   end

/* user wants to take the call; ask him for a password */

   say "Asking user for the password"
   nul = afr_VoiceSpeak("ipl_2.vox", 0)        /* "Please enter password " */
   key = afr_VoiceGetTones(4, "#", 10)

/* bad password; tell user good-bye, hang-up on him, and terminate */

   if (key <> "1234") then do
      say "User hit" key "for password; terminating"
      nul = afr_VoiceSpeak("ipl_7.vox", 0)                   /* "Good-bye" */
      return 0
   end

/* VTAM failure detected; tell the user, and ask for Touch Tone selection: */
/*             1 - Restart      2 - IPL      3 - Discontinue               */

   do forever
      say "Saying VTAM FAILED message"
      nul = afr_VoiceSpeak("ipl_3.vox", 0)     /* "VTAM failed to restart" */
      key = afr_VoiceGetTones(1, "#", 10)

   /* extract reply-number of currently outstanding WTOR */

      position = pos("*", trap)
      WTORNum = substr(trap, position+1, 2)

   /* if RESTART (1), reply 1 to get another WTOR, and wait for it */

      if (key = "1") then do

         if (afr_send_3270(" R " ||WTORNum|| ",1 @E") <> 0) then
            return "sending 3270 string"
         WaitNotBusy(TermMod_MVS)

         if (TermMod_MVS = 2) then
            handle = afr_set_wtor("EM102I", 0, 1920, 80)
         if (TermMod_MVS = 4) then
            handle = afr_set_wtor("EM102I", 0, 3440, 80)
         if (handle < 0) then
            return "getting handle for WTOR"

         say "Restart selected; waiting on next WTOR . . ."
         do forever
            trap = afr_check_wtor(handle,1000)
            if (trap <> "") then leave
            nul = afr_suspend(2000)
         end
         say "WTOR from AF/OPERATOR detected"

      end                                    /* end of TT-1 (RESTART) code */

   /* if IPL (2), respond to the WTOR with a 0 to stop the AFO script, */
   /* and break out of this loop to continue processing                */

      if (key = "2") then do
         if (afr_send_3270(" R " ||WTORNum|| ",0 @E") <> 0) then
            return "sending 3270 string"
         WaitNotBusy(TermMod_MVS)
         leave
      end

   /* if DISCONTINUE (3), say good-bye, hangup, and terminate */

      if (key = "3") then do                                /* Discontinue */
         if (afr_send_3270(" R " ||WTORNum|| ",0 @E") <> 0) then
            return "sending 3270 string"
         say "Discontinue selected; hanging up"
         nul = afr_VoiceSpeak("ipl_7.vox", 0)                /* "Good-bye" */
         return 0
      end
   end

/* make sure user really wants to IPL the system; ask for confirmation */

   say "IPL selected: asking for IPL confirmation"         /* IPL System A */
   nul = afr_VoiceSpeak("ipl_4.vox", 0)           /* "Hit 5 to really IPL" */
   key = afr_VoiceGetTones(1, "#", 10)            /* TT-5 for confirmation */

   if (key <> "5") then do
      say "Saying good-bye, and terminating"
      nul = afr_VoiceSpeak("ipl_7.vox", 0)                   /* "Good-bye" */
      return 0
   end

   say "Saying IPL request being processed"
   nul = afr_VoiceSpeak("ipl_5.vox", 0)                 /* "Beginning IPL" */

/* switch to the Hardware Console (Emulator A) to do the IPL */

   if (afr_select(Console_3090) <> 0) then return "selecting System Console"

   if (afr_send_3270(" F OPRCTL@E") <> 0) then
      return "sending 3270 string"
   WaitNotBusy(TermMod_3090)

/* make sure the command gets taken */

   do while (afr_load(150,6) <> "OPRCTL")
      nul = afr_suspend(1000)
      nul = afr_send_3270(" @R")
      nul = afr_suspend(1000)
      nul = afr_send_3270(" @E")
      WaitNotBusy(TermMod_3090)
   end

   if (afr_send_3270(" setlp mvsa@E") <> 0) then
      return "sending 3270 string"
   WaitNotBusy(TermMod_3090)

   if (afr_send_3270(" unlocklp mvsa@E") <> 0) then
      return "sending 3270 string"
   WaitNotBusy(TermMod_3090)

  /* and do the 07 for real IPL, or 08 for demo */

   if (afr_send_3270(" O8@E") <> 0) then
      return "sending 3270 string"
   WaitNotBusy(TermMod_3090)

/* tell user that the IPL is complete */

   say "Saying IPL complete message"
   nul = afr_VoiceSpeak("ipl_6.vox", 0)             /* "IPL complete; bye" */
   nul = afr_VoiceHangUp()

/* lock the Operator Console, and bring back the SAD display */

   if (afr_send_3270(" locklp mvsa@E") <> 0) then
      return "sending 3270 string"
   WaitNotBusy(TermMod_3090)

   if (afr_send_3270(" f sad@E") <> 0) then
      return "sending 3270 string"
   WaitNotBusy(TermMod_3090)

   if (afr_send_3270(" A1@E") <> 0) then
      return "sending 3270 string"
   WaitNotBusy(TermMod_3090)

   say "INTERACT script terminating now"
return 0

/***************************************************************************/
/*                                                                         */
/*     Subroutine: WaitNotBusy                                             */
/*                                                                         */
/*     Purpose: To wait up to 60 sec for the X-clock to go away.           */
/*                                                                         */
/***************************************************************************/

WaitNotBusy: procedure expose program_name
parse arg TermMod

   do i = 1 to 120                               /* check every 1/2 second */
      if (TermMod = 2) then                      /* Terminal Mod 2 ?       */
         busy = afr_load(1928, 1)                /*  look at X-CLOCK       */
      if (TermMod = 4) then                      /* Terminal Mod 4 ?       */
         busy = afr_load(3448, 1)                /*  look at X-CLOCK       */
      if (busy <> "X") then do                   /* if X-CLOCK gone,       */
         nul = afr_suspend(4000)                 /* ... pause for reset,   */
         return 0                                /* ... and RETURN         */
      end
      nul = afr_suspend(500)
   end

   say "ERROR! system still busy after 60 seconds"
   return 1
