/*-----------------------------------------------------------------------------
   MCSSend.REX
   16-Apr-1998 10:35:30

   Purpose: This procedure receives as an argument the command string to
   be sent to the already selected MVS console.  It verifies that the MCS
   console command line is available to accept and process commands sent
   to it, and the keyboard lock indicator on the OAI line is monitored for
   keyboard locked conditions.  It will attempt to clear error locks, wait
   on busy locks.

   Selected operating parameters may change by making a series of calls to
   the value() function.  These are provided to allow modification of timing
   and retries parameters, and type of MCS console (MOD 2 versus MOD 4).



   SYNOPSIS  n = MCSSend(string)

   CALLS:    CmdBusy, XBusy

   RETURNS:   0  Success
             -1  Unable to clear keyboard lock.
             -2  Serialization lockout.
             -3  Controller not accepting command.
                 This return will have the current content of the second
                 through the twenty-first byte of the OAI line appended, since
                 this data indicates the precise reason the -3 is being
                 returned

             -4  Controller not clearing command.
             -10 Zero length command string not accepted.  Note since the procedure
                 strips out many of the HLLAPI codes, attempting to send '@E' will
                 result in a -10 return code.

   Modificaton History:
   31OCT96 JMC - Added PF24 to Includelist, list of escape sequences that can
                 be sent.
               - Added a counter to account for escape sequences that are to be
                 sent.  The Length to send is decremented by this value so that
                 the resulting cursor position after the command is sent is
                 accurately calculated.
               - Added logic to account for @@ sequence to send one '@' sign
   08JUL99 DFB - Changed env = 'OS2ENVIRONMENT' to 'ENVIRONMENT' per Object
                 REXX for Windows documentation on the Value function.

 ---------------------------------------------------------------------------- */

Env = 'ENVIRONMENT'

/* The following three parameters may be changed dynamically */
Retries    = 5         /* Number times to retry sending command */
XBusyWait  = 250       /* keyboard lock test wait, milliseconds */
CommandPos = 1682      /* Mod 2 session = 1682, Mod 4 session = 3202
                          If the predominant or only Mod session is Mod 4,
                          the CommandPos should be changed to 3202 to avoid
                          unnecessary calls to the value function to make
                          dynamic changes
                          */

/* these probably will not need adjustment except in situations where the MCS
   Console is slow in responding
   */
XBusyTries = 10        /* keyboard lock spinloop tries */
BusyTries  = 10        /* Command line Busy retries */

/* the following variable defines those HLLAPI codes ('@' followed by a
   character) that will be allowed to remain in the string for transmittal.
   The list '@123456789abcdefghijklmn' are for the HLLAPI codes '@@' (to send
   the '@' (at) sign), and PFKeys 1 through 24.  Adding other codes may impact
   the operation of this support routine adversely.  Most of the other HLLAPI
   codes are concerned with cursor positioning.
   */
IncludeList = '@123456789abcdefghijklmno'


/* retrieve the command to send */
parse arg CommandToSend
if Length(CommandToSend) < 1 then
	return -10

/* retrieve optional retries value */
Console = value('MCSSend',' ',Env)
if Console <> '' then
   do
   /* retrieve optional retries value */
   Val = value(Console'Retries',,Env)
   if Val <> '' then
       if datatype(Val,'N') then
          Retries = Val

   /* retrieve optional delay value */
   Val = value(Console'Delay',,Env)
   if Val <> '' then
       if datatype(Val,'N') then
          XBusyWait = Val

   Val = value(Console'MCSModel',,Env)
   if Val = 4 then
       CommandPos = 3202
   if Val = 2 then
       CommandPos = 1682
   end

/* the position of the keyboard locked indicator, dependent on
   command line start position */
XPos = CommandPos + 239


/* strip out any positioning type of HLLAPI sequences, following is a list
   of sequences to retain */

/* point to the start of the string */
pos = 1
/* track the number of escape pairs NOT excluded */
count = 0

/* loop through the string for '@' signs */
do until pos = 0
   /* look for escape char */
   pos = pos('@', CommandToSend, pos)
   /* if found */
   if pos > 0 then
      do
      /* this one on the include list? */
      if pos(substr(CommandToSend,pos + 1,1),IncludeList) > 0 then
         do
         if substr(CommandToSend,pos + 1,1) = '@' then
            do
            /* found the @@ sequence, user wants to send '@' to MCS,
               delete the first and keep the second */
            CommandToSend = delstr(CommandToSend,pos,1)
            Pos = Pos + 1
            iterate
            end
         else
            do
            /* then skip past it ('@' plus the following character */
            pos = pos + 2
            /* count them as to be sent but not affecting MCS Cursor
               positioning */
            count = count + 2
            iterate
            end
         end
      /* otherwise strip it out */
      CommandToSend = delstr(CommandToSend,pos,2)
      end
   end

/* if resultant string is zero length then return a -10. */
if length(CommandToSend) < 1 then
   return -10

/* need to know position of end of the command string */
CommandLength  = length(CommandToSend) - count
CommandSentPos = CommandLength + CommandPos

/* determine if command line available */
CmdState = CmdBusy()
if CmdState <> 0 then
    return CmdState

/* command line available, try to send the command */
Tries = 0
do while Tries < Retries

   if afr_cursor() == CommandPos then
      do
      /* send the command */
      n = afr_send_3270(CommandToSend)

      /* wait for keyboard to unlock */
      do 10 while XBusy() <> 0
         nop
         end

      /* if length at least length of string, then probably did */
      if afr_cursor() >= CommandSentPos then
         do
         /* emit SEND code */
         n = afr_send_3270('@E')

         /* wait for command to clear from the command line */
         do 10 while afr_cursor() <> CommandPos
            XBusyState = XBusy()
            end

         /* check to see that command has finished, return success if so */
         if afr_cursor() = CommandPos then
             return 0
         end

      end

   else
      /* clear command line to retry */
      n = afr_send_3270('@B@F')

   /* wait on the console a bit */
   XBusyState = XBusy()

   Tries = Tries + 1
   end

return 0



/*------------------------ procedure CmdBusy ----------------------------------

   Purpose: This procedure monitors the current position of the
   MVS cursor. Depending on the position of the cursor one of
   three actions occurs:

   1.  If the cursor is above the command cursor (less then
   CommandPos then TABs are sent to the session until the cursor
   is at the command position.

   2.  If the cursor is to the right or below CommandPos then it
   waits.  This is a form of serialization lock.

   3.  If the cursor is positioned at CommandPos then the command
   line is considered to be available to accept commands.

   Returns 0 if command line is available.

           1 if keyboard locked

           2 if command line not available at expiration of
                BusyTries count.

 ---------------------------------------------------------------------------- */
CmdBusy:

/* check availability of command line */
XBusyState = XBusy()
if XBusyState <> 0 then
    return XBusyState

Tries = BusyTries

/* record current position of the cursor */
Cursor = afr_cursor()

/* we will conditionally home the cursor.  If the console definition is a
   3279 console then cursor will end up at the start of the first deletable
   line in the scroll area.  The while loops following this block of code will
   adjust for that situation */

if Cursor < CommandPos then
   do
   /* home cursor */
   n = afr_send_3270('@0')
   XBusyState = XBusy()
   if XBusyState <> 0 then
      return XBusyState
   end

/* while the cursor is not at the command line available position */
do while Cursor <> CommandPos
    /* do while cursor is not at command position but is available */
    do while Cursor < CommandPos

        /* send a back tab and then wait */
        n = afr_send_3270('@B')
        XBusyState = XBusy()
        if XBusyState <> 0 then
           return XBusyState

        /* and then record new cursor position */
        Cursor = afr_cursor()
        end

    /* count the number of time we've already attempted to acquire use of
       the command line */
    Tries = Tries - 1
    if Tries <= 0 then
        return -2

    /* wait a bit and then record new position of cursor */
    call afr_suspend XBusyWait
    Cursor = afr_cursor()
    end

return 0

/*------------------------ procedure XBusy ------------------------------------

   Purpose: to monitor the state of the keyboard locked indicator.

   This procedure takes as arguments:
      CommandPos which is the location on an AF/REMOTE MVS session
      screen where the start of the command entry line is located.

      XPos which is the location of the keyboard locked indicator

   Return  0 if keyboard not locked,
          -1 if locked
          -3 if error locked, reset (@R) could not clear

 ---------------------------------------------------------------------------- */

XBusy:

/* while keyboard in 'X' wait */

do XBusyTries

   /* wait a bit */

   call afr_suspend XBusyWait

   XState = afr_load(XPos,20)

   /* if first character position is a space then console is off line */
   if substr(Xstate,1,1) == ' ' then
      return -3 substr(Xstate,3)

   /* if xclock not found the keyboard not locked */
   if pos('X',XState) = 0 then
       return 0

   /* locked then simply spin wait */
   if pos('[]',XState) > 0 then
       nop

   /* otherwise send a keyboard reset */
   else
       n = afr_send_3270('@R')

   end

/* still keyboard still locked, then so advise */
if pos('[]',XState) > 0 then
   return -1

/* keyboard is in some error state */
return -3

