/*************************** REXX *********************************************
   SCRIPT: NETSCAN.REX
   16-Apr-1998 10:35:30

   PURPOSE: Continuously scans assigned emulator for WTOR with trigger string.

   Inputs:  Arg(1): Emulator Card title to monitor.
            Arg(2): Trigger string to monitor.

   Outputs: Nothing.

   Calls:   Program named in trapped WTORS.

 ******************************************************************************/

/* retrieve the name of the source program */
parse source name
program_name = substr(name,lastpos('\',name)+1)':'

say program_name "Started at" time() "on" date()

/* set some error detection signals */
signal on halt
signal on novalue
signal on syntax

/* constant and variable data definition */
trap_handle = 0                          /* handle for current trap */
trap_lines = 24                          /* number lines to search */
console_columns = 80                     /* number columns/line */
trap_start = 0 * console_columns         /* start point of search */
trap_stop = trap_lines * console_columns /* end point of search */
data_offset = 0                          /* offset of text from found */
data_length = 79                         /* length of data to return */
prev.wtor.number = -1                    /* extracted wtor number */
current.wtor = ''                        /* text of current wtor */
current.wtor.number = -2                 /* extracted wtor number */
full_trigger = 'IEE159E'                 /* trigger for MVS full message */
full_start = 22 * console_columns        /* start of line for full messge */
full_stop = full_start + console_columns + console_columns
                                         /* end of line for full message */
full_offset = 0                          /* offset of text from found */
full_length = 160                        /* length of text */
wtor_age = 15                            /* number of seconds until wtor
                                                   is tried again */

/* General Setup and Initialization */

/* get emulator assignment and trigger */
parse arg emulator trigger .

/* if no argument, then diagnose and exit */
if emulator = '' then
   do
      Say program_name 'Not Started, No Emulator Card Specified'
      return 'failed'
   end

/* try to select emulator */
ret = 'AFR_SELECT'(emulator)
if ret \= 0 then
   do
      say program_name 'Terminated, Error Selecting Emulator Card' emulator', RC =' ret
      return 'failed'
   end

/* verify is a MVS console */
console = 'AFR_CONSOLE_TYPE'()
flag = pos("MVS Console",console)
if flag = 0 then
   do
      say program_name 'Terminated, Not an MVS Console' emulator
      return "Invalid Console Type"
   end

/* loop for all possible emulators */
do i = 1 to 999 until translate(emulator_title) = translate(emulator)

   /* get title for next possible emulator */
   emulator_title = 'AFR_GETENV'("E"||i||"_TITLE")
   emulator_screen = 'AFR_GETENV'("E"||i||"_SCREEN")
end

/* find terminal lines and adjust screen size dependent data */
screen_lines = subword(emulator_screen, 4, 1)
trap_lines = screen_lines
trap_stop = trap_lines * console_columns
full_start = (trap_lines - 2) * console_columns
full_stop = trap_stop

/* set up to find a WTOR */
trap_handle = 'AFR_SET_WTOR'(trigger,trap_start,trap_stop,data_length)
if trap_handle < 0 then
   do
      say program_name 'Unable to set wtor trap, trap_handle =' trap_handle
      return 'N'
   end

/* check to see if MVS scroll area over full*/
full_handle = 'AFR_SET_TRAP'(full_trigger,full_start,full_stop,full_offset,full_length)
if full_handle < 0 then
   do
      say program_name 'Unable to set full message trap, handle =' full_handle
      return 'N'
   end

/* inform any body watching */
say program_name 'Started for Emulator Card' emulator 'for Trigger' "'"trigger"'"
say program_name 'Card' emulator 'WTOR Handle' trap_handle
say program_name 'Card' emulator 'Full Message Handle' full_handle

/* main polling loop */
do forever

   /* check for WTOR */
   current.wtor = 'AFR_CHECK_WTOR'(trap_handle,1000)

   /* if found, process it
      The data found should appear as follows:
      <time stamp> <message #> <WTOR #> <trigger> <script> <arguments>

      Actual example:
      *08.14.52 STC28328 *43 !ARNETV NETSAMP.REX 1

      The following parses with each blank delimited field into the
      array variable current.wtor.field.index where index is the indexing
      variable.  When the trigger field is found the field preceding
      contains the WTOR # and the remainder of the line contained by
      the variable current.wtor .
      */

   if length(current.wtor) > 0 then
      do
         /* initialize index and flag */
         index = 0
         found = 0

         /* parse out blank delimited fields */
         do while found = 0
            index = index + 1
            parse var current.wtor field.index current.wtor

            /* test for field containing trigger pattern */
            if trigger = field.index then
               do
                  /* previous field contains WTOR # */
                  wtor_index = index - 1
                  current.wtor.number = field.wtor_index
                  found = 1
               end

         end

         /* if current not equal to previous then attempt a paging call */
         if current.wtor.number \= prev.wtor.number then
            do
            say program_name 'Found:' current.wtor.number trigger current.wtor
               prev.wtor.number = current.wtor.number
               current.wtor.age = 0
               ret = wtorproc(current.wtor.number,current.wtor)
            end
         else
            do
               current.wtor.age = current.wtor.age + 1
               if current.wtor.age > wtor_age then
                  prev.wtor.number = -1;
            end

         /* wait a little bit */
         ret = 'AFR_SUSPEND'(500)

         /* reset the trap */
         trap_handle = 'AFR_SET_WTOR'(trigger,trap_start,trap_stop,data_length)
         say program_name 'Card' emulator 'Handle' trap_handle

      end

   /* check to see if MVS scroll area over full*/
   full_data = 'AFR_CHECK_TRAP'(full_handle,0)
   if full_data \= '' then
      do
         /* will erase the first line of the scroll area, might lose a Beeper
            WTOR in the process */
         ret = 'AFR_SEND_3270'('@0@RK E,1@E')
         full_handle = 'AFR_SET_TRAP'(full_trigger,full_start,full_stop,full_offset,full_length)
      end

   /* let other tasks have a little time */
   ret = 'AFR_SUSPEND'(1000)
end

return 'N'

/* branch to this point if program halted from AF/REMOTE Client Scripts window */
halt:
   result = program_name 'Terminated by Operator'
   say result
   ret = 'AFR_ERROR'(result,1)
   ret = 'AFR_CLEAR_TRAP'(full_handle)
   ret = 'AFR_CLEAR_WTOR'(trap_handle)
/* whatever else needs to be done to clean up after a shutdown */
   exit 'N'

/* branch to this point if novalue error encountered */
novalue:
   result = program_name 'Terminated for Novalue, Line' SIGL
   say result
   ret = 'AFR_ERROR'(result,0)
   ret = 'AFR_CLEAR_TRAP'(full_handle)
   ret = 'AFR_CLEAR_WTOR'(trap_handle)
/* whatever else needs to be done to clean up after a No Value shutdown */
   exit 'N'

/* branch to this point if syntax error encountered or function call in error */
syntax:
   result = program_name 'Terminated for Syntax, Line' SIGL 'Code' RC
   say result
   ret = 'AFR_ERROR'(result,0)
   ret = 'AFR_CLEAR_TRAP'(full_handle)
   ret = 'AFR_CLEAR_WTOR'(trap_handle)
/* whatever else needs to be done to clean up after a syntax shutdown */
   exit 'N'

/* this procedure initiates and monitors the results of a beeper paging call
   Inputs:
      Arg(1) Extracted WTOR number field in the form *nn where n in the set
               <0123456789>.
      Arg(2) Extracted WTOR data in the form:
               <Page type><Telephone Number> <PID> <Message>
                    Example:  'M9,395-7407 911420 This is a test'
   Outputs 'Y' for a successful Beeper page,
           'N' for an unsuccessful Beeper page.
*/

wtorproc:

/* retrieve arguments */
temp = arg(1)
parse var temp wtor_number .
wtor_at = pos('@',wtor_number)
wtor_as = pos('*',wtor_number)
wtor_pos = wtor_at + wtor_as + 1
wtor_number = substr(wtor_number,wtor_pos)

temp = arg(2)
parse var temp called_name msg

/* strip off leading and trailing blanks from arguments */
msg = strip(msg)

/* enclose in quotes so that any embedded commas will pass through
   REXX parser */
msg1 = "'"msg"'"

/* inform anybody watching */
say program_name 'Going to call' called_name"("msg")"

call_string = "answer = "called_name"("msg1")"
interpret call_string

say program_name 'Call returned:' answer

/* validate WTOR number, range 10 exp wtor_length */
do i = 1 to length(wtor_number)
   if pos(substr(wtor_number,i,1),'0123456789') = 0 then
      do
         say program_name 'Invalid WTOR Number' wtor_number
         return 'Failed'
      end
end

/* display arguments */
say program_name 'WTOR' wtor_number 'Response is' answer

/* format response */
string = '@0@RR 'wtor_number','answer' @E'
say program_name 'Responded' string

/* send response */
ret = 'AFR_SEND_3270'(string)

/* indicate success */
return 'Done'
