/*************************** REXX *********************************************
   PEERSOCK.REX
   27-Oct-1999 10:45:00

   Purpose:
      Example of a REXX socket program which can converse with an AF/REMOTE
      server.  This program is designed so that it can be run either inside
      or outside AF/REMOTE.
 
      The program first performs initial ID record handshaking with its
      AF/REMOTE peer and then sends the name of a REXX script to execute.
      Up to 100 bytes of reply data are requested.  The reply data must
      be returned via an AFR_PEER_SEND(, DATARPLY, data) API call in the
      triggered REXX script running under AF/REMOTE.                   

   Modification history:
      October 27, 1999 - Dan Biskar - Initial implementation    

 ******************************************************************************/


/* set some error detection signals */
signal on halt
signal on error


/*
   Supply as parms:
   1) the IP address of the target AF/REMOTE Server
   2) the peer port number of the target AF/REMOTE Server
   3) the name, along with any parameters, of the REXX script to execute
          on the AF/REMOTE peer system
*/

parse arg destaddr destport exec_data

/* Determine the number of bytes to be sent to the AF/REMOTE Server */
exec_data      = 'ex' exec_data		/* Insert the "EX" literal  */   
exec_length    = Length(exec_data)

Say 'The Destination TCP/IP Address =' destaddr
Say 'The Destination TCP/IP Port ='  destport
Say 'The REXX exec to execute =' exec_data
Say 'The data length to be sent =' exec_length


/* Main routine */
MAIN:
call INITIALIZE
call SOCKINIT
call CONNECT
call SEND_ID
call RECEIVE_ID
call SEND_EXEC
call RECEIVE_REPLY
call CLOSESOCK
RETURN


/* Load Object REXX's RxSock function package */
INITIALIZE:
if RxFuncQuery('SockLoadFuncs') <> 0 then
   do
      rc = RxFuncAdd("SockLoadFuncs", "rxSock", "SockLoadFuncs")
      if rc <> 0 then
         do
	    say 'Unable to load RxSock function package, rc:' rc
            exit
         end
      rc = SockLoadFuncs()
   end
RETURN


/* Obtain a socket descriptor handle to use throughout this program */
SOCKINIT:
sockhandle     = socksocket('AF_INET','SOCK_STREAM','0')
say 'Using socket handle:' sockhandle
address.family = AF_INET
address.addr   = destaddr
address.port   = destport
/* The following RxSock API call sets an upper limit of 5 seconds
   for any sockrecv call made in this REXX script.  This prevents
   indefinite hangs while waiting to receive reply data.	    */ 
rc = SockSetSockOpt(sockhandle, 'SOL_SOCKET', 'SO_RCVTIMEO', 5000)
RETURN


/* Connect to the AF peer system */
CONNECT:
rc = sockconnect(sockhandle, address.)
if rc = -1 then
   do
      say 'Connect failed with errno: ' SockSock_Errno()
      call closesock
      exit 
   end
say 'Socket connect successful'
RETURN


/* Send the initial ID record to start the conversation			*/
SEND_ID:

/* Note that position matters in the following sendstring.  For example,
   the "ID" datatype field must begin in the 9th position.  If the ID
   parm string is shortened or lengthened for any reason, the D2C(41)
   send length value must be altered accordingly.			*/   

sendstring = "RPLINK  ID      " || D2C(0) || D2C(0) || D2C(41) || D2C(0) || "NAME=MYAPP;VERSION=1;DATA=ASCII;REPLY=YES"
rc = socksend(sockhandle, sendstring)
if rc = -1 then
   do
      say 'SockSend failed with errno:' SockSock_Errno()
      call closesock
      exit 
   end
say 'Number of ID bytes sent:' rc
RETURN


/* Receive the acknowledgement ID record from the AF peer system */
RECEIVE_ID:
rc = sockrecv( sockhandle, rcvstring, 100 )
if rc = -1  |  rc = 0 then
   do
      say 'RECEIVE_ID failed with errno:' SockSock_Errno()
      call closesock
      exit 
   end
say 'Number of bytes received:' rc

/* The following logic tries to verify that the data received is the
   acknowledgement ID record we're expecting from the AF peer.  You can
   add other validation checks as needed.			       */

offset1 = pos("ID      ",   rcvstring, 9)	/* Check for ID record datatype       */
offset2 = pos("REPLY=NO",   rcvstring, 21)	/* Check for REPLY=NO acknowledgement */
if offset1 <> 0  &  offset2 <> 0 then 
   do
      say 'Received valid ID record'
   end
else
   do
      say 'Did not receive valid ID record.  Terminating connection...'
      call closesock
      exit
   end
RETURN


/* Send the name of a REXX exec to run on the AF peer system - Request 100 bytes of reply data */
SEND_EXEC:
sendstring = "RPLINK  EXEC    " || D2C(100) || D2C(0) || D2C(exec_length) || D2C(0) || exec_data
rc = socksend(sockhandle, sendstring)
if rc = -1 then
   do
      say 'SEND_EXEC failed with errno:' SockSock_Errno()
      call closesock
      exit 
   end
say 'Number of EXEC bytes sent:' rc
RETURN


/* Receive the expected reply data from a previous send request */
RECEIVE_REPLY:
rc = sockrecv( sockhandle, rcvreply, 100 )
if rc = -1  |  rc = 0 then
   do
      say 'SockRecv failed with errno: ' SockSock_Errno()
      call closesock
      exit 
   end
say 'Number of bytes received:' rc
/* Note: The first 20 bytes of any peer message are header information,
         so we look for the actual reply data starting in position 21.  */
parse var rcvreply 21 reply
say 'reply data =' reply
RETURN


/* Close the socket before exiting */
CLOSESOCK:
rc = sockclose(sockhandle)
if rc = 0 then
   say 'Socket close successful'
else
   say 'Socket close failed, rc:' SockSock_Errno()
RETURN


/* Error detection handling	   */
Halt:
/* Close the socket before exiting */
call closesock
EXIT

Error:
/* Close the socket before exiting */
call closesock
EXIT