/******************************* REXX ****************************************
   SUBFUNCT.REX
   08-Jul-1999 10:15:00

   Purpose:  Provides a script to display configuration parameters stored in
             the AF/REMOTE Server Configuration file.

   Inputs:   One or more of the following, separated by spaces:
               'TA'  for Time Acquisition,
               'BP'  for Beeper Parameters,
               'SY'  for System Parameters,
               'AP'  for ports per ARTIC card,
               'An'  for COM Port n Parameters,
               'Amn' for ARTIC Card m Port n Parameters,
               'En'  for Emulator Card n Parameters

   Outputs:  Environment Label and its associated value

 *****************************************************************************/

/* retrieve the name of the source program */
parse source name
program_name = substr(name,lastpos('\',name)+1)':'

say program_name "Started at" time() "on" date()

/* set some error detection signals */
signal on halt
signal on novalue
signal on syntax

/* get parameters */
parse upper arg args
log = pos('log',args)

/* initial delay */
display_wait = 0

/* delay between parameter groups */
reset_wait = 5000

/* available parameters */
label.1  = "TA_RATE"
label.2  = "TA_BITS"
label.3  = "TA_PARITY"
label.4  = "TA_STOP"
label.5  = "TA_PHONE"
label.6  = "TA_CALL"
label.7  = "TA_INTERVAL"
label.8  = "TA_AUTOSTART"
label.9  = "TA_REXXSTART"
label.10 = "TA_REXXCALL"

label.20 = "BP_RATE"
label.21 = "BP_BITS"
label.22 = "BP_PARITY"
label.23 = "BP_STOP"
label.24 = "BP_TRIGGER"
label.25 = "BP_AUTOSTART"
label.26 = "BP_REXXSTART"
label.27 = "BP_REXXCALL"

label.30 = "SYS_KEYBOARD"
label.31 = "SYS_LANGUAGE"
label.32 = "SYS_DATE"
label.33 = "SYS_TIME"
label.34 = "SYS_DATETIME"
label.35 = "SYS_TIMEZONE"
label.36 = "SYS_DST"
label.37 = "SYS_VERSION"
label.38 = "SYS_HOSTNAME"
label.39 = "SYS_SADTIMEOUT"

label.90 = "C1_PORTS"
label.91 = "C2_PORTS"
label.92 = "C3_PORTS"
label.93 = "C4_PORTS"

/* port parameter suffixes */
label.50 = "_USE"
label.51 = "_CLASS"
label.52 = "_TITLE"
label.53 = "_HOTKEY"
label.54 = "_LANGUAGE"
label.55 = "_SAD"
label.56 = "_LINE"
label.57 = "_RATE"
label.58 = "_BITS"
label.59 = "_PARITY"
label.60 = "_STOP"
label.61 = "_FLOW"
label.62 = "_TYPE"
label.63 = "_CTRL"
label.64 = "_MODEM"
label.65 = "_IDLETO"

/* emulator parameter suffixes */
label.70 = "_USE"
label.71 = "_CLASS"
label.72 = "_TITLE"
label.73 = "_HOTKEY"
label.74 = "_LANGUAGE"
label.75 = "_SCREEN"
label.76 = "_SAD"
label.77 = "_EABS"
label.78 = "_ADDRESS"
label.79 = "_MEMORY"
label.80 = "_PATH"
label.81 = "_NAME"
label.82 = "_PARM"

/* display each group */
port = ''
if pos('TA',args) > 0 then
   call display  1 10 "Time Acquisition Parameters"

if pos('BP',args) > 0 then
   call display 20 27 "Beeper Paging Parameters"

if pos('SY',args) > 0 then
   call display 30 39 "System Parameters"

if pos('AP',args) > 0 then
   call display 90 93 "Ports per ARTIC card"

do j = 1 to 4
   port = 'A'||j

   k = pos(port,args)
   do while k \= 0 & datatype(substr(args, k + 2, 1)) == "NUM"
      k = pos(port,args,k+1)
   end

   if k > 0 then
      call display 50 65 "Ascii Port" j "Parameters"
end

do j = 11 to 18
   port = 'A'||j
   if pos(port,args) > 0 then
      call display 50 65 "ARTIC Card" j % 10  "Port" j // 10 "Parameters"
end

do j = 21 to 28
   port = 'A'||j
   if pos(port,args) > 0 then
      call display 50 65 "ARTIC Card" j % 10  "Port" j // 10 "Parameters"
end

do j = 31 to 38
   port = 'A'||j
   if pos(port,args) > 0 then
      call display 50 65 "ARTIC Card" j % 10  "Port" j // 10 "Parameters"
end

do j = 41 to 48
   port = 'A'||j
   if pos(port,args) > 0 then
      call display 50 65 "ARTIC Card" j % 10  "Port" j // 10 "Parameters"
end

do j = 1 to 8
   port = 'E'||j
   if pos(port,args) > 0 then
      call display 70 82 "Emulator" j "Parameters"
end

/* advise completion of program */
say program_name "Terminated Normally at" time() "on" date()

/* just for drill */
return 'OK'

/* branch to this point if program halted from AF/REMOTE Client Scripts window */
halt:
   result = program_name 'Terminated by Operator'
   say result
   ret = 'AFR_ERROR'(result,1)
/* whatever else needs to be done to clean up after a shutdown */
   exit 'N'

/* branch to this point if novalue error encountered */
novalue:
   result = program_name 'Terminated for NoValue, Line' SIGL
   say result
   ret = 'AFR_ERROR'(result,0)
/* whatever else needs to be done to clean up after a No Value shutdown */
   exit 'N'

/* branch to this point if syntax error encountered or function call in error */
syntax:
   result = program_name 'Terminated for Syntax, Line' SIGL 'Code' RC
   say result
   ret = 'AFR_ERROR'(result,0)
/* whatever else needs to be done to clean up after a syntax shutdown */
   exit 'N'

display:

parse arg start stop label

if display_wait > 0 then
   rc = 'AFR_SUSPEND'(display_wait)
display_wait = reset_wait

ret = 'AFR_CLS'()
say label

do i = start to stop

   rc = 'AFR_GETENV'(port||label.i)
   text = port||label.i "'"rc"'"
   say text

   if log > 0 then
      rc = 'AFR_LOG'(text)
end

say ' '
return 'done'
