/******************************* REXX *****************************************
 SCRIPT: TIME.REX
   4 June, 2003 - 1255 PDT 
   
 PURPOSE: Script periodically starts a script (default name: TIMECALL.REX)
          based on next date and time meters.  At the successful conclusion of
          script, the next date and time are updated based on current time and
          the time interval obtained from the server configuration via
          AFR_GETENV(TA_CAL) calls.  Time.rex runs until manually halted 
          from the AF/REMOTE Client scripts window.

 Inputs:  None.

 Outputs: None.

 Modification history:
    16APR98 JMC Modified for use with AF/Remote for NT.
    
    08May01 JMC The AFR_SETENV(TA_CALL) is not working!.  it is essentially
        a NOP (no operation).  This version of Time.Rexx is being provided as
        a work around until this REXX function can be repaired.  
        
        1.  The timecall routine is called immediately at startup to update the
              current system time.
         2.  The TA_CALL parameter is retrieved to establish the timeof the
               next cyclic call to the timecall routine.  The next timecall time is
               the current date and the hour retreived from TA_CALL.  At that 
               time the timecall routine is called to update the time, and the next
               date/time to callis based on the current date/time and the data 
               retreived from TA_CALL.
    30 May 2003 JMC - AFR_GETENV('SYS_DATETIME') and AFR_GETENV('SYS_TIME') calls
        are returning time one hour off on some systems during daylight saving time.
        The calls to Object Rexx Time() and Date() functions return the correct time.
        Replaced all AFR_GETENV('SYS_DATETIME') calls with Date('o') Time('n')
        Replaced all AFR_GETENV('SYS_TIME') calls with Time('n')
        Replaced all AFR_GETENV('SYS_DATE') calls with Date('o')
    
 ******************************************************************************/

/* retrieve the name of the source program */
parse source name
program_name = substr(name,lastpos('\',name)+1)':'
call LogSay name
call LogSay "Started at" time() "on" date()

/* set some error detection signals */
signal on halt
/* signal on novalue */
signal on syntax

Debug = 0

call_delay_seconds = 15       /* initial call delay - seconds */
wait_delay_minutes = 60       /* maximum minutes between reacquisition tests */
wait_error_seconds = 89       /* minimum seconds between reacquisition tests */
/* acquire name of Time acquisition script */
TA_REXXCALL = AFR_GETENV('TA_REXXCALL')

/* if script name is blank then there is no point in continuing */
if TA_REXXCALL = '' then
   do
      call LogSay 'No Time Update Script Specified, Task Ending'
      return ' '
   end

/* here we unconditionally request a call to the timecall routine.*/
script_string = 'call' TA_REXXCALL
call LogSay 'line 68:' script_string
interpret script_string

/* acquire the time to make the next call */
TA_CALL = afr_getenv('TA_CALL')
if length(TA_CALL) = 0 then
    do
    call LogSay "Time Acquisition variable 'TA_CALL' not set"
    call AFR_ERROR "Time Acquisition variable 'TA_CALL' not set", 0
    TA_CALL = translate(Date('o'),'-','/') Time('n')
    end

/* Acquire interval between Time Acquisition calls */
TA_INTERVAL = AFR_GETENV('TA_INTERVAL')

/* acquire the current system date/time */
sys_datetime = translate(Date('o'),'-','/') Time('n')

/* let's sync execution time to the start of a minute */
second = right(SYS_DATETIME, 2);
Delay = 1000 * (60 - second)
call LogSay SYS_DATETIME Second delay
call afr_suspend Delay

SYS_DATETIME = translate(Date('o'),'-','/') Time('n')
SYS_DATE  = translate(Date('o'),'-','/') 
if Debug then
    call LogSay 'Current time:' translate(Date('o'),'-','/') Time('n')
sys_datetime = left(sys_datetime,14)

/* test for a non-zero reacquisition interval */
if TA_INTERVAL_DAYS = '0' & (TA_INTERVAL_TIME = '00:00' | TA_INTERVAL_TIME = '  :  ') then
   do
      call LogSay 'No Time Update Interval Specified, Task Ending'
      return ' '
   end
/* separate out days between call from time */
parse var TA_INTERVAL TA_INTERVAL_DAYS TA_INTERVAL_TIME

parse var TA_CALL DATE_TA_CALL TIME_TA_CALL

if Debug then
    do
    call LogSay
    call LogSay "Rexx script:" ta_rexxcall
    call LogSay "Repeat every" TA_INTERVAL_DAYS "Days," TA_INTERVAL_TIME "Hour:Minutes"
    call LogSay "Next call:" ta_call
    call LogSay "Current date/time:" sys_datetime
    call LogSay
    end
if sys_datetime >= TA_CALL then
    do
    /* force TA_CALL to current date, but don't change the time */
    DATE_TA_CALL = SYS_DATE
    TA_CALL_NEW = SYS_DATE TIME_TA_CALL
    call LogSay 'TA_CALL Old:'TA_CALL 'New:'TA_CALL_NEW
    TA_CALL = TA_CALL_NEW
    if Debug then
        call LogSay 'Local TA_CALL revised to:' TA_CALL
    /* Then we will wait until it's time to do a time acquisition */
    end
    
/* then we get into our forever loop to acquire time periodicaly */
call LogSay "---------------------------------------------------------------------------"
call LogSay

do forever
    SYS_DATE = translate(Date('o'),'-','/') 
    if Debug then
        call LogSay "Waiting for date match:" DATE_TA_CALL SYS_DATE
    do while SYS_DATE < DATE_TA_CALL 
        call afr_suspend 60000
        SYS_DATE = translate(Date('o'),'-','/') 
        end 

    if Debug then
        call LogSay "Dates match now find a time match"
    SYS_TIME = Time('n') 
    do while SYS_TIME < TIME_TA_CALL 
        /* wait for next time interval */
        if Debug then
            call LogSay "No time match:" TIME_TA_CALL SYS_TIME 
        call afr_suspend 60000  /* one minute */
        SYS_TIME = Time('n') 
        end

    call UpdateSystemTime
    parse var TA_CALL DATE_TA_CALL TIME_TA_CALL
    end /* do forever */

/* this instruction should never be reached !!! */
return ' '

/* branch to this point if program halted from AF/REMOTE Client Scripts window */
halt:
   result = 'Terminated by Operator'
   call LogSay result
   ret = 'AFR_ERROR'(result,1)
/* whatever else needs to be done to clean up after a shutdown */
   exit 'N'

/* branch to this point if novalue error encountered */
novalue:
   result = 'Terminated for NoValue, Line' SIGL
   call LogSay result
   ret = 'AFR_ERROR'(result,0)
/* whatever else needs to be done to clean up after a No Value shutdown */
   exit 'N'

/* branch to this point if syntax error encountered or function call in error */
syntax:
   result = 'Terminated for Syntax, Line' SIGL 'Code' RC
   call LogSay result
   ret = 'AFR_ERROR'(result,0)
/* whatever else needs to be done to clean up after a syntax shutdown */
   exit 'N'

/* subroutines, functions, procedures, ??? */
LogSay:
   text = arg(1)
   say time() program_name text
   x = afr_log(program_name text)
   return 0
   
UpdateSystemTime:  procedure expose Debug program_name TA_CALL TA_INTERVAL SYS_DATETIME
    /* Now we initiate a time acquisition.  If that is successful
        then update TA_CALL locally and in the RpServer environment
        */
    script_string = 'call' afr_getenv("TA_REXXCALL")
    Retries = 3
    do forever
        interpret script_string

        /* analyse result to Time Acquisition Call */
        if substr(result,1,3) \= 'ERR' then
            do
            call UpdateTimeControl
            leave
            end
        else
            do
            /* retry in one minute */
            call LogSay "Error returned by time acquisition program, retry"
            if Retries <= 0 then
                leave
            Retries = Retries -1
            call afr_suspend 60000
            end
        end /* do forever */
    return 

UpdateTimeControl:  procedure  expose Debug program_name TA_CALL TA_INTERVAL SYS_DATETIME

MDay.   = 31
MDay.0 = 0
MDay.2 = 28
MDay.4 = 30
MDay.6 = 30
MDay.9 = 30
MDay.11 = 30

    /* update our copy of system time */
    SYS_DATETIME = translate(Date('o'),'-','/') Time('n')
    SYS_DATE  = translate(Date('o'),'-','/') 
    call LogSay "System date/time updated to:" SYS_DATETIME

    /* update TA_CALL */
    if debug then
        do
        call LogSay "Interval:" TA_INTERVAL
        call LogSay "Current TA_CALL:" TA_CALL
        end
    parse var TA_CALL YR "-" MON "-" DAY HR ":" MIN
    parse var TA_INTERVAL DAYS HOURS ":" MINUTES
    call logsay 'Day:'days 'Hours:'Hours 'Minutes:'Minutes
    HR = HR + HOURS
    DAY = DAY + DAYS
    if HR > 23 then
        do
        HR = HR - 24
        DAY = DAY + 1
        end /* Hr > 23 */
    if DAY > MDay.MON then
        do
        Day = DAY - MDay.MON 
        Day = Day + 1
        MON = MON + 1
        end /* MON > 12 */
    if MON > 12 then
        do
        MON = 1
        YR = YR + 1
        end /* DAY > MDay.mon */
    /* reassembe TA_CALL */
    if length(YR) = 1 then
        YR = "0"||YR
    if length(MON)  = 1 then
        MON = "0"||MON
    if length(DAY) = 1 then
        DAY = "0"||DAY
    if length(HR) = 1 then
        HR = "0"||HR
    TA_CALL = YR'-'MON'-'DAY HR':'MIN
    call afr_setenv "TA_CALL", TA_INTERVAL
    parse var TA_CALL DATE_TA_CALL TIME_TA_CALL
    call LogSay "Updated TA_CALL:" TA_CALL
    call LogSay "---------------------------------------------------------------------------"
    call LogSay
    
return