/******************************* REXX *****************************************
 SCRIPT: TIMECALL.REX
   16-Apr-1998 13:48:38

 PURPOSE: Implements acquisition of current Time of Day from the National
          Bureau of Standards provided time inquiry service.

 SYNOPSIS:
    SUCCESS = call timecall.rex
       where SUCCESS = character string indicating result of attempted Time
                       of Day Acquisition.

 Modification History:
    Added ret = AFR_MODEMDISCONNECT(handle) statements for NO CONNECT
    error path and signal handlers (lines 72, 209, 219, 220)
    JMC 29 Sept 1992

    Provide capability to run modem outdial/disconnect using high-level
    AFR_MODEMCALL()/AFR_MODEMDISCONNECT() functions OR using the
    lower level AFR_COMPUTSTR(), AFR_COMGETSTR(), AFR_COMCTRL(), and
    associated function calls.  Selected by setting the variable
    HighLevel to 1 or 0 respectively, default 0.
    JMC 1 November 1994

 ******************************************************************************/

/* retrieve the name of the source program */
parse source name
program_name = substr(name,lastpos('\',name)+1)':'

say program_name "Started at" time() "on" date()

handle      = 0             /* handle returned by AFR_COMOPEN() */
HighLevel   = 0             /* selects high or level outdial logic */
ShowSends   = 1             /* enables display of strings sent, diagnostic
                               aid */
ShowRcvs    = 1             /* enables display of string received, diagnostic
                               aid */
ShowDateTime= 1             /* enables display of date/time stamp at start and
                               end of IXO sequence */
Broadcast   = 0             /* enables broadcast of paging activity to console
                               sessions */
LogActivity = 1             /* enables logging of computer to computer dialog
                               diagnostic aid */
Color       = 0             /* color of broadcast message, set by program */

ReceivedData = ''

CR          = 'd'x
LF          = 'a'x
DCD         = '8'x


Constants   = "handle HighLevel ShowSends ShowRcvs ShowDateTime Broadcase",
              "LogActivity Color ReceivedData CR LF DCD"

/* set some error detection signals */
signal on halt
signal on novalue
signal on syntax

/* set up success announcement */
success = "ERROR: TIME REQUEST FAILED DUE TO BAD DATA"

/* retrieve the GMT to local time zone adjustment data */
timezone = 'AFR_GETENV'("SYS_TIMEZONE")

/* set for time to wait for modem to connect */
DIAL_TO = '120000'     /* takes time to get to Colorado */

/* Acquire telephone number for ACTS from server config */
phone = 'AFR_GETENV'("TA_PHONE")
advise = 'CALLING ACTS ON' phone

/* establish connection with paging service */
handle = 'AFR_COMOPEN'(0)
if handle = 0 then
   do
      success = 'ERROR: THE MODEM IS BUSY'
      say program_name success
      if Broadcast then
          rc = 'AFR_ERROR'(success,0)
      else
          if LogActivity then
              rc = 'AFR_LOG'(success)
      return success
   end

say program_name advise
if Broadcast then
    rc = 'AFR_ERROR'(advise,1);
else
    if LogActivity then
        rc = 'AFR_LOG'(advise)

/* Acquire modem port characteristics from server config */
rate = 'AFR_GETENV'("TA_RATE")
bits = 'AFR_GETENV'("TA_BITS")
stop = 'AFR_GETENV'("TA_STOP")
parity = 'AFR_GETENV'("TA_PARITY")

/* Now set up modem port */
rc = 'AFR_COMSETBAUD'(handle,rate)
rc = 'AFR_COMSETLINEC'(handle,bits,stop,parity);

/* Dial the telephone */
if HighLevel then  /* use high-level AFR_MODEMCALL() ? */
  do
  rc = 'AFR_MODEMCALL'(handle,phone,DIAL_TO);
  mstat = MODSTAT.REX(rc)
  end
else                /* do outcall with low-level comm transmission
                       functions */
  MSTAT = OUTCALL(HANDLE,PHONE,DIAL_TO)


/* evaluate the return from the modem call */
if mstat \= 'CONNECT' then
   do
      /* analyse rc for reason of connection failure */
      success = 'ERROR: MODEM' mstat
      say program_name success
      if Broadcast then
          rc = 'AFR_ERROR'(success,0)
      else
          if LogActivity then
              rc = 'AFR_LOG'(success)
      if HighLevel then
          rc = 'AFR_MODEMDISCONNECT'(handle)
      else
          rc = 'AFR_COMCTRL'(handle,0,1)

            rc = 'AFR_COMCLOSE'(handle)
            return success
   end

/* set up to process data from ACTS */
records     = 20    /* maximum number of ACTS records to try */
record      = 0     /* last ACTS record */

PosMJD      = 1
PosDate     = 2
PosTime     = 3
PosDST      = 4
PosLS       = 5
PosDUT1     = 6
PosmsADV    = 7
PosNIST     = 8
PosOTM      = 9

/* search ACTS data for time records */
do while record < records

    /* initialize field count variable */
    field = 0

    /* get the data from the com port */
    TimeData = ReceiveLine()

    /* check for modem disconnect */
    if pos("NO CARRIER",TimeData) > 0 then
       leave

    /* record of data appears as follows:
       48009 90-04-17 18:50:30 50 0 +.1 045.0 UTC(NIST) *<CR><LF>
       */

    /* do while data to be parsed */
    do while TimeData \= ''
       field = field + 1
       parse var TimeData temp.field TimeData
       end

    if field >= PosOTM then
        do
        record = record + 1
        NistDate.record = temp.PosDate
        NistTime.record = temp.PosTime
        NistDst.record  = temp.posDST
        end

    /* if we have acquired at least two records of data */
    if record < 2 then
       iterate

    previous = record - 1
    /* the dates from those records are the same */
    if NistDate.record  <> NistDate.previous then
       iterate

    /* the times are equal except for seconds */
    if substr(NistTime.record,1,7) = substr(NistTime.previous,1,7) then
       do
       /* then update system time */
       rc = 'AFR_GMT'(timezone NistDate.record NistTime.record)
       if rc = 0 then
          do
          success = "DATE/TIME SUCCESSFULLY UPDATED!"

          /* are we in the countdown period for standard/daylight changeover */
          DaysToGo = NistDst.record
          if Daystogo > 49 then
              do
              period = 'Daylight savings'
              DaysToGo = DaysToGo - 50
              end
          else
              period = 'Standard'
          if DaysToGo > 0 then
            say program_name DaysToGo 'day(s) until change to' period 'time'

          leave
          end
       end
end

/* give completion message */
say program_name success
flag = 1
if substr(success,1,3) = "ERR"
   then flag = 0
if Broadcast then
    rc = 'AFR_ERROR'(success,flag)
else
    if LogActivity then
        rc = 'AFR_LOG'(success)

now = program_name 'Completed at' date() time()
if ShowDateTime then
    do
    if Broadcast then
        rc = 'AFR_ERROR'(now,1)
    else
        rc = 'AFR_LOG'(now)
    say now
    end


/* break the connection and release the resource */
if HighLevel then
    rc = 'AFR_MODEMDISCONNECT'(handle)
else
    rc = 'AFR_COMCTRL'(handle,0,1)

rc = 'AFR_SUSPEND'(2000)
rc = 'AFR_COMCLOSE'(handle)

return success

/* branch to this point if program halted from AF/REMOTE Client Scripts window */
halt:
   result = program_name 'Terminated by Operator'
   say result
   if Broadcast then
       rc = 'AFR_ERROR'(result,1)
   else
       if LogActivity then
           rc = AFR_LOG(result)

   if HighLevel then
      rc = 'AFR_MODEMDISCONNECT'(handle)
   else
      rc = 'AFR_COMCTRL'(handle,0,1)

   ret = 'AFR_COMCLOSE'(handle)
/* whatever else needs to be done to clean up after a shutdown */
   exit 'Y'


/* branch to this point if novalue error encountered */
novalue:
   result = program_name 'Terminated for Novalue, Line' SIGL
   say result
   if Broadcast then
       rc = 'AFR_ERROR'(result,0)
   else
       if LogActivity then
           rc = AFR_LOG(result)

   if HighLevel then
      rc = 'AFR_MODEMDISCONNECT'(handle)
   else
      rc = 'AFR_COMCTRL'(handle,0,1)

   ret = 'AFR_COMCLOSE'(handle)
/* whatever else needs to be done to clean up after a No Value shutdown */
   exit 'N'

/* branch to this point if syntax error encountered or function call in error */
syntax:
   result = program_name 'Terminated for Syntax, Line' SIGL 'Code' RC
   say result
   if Broadcast then
       rc = 'AFR_ERROR'(result,0)
   else
       if LogActivity then
           rc = AFR_LOG(result)

   if HighLevel then
      rc = 'AFR_MODEMDISCONNECT'(handle)
   else
      rc = 'AFR_COMCTRL'(handle,0,1)

   ret = 'AFR_COMCLOSE'(handle)
/* whatever else needs to be done to clean up after a syntax shutdown */
   exit 'Y'

ReceiveLine:

   do while afr_comstatus(handle) >= DCD
      /* wait a bit for data to come in, 100 msec is 12 bytes at 1200 baud */
      rc = 'AFR_SUSPEND'(100)

      /* get the data from the com port */
      Available = 'AFR_COMINPUTBYTES'(handle)
      ReceivedData = ReceivedData||'AFR_COMGETSTR'(handle,Available)

      /* check for modem disconnect */
      if pos("NO CARRIER",ReceivedData) > 0 then
         return 'NO CARRIER'

      /* check for '*' which is the time hack each second */
      Pos = pos('*',ReceivedData)
      if Pos > 0 then
        do
        DataLine = left(ReceivedData, Pos)
        ReceivedData = substr(ReceivedData, Pos + 1)
        if ShowRcvs then
            do
            say program_name 'Received:' DataLine
            if LogActivity then
                call afr_log DataLine
            end
        return DataLine
        end

      LFPos = pos(LF, ReceivedData)
      CRPos = pos(CR, ReceivedData)
      if LFPos > CRPos then
         Pos = LFPos
      else
         Pos = CRPos
      if Pos > 0 then
        do
        DataLine = left(ReceivedData, Pos)
        ReceivedData = substr(ReceivedData, Pos + 1)
        if ShowRcvs then
            if length(DataLine) > 2 then
               say program_name 'Received:' DataLine
        end

      end

   return 'NO CARRIER'

/* OUTCALL implements the outdial to a modem, and evaluates the response
   returned by the modem as a result of the outdial attempt */

OUTCALL:

   RESP_SIZE = AFR_COMINPUTBYTES(HANDLE)
   IF RESP_SIZE > 0 THEN
      RC1 = AFR_COMGETSTR(HANDLE,RESP_SIZE)

   RESPONSE = ''

   RC1 = AFR_COMCTRL(HANDLE,1,1)
   RC1 = AFR_COMPUTSTR(HANDLE,'ATDT'PHONE,1)
   ODELAY = 60;
   DO WHILE ODELAY > 0
      RC1 = AFR_SUSPEND(1000)
      RESP_SIZE = AFR_COMINPUTBYTES(HANDLE)
      IF RESP_SIZE > 0 THEN
         DO
         RESPONSE = RESPONSE||AFR_COMGETSTR(HANDLE,RESP_SIZE)
         SELECT
            WHEN POS('CONNECT', RESPONSE) > 0 THEN
               RETURN 'CONNECT'
            WHEN POS('BUSY', RESPONSE) > 0 THEN
               RETURN 'BUSY'
            WHEN POS('ERROR', RESPONSE) > 0 THEN
               RETURN 'ERROR'
            WHEN POS('NO CARRIER', RESPONSE) > 0 THEN
               RETURN 'NO CARRIER'
            WHEN POS('NO DIALTONE', RESPONSE) > 0 THEN
               RETURN 'NO DIALTONE'
            WHEN POS('NO ANSWER', RESPONSE) > 0 THEN
               RETURN 'NO ANSWER'
            OTHERWISE
               NOP
            END
         END

      ODELAY = ODELAY - 1
      END

   return 'TIMED OUT'
