/*************************** REXX *********************************************
   SCRIPT: WTORSCAN.REX
   27-Sep-1999 08:50:10

   PURPOSE: Continuously scans assigned emulator for WTOR with trigger string
            defined in the Trigger Text entry field on the AF/REMOTE Server
            Configuration Notification property page.

   Inputs:  Arg(1): Emulator Card title to monitor.
            Arg(2): Number of lines on the terminal screen (24 or 43).

   Outputs: Nothing.

   Calls:   Program named in external variable 'BP_REXXCALL' (retrieved by
            external function 'AFR_GETENV' ).

   Modification History:
            July 3, 1995, Jim Collins, Candle Corporation.
            Added logic to send paging data to a data array instead of
            performing direct calls to the paging program.  This is done by
            setting the variable Queues to 1.  Setting this variable to 0
            results in the program initiating beeper calls in the original
            manner.
            April 26, 1996, Jim Collins, Candle Corporation
            Added control variable Shell which indicates the program to be
            called to handle the paging call.  See comment accompanying that
            variable.
            August 21, 1996, Jim Collins, Candle Corporation
            Changed search for trigger word such that the trigger doesn't
            need to match the whole word in the WTOR line containing the
            trigger pattern.  Purpose of the search is to identify the
            reply number, and the position of the text following the reply
            number.

            September 24, 1999, Jim Collins.  AF/Remote for NT,
            AFR_GETENV(E1_Screen) returns a value 1920 rather than a string
            showing lines and columns as in AF/Remote for OS/2.  Revised the
            mathmatics operations to correctly compute the stop point for
            AFR_SET_WTOR() and AFR_SET_TRAP().

 ******************************************************************************/

/* retrieve the name of the source program */
parse source name
program_name = substr(name,lastpos('\',name)+1)':'

say program_name "Started at" time() "on" date()

/* set some error detection signals */
signal on halt
signal on novalue
signal on syntax

/* constant and variable data definition */
Queues                = 0                /* set to 1 to send pages to a queue
                                            set to 0 to call paging routine
                                            directly */
QueueName             = 'BEEPQUED'       /* name of beeper queue */
Shell                 = 0                /* 1, BEEPQSHL.REX, 0, BEEPCALL.REX */
FirstMsg              = 0
LastMsg               = 0
test_mode             = 0                /* set to 1 to not do paging call */
multi_line_wtor       = 1                /* set to 1 to process wtor which wraps
                                            to the second line on MVS(MCS)
                                            console */
EnableClearMsgWaiting = 0                /* set to 1 to automatically clear
                                            IEE159E Message Wait */
EnableClearHold       = 0                /* set to 1 to automatically clear
                                            mode = HOLD */
trap_handle           = 0                /* handle for current trap */
trap_lines            = 24               /* number lines to search */
console_columns       = 80               /* number columns/line */
trap_start = 0 * console_columns         /* start point of search */
trap_stop = trap_lines * console_columns /* end point of search */
data_offset           = 0                /* offset of text from found */
data_length           = 79               /* length of data to return */
if (multi_line_wtor) then
    data_length       = 159
prev.wtor.number      = -1               /* extracted wtor number */
current.wtor          = ''               /* text of current wtor */
current.wtor.number   = -2               /* extracted wtor number */
full_trigger          = 'IEE159E'        /* trigger for MVS full message */
full_start = 22 * console_columns        /* start of line for full message */
full_stop = full_start + console_columns + console_columns
                                         /* end of line for full message */
full_offset           = 0                /* offset of text from found */
full_length           = 160              /* length of text */
wtor_age              = 15               /* number of seconds until wtor
                                                   is tried again */
CommandPos            = 1682             /* 3278 Model 2 offsets */

/* General Setup and Initialization */

/* get trigger specified in Trigger Text field on Server Config Notification page */
trigger = 'AFR_GETENV'('BP_TRIGGER')

/* get emulator assignment arguments and adjust screen size dependent data */
parse arg emulator
i = words(emulator)

if i < 2 then
   do
      Say program_name 'Not Started, Insufficient Parameters Specified'
      return 'failed'
   end

screen_length = subword(emulator, i)
emulator = subword(emulator, 1, i-1)

if datatype(screen_length, 'N') \= 1 then
   do
      Say program_name 'Not Started, Screen Lines Parameter Invalid'
      return 'failed'
   end

trap_lines = screen_length % console_columns
trap_stop = trap_lines * console_columns
full_start = (trap_lines - 2) * console_columns
full_stop = trap_stop

/* try to select emulator */
ret = 'AFR_SELECT'(emulator)
if ret \= 0 then
   do
      say program_name 'Terminated, Error Selecting Emulator Card' emulator', RC =' ret
      return 'failed'
   end

ret = afr_load(0, 9999)
if 2000 < length(ret) then
   CommandPos = 3202           /* 3278 Model 4 offsets */

XPos  = CommandPos + 246

/* verify it's an MVS beeper console */
console = 'AFR_CONSOLE_TYPE'()
flag = pos("Beeper",console)
if flag = 0 then
   do
      say program_name 'Terminated, Not an MVS Beeper Console' emulator
      return "Invalid Console Type"
   end

/* set up to find a WTOR */
trap_handle = 'AFR_SET_WTOR'(trigger,trap_start,trap_stop,data_length)
if trap_handle < 0 then
   do
      say program_name 'Unable to set wtor trap, trap_handle =' trap_handle
      return 'N'
   end

/* check to see if MVS scroll area over full*/
full_handle = 'AFR_SET_TRAP'(full_trigger,full_start,full_stop,full_offset,full_length)
if full_handle < 0 then
   do
      say program_name 'Unable to set full message trap, handle =' full_handle
      return 'N'
   end

/* inform anybody watching */
say program_name 'Started for Emulator Card' emulator 'for Trigger' "'"trigger"'"
say program_name 'Card' emulator 'WTOR Handle' trap_handle
say program_name 'Card' emulator 'Full Message Handle' full_handle

/* main polling loop */
do forever

   /* check for WTOR */
   current.wtor = 'AFR_CHECK_WTOR'(trap_handle,1000)

   /* if found, process it
      The data found should appear as follows:
      <time stamp> <message #> <WTOR #> <trigger> <message ... >

      Actual example:
      *08.14.52 STC28328 *43 BEEPJC m9,849-1050 201532 This ia a test.

      The following parses with each blank delimited field into the
      array variable current.wtor.field.index where index is the indexing
      variable.  When the trigger field is found the field preceding
      contains the WTOR # and the remainder of the line contained by
      the variable current.wtor .
      */

   if length(current.wtor) > 0 then
      do
         /* initialize index and flag */
         index = 0
         found = 0

        /* Support for second line processing, conditionally enabled */
        if (multi_line_wtor) then
           do
           /* get first line of wtor */
           FirstLine = left(current.wtor,80)
           /* the remainder will be the second line */
           SecondLine = substr(current.wtor,81)
           /* save the first line regardless MLWTOR or not */
           current.wtor = FirstLine
           /* look for the wtor tag '*' or '@' */
           do Tags = 1 to 2
              /* select tag to test for */
              TagChr = substr('*@',Tags,1)
              /* test for tag in First Line */
              LeftTagPos = pos(TagChr,FirstLine)
              /* if tag not found then abort further testing with this tag */
              if LeftTagPos = 0 then
                iterate
              /* if valid WTOR tag shows up twice in the first line */
              RightTagPos = pos(TagChr,FirstLine,LeftTagPos+1)
              /* if second tag not found, abort further test with this tag */
              if RightTagPos = 0 then
                iterate
              /* MLWTOR the same tag will be at same position, both lines */
              if substr(SecondLine,LeftTagPos,1) = Tagchr then
                  do
                  /* but if second tag found then second line is another WTOR */
                  if substr(SecondLine,RightTagPos,1) <> Tagchr then
                    /* must be MLWTOR */
                    current.wtor = FirstLine||substr(SecondLine,LeftTagPos + 1)
                  end
              end /* do Tags */
           end /* if (multi_ */

         /* parse out blank delimited fields */
         do while found = 0
            index = index + 1
            parse var current.wtor field.index current.wtor

            /* test for field containing trigger pattern */
            if pos(trigger,field.index) > 0 then
               do
                  /* previous field contains WTOR # */
                  wtor_index = index - 1
                  current.wtor.number = field.wtor_index
                  found = 1
               end

         end

         /* if current not equal to previous then attempt a paging call */
         if current.wtor.number \= prev.wtor.number then
            do
            say program_name 'Found:' current.wtor.number trigger current.wtor
               prev.wtor.number = current.wtor.number
               current.wtor.age = 0
               ret = wtorproc(current.wtor.number,current.wtor)
            end
         else
            do
               current.wtor.age = current.wtor.age + 1
               if current.wtor.age > wtor_age then
                  prev.wtor.number = -1;
            end

         /* wait a little bit */
         ret = 'AFR_SUSPEND'(500)

         /* reset the trap */
         trap_handle = 'AFR_SET_WTOR'(trigger,trap_start,trap_stop,data_length)
         say program_name 'Card' emulator 'Handle' trap_handle

      end

   /* check to see if MVS scroll area over full*/
   full_data = 'AFR_CHECK_TRAP'(full_handle,0)
   if full_data \= '' then
      do
         /* will erase the first line of the scroll area, might lose a Beeper
            WTOR in the process */
         n = SendMvs('K E,1')
         full_handle = 'AFR_SET_TRAP'(full_trigger,full_start,full_stop,full_offset,full_length)
      end

   if FirstMsg > LastMsg then
      do
      chkPort = afr_comopen(0)
      if chkPort <> 0 then
         do
         n = afr_comclose(chkPort)
         if Shell then
            /* call the shell program to capture the return string */
            n = AFR_START_PROGRAM('BEEPQSHL.REX' emulator'+/+'WtorNumber.LastMsg Message.LastMsg)
         else
            /* call the paging program directly */
            n = AFR_START_PROGRAM('BEEPCALL.REX' Message.LastMsg)
         drop Message.LastMsg WtorNumber.LastMsg
         LastMsg = LastMsg + 1
         end
      end

   /* let other tasks have a little time */
   ret = 'AFR_SUSPEND'(1000)
end

return 'N'

/* branch to this point if program halted from AF/REMOTE Client Scripts window */
halt:
   result = program_name 'Terminated by Operator'
   say result
   ret = 'AFR_ERROR'(result,1)
   ret = 'AFR_CLEAR_TRAP'(full_handle)
   ret = 'AFR_CLEAR_WTOR'(trap_handle)
/* whatever else needs to be done to clean up after a shutdown */
   exit 'N'

/* branch to this point if novalue error encountered */
novalue:
   result = program_name 'Terminated for Novalue, Line' SIGL
   say result
   ret = 'AFR_ERROR'(result,0)
   ret = 'AFR_CLEAR_TRAP'(full_handle)
   ret = 'AFR_CLEAR_WTOR'(trap_handle)
/* whatever else needs to be done to clean up after a No Value shutdown */
   exit 'N'

/* branch to this point if syntax error encountered or function call in error */
syntax:
   result = program_name 'Terminated for Syntax, Line' SIGL 'Code' RC
   say result
   ret = 'AFR_ERROR'(result,0)
   ret = 'AFR_CLEAR_TRAP'(full_handle)
   ret = 'AFR_CLEAR_WTOR'(trap_handle)
/* whatever else needs to be done to clean up after a syntax shutdown */
   exit 'N'

/* this procedure initiates and monitors the results of a beeper paging call
   Inputs:
      Arg(1) Extracted WTOR number field in the form *nn where n in the set
               <0123456789>.
      Arg(2) Extracted WTOR data in the form:
               <Page type><Telephone Number> <PID> <Message>
                    Example:  'M9,395-7407 911420 This is a test'
   Outputs 'Y' for a successful Beeper page,
           'N' for an unsuccessful Beeper page.
*/

wtorproc:

/* constant and variable definitions */
retries = 3
retry_delay = 30000
beep_prefix = ''
test = 'no'

/* retrieve arguments */
temp = arg(1)
parse var temp wtor_number .
wtor_at = pos('@',wtor_number)
wtor_as = pos('*',wtor_number)
wtor_pos = wtor_at + wtor_as + 1
wtor_number = substr(wtor_number,wtor_pos)

/* strip off leading and trailing blanks from message */
msg = strip(arg(2))

/* enclose in quotes so that any embedded commas will pass through
   REXX parser */
msg1 = "'"msg"'"

/* inform anybody watching */
say program_name 'Going to beep' msg

/* retrieve the name of the program to make the paging call */
page_program = AFR_GETENV("BP_REXXCALL")

page_string = "answer = "page_program"("msg1")"

/* paging queues enabled, then send paging information to the queue */
if Queues then
   do
   /* place the message in the next position in the array */
   Message.FirstMsg = msg1
   WtorNumber.FirstMsg = wtor_number
   FirstMsg = FirstMsg + 1
   answer = 'Y'
   end
/* otherwise we do as before, by direct call to paging program */
else
   interpret page_string

if answer \= 'Y' then
   do
      say program_name 'Beep Failed:' answer
      answer = 'N'
   end

/* validate WTOR number, range 10 exp wtor_length */
do i = 1 to length(wtor_number)
   if pos(substr(wtor_number,i,1),'0123456789') = 0 then
      do
         say program_name 'Invalid WTOR Number' wtor_number
         return 'Failed'
      end
end

/* display arguments */
say program_name 'WTOR' wtor_number 'Response is' answer

/* format response */
string = '@0@RR 'wtor_number','answer' @E'
say program_name 'Responded' string

/* send response */
/* ret = 'AFR_SEND_3270'(string) */
n = SendMvs('R 'wtor_number','answer)

/* indicate success */
return 'Done'

/*
Note:  These lines of code need to be inserted in the main body of
       the program if the following capability is being incorporated.
       This code expects that the console to be accessed has already
       been made available by successfully calling AFR_SELECT().

CommandPos = 1682
if 2000 < length(afr_load(0,1999)) then
    CommandPos = 3202

XPos = CommandPos + 246

*/

/* procedure XBusy --------------------------------------------

   Purpose: to monitor the state of the keyboard locked indicator.

   This procedure takes as arguments:
      CommandPos which is the location on an AF/REMOTE MVS session
      screen where the start of the command entry line is located.

      XPos which is the location of the keyboard locked indicator

   Return 0 if keyboard not locked, 1 otherwise

 -------------------------------------------------------------- */

XBusy: procedure expose CommandPos XPos

BusyTries = 40
XState = afr_load(XPos,4)

/* while keyboard in 'X' wait */
do while pos('X',XState) > 0

    /* wait a bit */
    n = afr_suspend(250)

    /* locked then simply spin wait */
    if pos('[]',XState) > 0 then
        nop

    /* otherwise send a keyboard reset */
    else
        n = afr_send_3270('@R')

    /* count the try */
    BusyTries = BusyTries - 1

    /* quit if alread tried a lot of times */
    if BusyTries <= 0 then
        return 1

    /* check the keyboard locked indicator */
    XState = afr_load(XPos,4)
    end

return 0


/* procedure CmdBusy -----------------------------------------

   Purpose: This procedure monitors the current position of the
   MVS cursor. Depending on the position of the cursor one of
   three actions occurs:

   1.  If the cursor is above the command cursor (less than
   CommandPos then TAB are sent to the session until the cursor
   is at the command position).

   2.  If the cursor is to the right or below CommandPos then it
   waits.  This is a form of serialization lock.

   3.  If the cursor is positioned at CommandPos then the command
   line is available to accept commands.

   Returns 2 if command line not available at expiration of
                BusyTries count.
           1 if keyboard locked
           0 if command line is available.

 ------------------------------------------------------------- */
CmdBusy: procedure expose CommandPos XPos

/* check availability of command line */
if XBusy() <> 0 then
    return 1

BusyTries = 40

/* record current position of the cursor */
Cursor = afr_cursor()

/* while the cursor is not at the command line available position */
do while Cursor <> CommandPos
    /* do while cursor is not at command position but is available */
    do while Cursor < CommandPos

        /* send a back tab and then wait */
        n = afr_send_3270('@B')
        n = afr_suspend(250)

        /* and then record new cursor position */
        Cursor = afr_cursor()
        end

    /* count the number of time we've already attempted to use
       command line */
    BusyTries = BusyTries - 1
    if BusyTries <= 0 then
        return 2

    /* wait a bit and then record new position of cursor */
    n = afr_suspend(250)
    Cursor = afr_cursor()
    end

return 0

/* procedure SendMVS -----------------------------------------
   Purpose: This procedure receives as an argument the
   command string to be sent to the already selected MVS console.
   Other optional arguments can modify the operating parameter for
   time to retry and delay between retries.  It calls the function
   CmdBusy to determine availability of the command line.  If
   available it transmits the command string.  It then checks the
   position of the cursor as a mechanism to indicate whether the
   transmission is successful or not. If successful then the send
   HLLAPI code is sent, otherwise the command line is cleared
   and retransmission is attempted.

   Returns 0 if successful, 1 or 2 otherwise


 ------------------------------------------------------------- */

SendMVS: procedure expose CommandPos XPos

/* retrieve the command to send */
CommandToSend = arg(1)

/* retrieve optional retries value */
if arg() >= 2 then
    Retries = arg(2)
else
    Retries = 5

/* retrieve optional delay value */
if arg() >= 3 then
    Delay = arg(3)
else
    Delay = 1000

/* need to know length of the command string */
CommandLength = length(CommandToSend)

/* determine if command line available */
ret = CmdBusy()
if ret <> 0 then
    return ret

/* command line available, try to send the command */
do Retries
    /* send the command */
    n = afr_send_3270(CommandToSend)

    /* wait on the console a bit */
    n = afr_suspend(250)

    /* test the status on the OAI line */
    if XBusy() then
        return 1

    /* did the whole string appear on the console? */
    Length = afr_cursor() - CommandPos

    /* if length at least length of string, then probably did */
    if Length >= CommandLength then
        do
        /* emit SEND code */
        n = afr_send_3270('@E')

        /* wait for console to finish */
        if XBusy() then
            do
            n = afr_send_3270('@A@F')
            end
        else
            /* wait for console to become available */
            do 10 while afr_cursor() <> CommandPos
                n = afr_suspend(Delay)
                end
        end

    /* check to see that command has finished */
    if afr_cursor() = CommandPos then
        return 0

    /* clear command line to retry */
    n = afr_send_3270('@A@F')
    call XBusy
    end

return 0
